@@ -91,9 +91,9 @@ else
 fi
 
 if [ -z "${GECKO_INCLUDES}" -a -z "${GECKO_LIBS}" ]; then
-	if [ x`pkg-config --exists mozilla-xpcom && echo YES` = "xYES" ]; then
-		GECKO_INCLUDES="`pkg-config --cflags mozilla-xpcom` -I`pkg-config --variable=includedir mozilla-xpcom`/embed_base"
-		GECKO_LIBS="`pkg-config --libs mozilla-xpcom` `pkg-config --libs mozilla-gtkmozembed`"
+	if [ x`pkg-config --exists libxul && echo YES` = "xYES" ]; then
+		GECKO_INCLUDES="`pkg-config --cflags libxul`"
+		GECKO_LIBS="`pkg-config --libs libxul` "
 		export GECKO_INCLUDES
 		export GECKO_LIBS
 		MAKE_MOZILLA=make_mozilla
