@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
  * http://www.eclipse.org/legal/epl-v10.html
- * 
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -15,16 +15,16 @@ import java.io.IOException;
 import java.util.*;
 import javax.xml.parsers.ParserConfigurationException;
 import javax.xml.parsers.SAXParserFactory;
-import org.eclipse.core.internal.runtime.*;
 import org.eclipse.core.runtime.*;
 import org.eclipse.osgi.util.NLS;
-import org.osgi.util.tracker.ServiceTracker;
 import org.xml.sax.*;
 import org.xml.sax.helpers.DefaultHandler;
 
 public class ExtensionsParser extends DefaultHandler {
 	// Introduced for backward compatibility
 	private final static String NO_EXTENSION_MUNGING = "eclipse.noExtensionMunging"; //$NON-NLS-1$ //System property
+	private static final String VERSION_3_0 = "3.0"; //$NON-NLS-1$
+	private static final String VERSION_3_2 = "3.2"; //$NON-NLS-1$
 	private static Map extensionPointMap;
 
 	static {
@@ -73,13 +73,16 @@ public class ExtensionsParser extends DefaultHandler {
 	// A status for holding results.
 	private MultiStatus status;
 
+	// Owning extension registry
+	private ExtensionRegistry registry;
+
 	// Resource bundle used to translate the content of the plugin.xml
-	private ResourceBundle resources;
+	protected ResourceBundle resources;
 
 	// Keep track of the object encountered.
 	private RegistryObjectManager objectManager;
 
-	private Contribution namespace;
+	private Contribution contribution;
 
 	//This keeps tracks of the value of the configuration element in case the value comes in several pieces (see characters()). See as well bug 75592. 
 	private String configurationElementValue;
@@ -131,22 +134,27 @@ public class ExtensionsParser extends DefaultHandler {
 
 	private ArrayList scratchVectors[] = new ArrayList[LAST_INDEX + 1];
 
-	private String manifestType;
-
 	private Locator locator = null;
 
-	public ExtensionsParser(MultiStatus status) {
+	// Cache the behavior toggle (true: attempt to extract namespace from qualified IDs)
+	private boolean extractNamespaces = false;
+
+	public ExtensionsParser(MultiStatus status, ExtensionRegistry registry) {
 		super();
 		this.status = status;
+		this.registry = registry;
 	}
 
-	/**
-	 * @see ContentHandler#setDocumentLocator
+	/* (non-Javadoc)
+	 * @see org.xml.sax.helpers.DefaultHandler#setDocumentLocator(org.xml.sax.Locator)
 	 */
 	public void setDocumentLocator(Locator locator) {
 		this.locator = locator;
 	}
 
+	/* (non-Javadoc)
+	 * @see org.xml.sax.helpers.DefaultHandler#characters(char[], int, int)
+	 */
 	public void characters(char[] ch, int start, int length) {
 		int state = ((Integer) stateStack.peek()).intValue();
 		if (state != CONFIGURATION_ELEMENT_STATE)
@@ -168,10 +176,16 @@ public class ExtensionsParser extends DefaultHandler {
 		}
 	}
 
+	/* (non-Javadoc)
+	 * @see org.xml.sax.helpers.DefaultHandler#endDocument()
+	 */
 	public void endDocument() {
 		// do nothing
 	}
 
+	/* (non-Javadoc)
+	 * @see org.xml.sax.helpers.DefaultHandler#endElement(java.lang.String, java.lang.String, java.lang.String)
+	 */
 	public void endElement(String uri, String elementName, String qName) {
 		switch (((Integer) stateStack.peek()).intValue()) {
 			case IGNORED_ELEMENT_STATE :
@@ -179,36 +193,34 @@ public class ExtensionsParser extends DefaultHandler {
 				break;
 			case INITIAL_STATE :
 				// shouldn't get here
-				internalError(NLS.bind(Messages.parse_internalStack, elementName));
+				internalError(NLS.bind(RegistryMessages.parse_internalStack, elementName));
 				break;
 			case BUNDLE_STATE :
-				if (elementName.equals(manifestType)) {
-					stateStack.pop();
+				stateStack.pop();
 
-					ArrayList extensionPoints = scratchVectors[EXTENSION_POINT_INDEX];
-					ArrayList extensions = scratchVectors[EXTENSION_INDEX];
-					int[] namespaceChildren = new int[2 + extensionPoints.size() + extensions.size()];
-					int position = 2;
-					// Put the extension points into this namespace
-					if (extensionPoints.size() > 0) {
-						namespaceChildren[Contribution.EXTENSION_POINT] = extensionPoints.size();
-						for (Iterator iter = extensionPoints.iterator(); iter.hasNext();) {
-							namespaceChildren[position++] = ((RegistryObject) iter.next()).getObjectId();
-						}
-						extensionPoints.clear();
+				ArrayList extensionPoints = scratchVectors[EXTENSION_POINT_INDEX];
+				ArrayList extensions = scratchVectors[EXTENSION_INDEX];
+				int[] namespaceChildren = new int[2 + extensionPoints.size() + extensions.size()];
+				int position = 2;
+				// Put the extension points into this namespace
+				if (extensionPoints.size() > 0) {
+					namespaceChildren[Contribution.EXTENSION_POINT] = extensionPoints.size();
+					for (Iterator iter = extensionPoints.iterator(); iter.hasNext();) {
+						namespaceChildren[position++] = ((RegistryObject) iter.next()).getObjectId();
 					}
-
-					// Put the extensions into this namespace too
-					if (extensions.size() > 0) {
-						Extension[] renamedExtensions = fixRenamedExtensionPoints((Extension[]) extensions.toArray(new Extension[extensions.size()]));
-						namespaceChildren[Contribution.EXTENSION] = renamedExtensions.length;
-						for (int i = 0; i < renamedExtensions.length; i++) {
-							namespaceChildren[position++] = renamedExtensions[i].getObjectId();
-						}
-						extensions.clear();
-					}
-					namespace.setRawChildren(namespaceChildren);
+					extensionPoints.clear();
 				}
+
+				// Put the extensions into this namespace too
+				if (extensions.size() > 0) {
+					Extension[] renamedExtensions = fixRenamedExtensionPoints((Extension[]) extensions.toArray(new Extension[extensions.size()]));
+					namespaceChildren[Contribution.EXTENSION] = renamedExtensions.length;
+					for (int i = 0; i < renamedExtensions.length; i++) {
+						namespaceChildren[position++] = renamedExtensions[i].getObjectId();
+					}
+					extensions.clear();
+				}
+				contribution.setRawChildren(namespaceChildren);
 				break;
 			case BUNDLE_EXTENSION_POINT_STATE :
 				if (elementName.equals(EXTENSION_POINT)) {
@@ -220,7 +232,9 @@ public class ExtensionsParser extends DefaultHandler {
 					stateStack.pop();
 					// Finish up extension object
 					Extension currentExtension = (Extension) objectStack.pop();
-					currentExtension.setNamespace(namespace.getNamespace());
+					if (currentExtension.getNamespaceIdentifier() == null)
+						currentExtension.setNamespaceIdentifier(contribution.getDefaultNamespace());
+					currentExtension.setContributorId(contribution.getContributorId());
 					scratchVectors[EXTENSION_INDEX].add(currentExtension);
 				}
 				break;
@@ -252,10 +266,16 @@ public class ExtensionsParser extends DefaultHandler {
 		}
 	}
 
+	/* (non-Javadoc)
+	 * @see org.xml.sax.helpers.DefaultHandler#error(org.xml.sax.SAXParseException)
+	 */
 	public void error(SAXParseException ex) {
 		logStatus(ex);
 	}
 
+	/* (non-Javadoc)
+	 * @see org.xml.sax.helpers.DefaultHandler#fatalError(org.xml.sax.SAXParseException)
+	 */
 	public void fatalError(SAXParseException ex) throws SAXException {
 		logStatus(ex);
 		throw ex;
@@ -280,8 +300,8 @@ public class ExtensionsParser extends DefaultHandler {
 		configurationElementValue = null;
 
 		// create a new Configuration Element and push it onto the object stack
-		ConfigurationElement currentConfigurationElement = new ConfigurationElement();
-		currentConfigurationElement.setContributingBundle(namespace.getNamespaceBundle());
+		ConfigurationElement currentConfigurationElement = registry.getElementFactory().createConfigurationElement(contribution.shouldPersist());
+		currentConfigurationElement.setContributorId(contribution.getContributorId());
 		objectStack.push(currentConfigurationElement);
 		currentConfigurationElement.setName(elementName);
 
@@ -294,16 +314,11 @@ public class ExtensionsParser extends DefaultHandler {
 	}
 
 	private void handleInitialState(String elementName, Attributes attributes) {
-		if (!elementName.equals(manifestType)) {
-			stateStack.push(new Integer(IGNORED_ELEMENT_STATE));
-			internalError(NLS.bind(Messages.parse_unknownTopElement, elementName));
-			return;
-		}
 		// new manifests should have the plugin (or fragment) element empty
 		// in compatibility mode, any extraneous elements will be silently ignored
 		compatibilityMode = attributes.getLength() > 0;
 		stateStack.push(new Integer(BUNDLE_STATE));
-		objectStack.push(namespace);
+		objectStack.push(contribution);
 	}
 
 	private void handleBundleState(String elementName, Attributes attributes) {
@@ -322,7 +337,7 @@ public class ExtensionsParser extends DefaultHandler {
 		// Set the state to indicate that this element will be ignored
 		stateStack.push(new Integer(IGNORED_ELEMENT_STATE));
 		if (!compatibilityMode)
-			unknownElement(manifestType, elementName);
+			unknownElement(PLUGIN, elementName);
 	}
 
 	private void logStatus(SAXParseException ex) {
@@ -336,32 +351,25 @@ public class ExtensionsParser extends DefaultHandler {
 
 		String msg;
 		if (name.equals("")) //$NON-NLS-1$
-			msg = NLS.bind(Messages.parse_error, ex.getMessage());
+			msg = NLS.bind(RegistryMessages.parse_error, ex.getMessage());
 		else
-			msg = NLS.bind(Messages.parse_errorNameLineColumn, (new Object[] {name, Integer.toString(ex.getLineNumber()), Integer.toString(ex.getColumnNumber()), ex.getMessage()}));
-		error(new Status(IStatus.WARNING, Platform.PI_RUNTIME, PARSE_PROBLEM, msg, ex));
+			msg = NLS.bind(RegistryMessages.parse_errorNameLineColumn, (new Object[] {name, Integer.toString(ex.getLineNumber()), Integer.toString(ex.getColumnNumber()), ex.getMessage()}));
+		error(new Status(IStatus.WARNING, RegistryMessages.OWNER_NAME, PARSE_PROBLEM, msg, ex));
 	}
 
-	public Contribution parseManifest(ServiceTracker factoryTracker, InputSource in, String manifestKind, String manifestName, RegistryObjectManager registryObjects, Contribution currentNamespace, ResourceBundle bundle) throws ParserConfigurationException, SAXException, IOException {
+	public Contribution parseManifest(SAXParserFactory factory, InputSource in, String manifestName, RegistryObjectManager registryObjects, Contribution currentNamespace, ResourceBundle bundle) throws ParserConfigurationException, SAXException, IOException {
 		long start = 0;
 		this.resources = bundle;
 		this.objectManager = registryObjects;
 		//initialize the parser with this object
-		this.namespace = currentNamespace;
-		if (InternalPlatform.DEBUG)
+		this.contribution = currentNamespace;
+		if (registry.debug())
 			start = System.currentTimeMillis();
 
-		SAXParserFactory factory = (SAXParserFactory) factoryTracker.getService();
-
 		if (factory == null)
-			throw new SAXException(Messages.parse_xmlParserNotAvailable);
+			throw new SAXException(RegistryMessages.parse_xmlParserNotAvailable);
 
 		try {
-			if (manifestKind == null)
-				throw new NullPointerException();
-			if (!(manifestKind.equals(PLUGIN) || manifestKind.equals(FRAGMENT)))
-				throw new IllegalArgumentException("Invalid manifest type: " + manifestType); //$NON-NLS-1$
-			this.manifestType = manifestKind;
 			locationName = in.getSystemId();
 			if (locationName == null)
 				locationName = manifestName;
@@ -375,9 +383,9 @@ public class ExtensionsParser extends DefaultHandler {
 			factory.newSAXParser().parse(in, this);
 			return (Contribution) objectStack.pop();
 		} finally {
-			if (InternalPlatform.DEBUG) {
+			if (registry.debug()) {
 				cumulativeTime = cumulativeTime + (System.currentTimeMillis() - start);
-				InternalPlatform.getDefault().setOption("org.eclipse.core.runtime/registry/parsing/timing/value", Long.toString(cumulativeTime)); //$NON-NLS-1$
+				System.out.println("Cumulative parse time so far : " + cumulativeTime); //$NON-NLS-1$
 			}
 		}
 	}
@@ -401,7 +409,7 @@ public class ExtensionsParser extends DefaultHandler {
 	}
 
 	private void parseExtensionAttributes(Attributes attributes) {
-		Extension currentExtension = new Extension();
+		Extension currentExtension = registry.getElementFactory().createExtension(contribution.shouldPersist());
 		objectStack.push(currentExtension);
 
 		// Process Attributes
@@ -412,19 +420,30 @@ public class ExtensionsParser extends DefaultHandler {
 
 			if (attrName.equals(EXTENSION_NAME))
 				currentExtension.setLabel(translate(attrValue));
-			else if (attrName.equals(EXTENSION_ID))
-				currentExtension.setSimpleIdentifier(attrValue);
-			else if (attrName.equals(EXTENSION_TARGET)) {
+			else if (attrName.equals(EXTENSION_ID)) {
+				String simpleId;
+				String namespaceName;
+				int simpleIdStart = attrValue.lastIndexOf('.');
+				if ((simpleIdStart != -1) && extractNamespaces) {
+					simpleId = attrValue.substring(simpleIdStart + 1);
+					namespaceName = attrValue.substring(0, simpleIdStart);
+				} else {
+					simpleId = attrValue;
+					namespaceName = contribution.getDefaultNamespace();
+				}
+				currentExtension.setSimpleIdentifier(simpleId);
+				currentExtension.setNamespaceIdentifier(namespaceName);
+			} else if (attrName.equals(EXTENSION_TARGET)) {
 				// check if point is specified as a simple or qualified name
 				String targetName;
 				if (attrValue.lastIndexOf('.') == -1) {
-					String baseId = namespace.getNamespace();
-					targetName = baseId + "." + attrValue; //$NON-NLS-1$
+					String baseId = contribution.getDefaultNamespace();
+					targetName = baseId + '.' + attrValue;
 				} else
 					targetName = attrValue;
 				currentExtension.setExtensionPointIdentifier(targetName);
 			} else
-				unknownAttribute(EXTENSION, attrName); //$NON-NLS-1$
+				unknownAttribute(EXTENSION, attrName);
 		}
 		if (currentExtension.getExtensionPointIdentifier() == null) {
 			missingAttribute(EXTENSION_TARGET, EXTENSION);
@@ -440,27 +459,27 @@ public class ExtensionsParser extends DefaultHandler {
 	//todo: Are all three methods needed??
 	private void missingAttribute(String attribute, String element) {
 		if (locator == null)
-			internalError(NLS.bind(Messages.parse_missingAttribute, attribute, element));
+			internalError(NLS.bind(RegistryMessages.parse_missingAttribute, attribute, element));
 		else
-			internalError(NLS.bind(Messages.parse_missingAttributeLine, (new String[] {attribute, element, Integer.toString(locator.getLineNumber())})));
+			internalError(NLS.bind(RegistryMessages.parse_missingAttributeLine, (new String[] {attribute, element, Integer.toString(locator.getLineNumber())})));
 	}
 
 	private void unknownAttribute(String attribute, String element) {
 		if (locator == null)
-			internalError(NLS.bind(Messages.parse_unknownAttribute, attribute, element));
+			internalError(NLS.bind(RegistryMessages.parse_unknownAttribute, attribute, element));
 		else
-			internalError(NLS.bind(Messages.parse_unknownAttributeLine, (new String[] {attribute, element, Integer.toString(locator.getLineNumber())})));
+			internalError(NLS.bind(RegistryMessages.parse_unknownAttributeLine, (new String[] {attribute, element, Integer.toString(locator.getLineNumber())})));
 	}
 
 	private void unknownElement(String element, String parent) {
 		if (locator == null)
-			internalError(NLS.bind(Messages.parse_unknownElement, parent, element));
+			internalError(NLS.bind(RegistryMessages.parse_unknownElement, parent, element));
 		else
-			internalError(NLS.bind(Messages.parse_unknownElementLine, (new String[] {parent, element, Integer.toString(locator.getLineNumber())})));
+			internalError(NLS.bind(RegistryMessages.parse_unknownElementLine, (new String[] {parent, element, Integer.toString(locator.getLineNumber())})));
 	}
 
 	private void parseExtensionPointAttributes(Attributes attributes) {
-		ExtensionPoint currentExtPoint = new ExtensionPoint();
+		ExtensionPoint currentExtPoint = registry.getElementFactory().createExtensionPoint(contribution.shouldPersist());
 
 		// Process Attributes
 		int len = (attributes != null) ? attributes.getLength() : 0;
@@ -471,11 +490,23 @@ public class ExtensionsParser extends DefaultHandler {
 			if (attrName.equals(EXTENSION_POINT_NAME))
 				currentExtPoint.setLabel(translate(attrValue));
 			else if (attrName.equals(EXTENSION_POINT_ID)) {
-				currentExtPoint.setUniqueIdentifier(namespace.getNamespace() + '.' + attrValue);
+				String uniqueId;
+				String namespaceName;
+				int simpleIdStart = attrValue.lastIndexOf('.');
+				if (simpleIdStart != -1 && extractNamespaces) {
+					namespaceName = attrValue.substring(0, simpleIdStart);
+					uniqueId = attrValue;
+				} else {
+					namespaceName = contribution.getDefaultNamespace();
+					uniqueId = namespaceName + '.' + attrValue;
+				}
+				currentExtPoint.setUniqueIdentifier(uniqueId);
+				currentExtPoint.setNamespace(namespaceName);
+
 			} else if (attrName.equals(EXTENSION_POINT_SCHEMA))
 				currentExtPoint.setSchema(attrValue);
 			else
-				unknownAttribute(EXTENSION_POINT, attrName); //$NON-NLS-1$
+				unknownAttribute(EXTENSION_POINT, attrName);
 		}
 		if (currentExtPoint.getSimpleIdentifier() == null || currentExtPoint.getLabel() == null) {
 			String attribute = currentExtPoint.getSimpleIdentifier() == null ? EXTENSION_POINT_ID : EXTENSION_POINT_NAME;
@@ -486,13 +517,17 @@ public class ExtensionsParser extends DefaultHandler {
 		}
 
 		objectManager.addExtensionPoint(currentExtPoint, true);
-		currentExtPoint.setNamespace(namespace.getNamespace());
-		currentExtPoint.setBundleId(namespace.getNamespaceBundle().getBundleId());
+		if (currentExtPoint.getNamespace() == null)
+			currentExtPoint.setNamespace(contribution.getDefaultNamespace());
+		currentExtPoint.setContributorId(contribution.getContributorId());
 
 		// Now populate the the vector just below us on the objectStack with this extension point
 		scratchVectors[EXTENSION_POINT_INDEX].add(currentExtPoint);
 	}
 
+	/* (non-Javadoc)
+	 * @see org.xml.sax.helpers.DefaultHandler#startDocument()
+	 */
 	public void startDocument() {
 		stateStack.push(new Integer(INITIAL_STATE));
 		for (int i = 0; i <= LAST_INDEX; i++) {
@@ -500,6 +535,9 @@ public class ExtensionsParser extends DefaultHandler {
 		}
 	}
 
+	/* (non-Javadoc)
+	 * @see org.xml.sax.helpers.DefaultHandler#startElement(java.lang.String, java.lang.String, java.lang.String, org.xml.sax.Attributes)
+	 */
 	public void startElement(String uri, String elementName, String qName, Attributes attributes) {
 		switch (((Integer) stateStack.peek()).intValue()) {
 			case INITIAL_STATE :
@@ -518,16 +556,19 @@ public class ExtensionsParser extends DefaultHandler {
 			default :
 				stateStack.push(new Integer(IGNORED_ELEMENT_STATE));
 				if (!compatibilityMode)
-					internalError(NLS.bind(Messages.parse_unknownTopElement, elementName));
+					internalError(NLS.bind(RegistryMessages.parse_unknownTopElement, elementName));
 		}
 	}
 
+	/* (non-Javadoc)
+	 * @see org.xml.sax.helpers.DefaultHandler#warning(org.xml.sax.SAXParseException)
+	 */
 	public void warning(SAXParseException ex) {
 		logStatus(ex);
 	}
 
 	private void internalError(String message) {
-		error(new Status(IStatus.WARNING, Platform.PI_RUNTIME, PARSE_PROBLEM, message, null));
+		error(new Status(IStatus.WARNING, RegistryMessages.OWNER_NAME, PARSE_PROBLEM, message, null));
 	}
 
 	/* (non-Javadoc)
@@ -539,11 +580,11 @@ public class ExtensionsParser extends DefaultHandler {
 		// the start of the manifest file is used to indicate the plug-in manifest
 		// schema version in effect. Pre-3.0 (i.e., 2.1) plug-in manifest files do not
 		// have one of these, and this is how we can distinguish the manifest of a
-		// pre-3.0 plug-in from a post-3.0 one (for compatibility tranformations).
+		// pre-3.0 plug-in from a post-3.0 one (for compatibility transformations).
 		if (target.equalsIgnoreCase("eclipse")) { //$NON-NLS-1$
 			// just the presence of this processing instruction indicates that this
 			// plug-in is at least 3.0
-			schemaVersion = "3.0"; //$NON-NLS-1$
+			schemaVersion = VERSION_3_0;
 			StringTokenizer tokenizer = new StringTokenizer(data, "=\""); //$NON-NLS-1$
 			while (tokenizer.hasMoreTokens()) {
 				String token = tokenizer.nextToken();
@@ -555,6 +596,7 @@ public class ExtensionsParser extends DefaultHandler {
 					break;
 				}
 			}
+			initializeExtractNamespace();
 		}
 	}
 
@@ -568,8 +610,8 @@ public class ExtensionsParser extends DefaultHandler {
 		status.add(error);
 	}
 
-	private String translate(String key) {
-		return ResourceTranslator.getResourceString(null, key, resources);
+	protected String translate(String key) {
+		return registry.translate(key, resources);
 	}
 
 	/**
@@ -577,7 +619,7 @@ public class ExtensionsParser extends DefaultHandler {
 	 * for extension points that were renamed between release 2.1 and 3.0.
 	 */
 	private Extension[] fixRenamedExtensionPoints(Extension[] extensions) {
-		if (extensions == null || (schemaVersion != null && schemaVersion.equals("3.0")) || System.getProperties().get(NO_EXTENSION_MUNGING) != null) //$NON-NLS-1$
+		if (extensions == null || versionAtLeast(VERSION_3_0) || RegistryProperties.getProperty(NO_EXTENSION_MUNGING) != null)
 			return extensions;
 		for (int i = 0; i < extensions.length; i++) {
 			Extension extension = extensions[i];
@@ -589,4 +631,34 @@ public class ExtensionsParser extends DefaultHandler {
 		}
 		return extensions;
 	}
+
+	/**
+	 * To preserve backward compatibility, we will only attempt to extract namespace form the name 
+	 * if Eclipse version specified in the plugin.xml (<?eclipse version="3.2"?>) is at least 3.2.
+	 */
+	private void initializeExtractNamespace() {
+		extractNamespaces = new Boolean(versionAtLeast(VERSION_3_2)).booleanValue();
+	}
+
+	/**
+	 * Makes sense only for plugin.xml versions >= 3.0 (Eclipse version was introduced in 3.0).
+	 * Assumes that version is stored as "X1.X2.....XN" where X1 is a major version; X2 is a minor version
+	 * and so on.
+	 */
+	private boolean versionAtLeast(String testVersion) {
+		if (schemaVersion == null)
+			return false;
+
+		StringTokenizer testVersionTokenizer = new StringTokenizer(testVersion, "."); //$NON-NLS-1$
+		StringTokenizer schemaVersionTokenizer = new StringTokenizer(schemaVersion, "."); //$NON-NLS-1$
+		while (testVersionTokenizer.hasMoreTokens() && schemaVersionTokenizer.hasMoreTokens()) {
+			try {
+				if (Integer.parseInt(schemaVersionTokenizer.nextToken()) < Integer.parseInt(testVersionTokenizer.nextToken()))
+					return false;
+			} catch (NumberFormatException e) {
+				return false;
+			}
+		}
+		return true;
+	}
 }
