@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2004 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -13,12 +13,11 @@ package org.eclipse.core.runtime.internal.adaptor;
 import java.io.InputStream;
 import java.util.*;
 import javax.xml.parsers.SAXParserFactory;
-import org.eclipse.core.runtime.adaptor.EclipseAdaptor;
-import org.eclipse.core.runtime.adaptor.EclipseAdaptorMsg;
 import org.eclipse.osgi.framework.adaptor.FrameworkAdaptor;
 import org.eclipse.osgi.framework.log.FrameworkLogEntry;
 import org.eclipse.osgi.util.NLS;
 import org.osgi.framework.BundleContext;
+import org.osgi.framework.Version;
 import org.osgi.util.tracker.ServiceTracker;
 import org.xml.sax.*;
 import org.xml.sax.helpers.DefaultHandler;
@@ -31,7 +30,9 @@ public class PluginParser extends DefaultHandler implements IModel {
 
 	private PluginInfo manifestInfo = new PluginInfo();
 	private BundleContext context;
-	private String target; // The targeted platform for the given manifest
+	private FrameworkAdaptor adaptor;
+	private Version target; // The targeted platform for the given manifest
+	private static final Version TARGET21 = new Version(2, 1, 0);
 
 	public class PluginInfo implements IPluginInfo {
 		private String schemaVersion;
@@ -56,7 +57,7 @@ public class PluginParser extends DefaultHandler implements IModel {
 		private String pluginName;
 		private boolean singleton;
 		private boolean fragment;
-		private static final String TARGET21 = "2.1"; //$NON-NLS-1$
+		private final static String TARGET21_STRING = "2.1"; //$NON-NLS-1$
 		private boolean hasExtensionExtensionPoints = false;
 
 		public boolean isFragment() {
@@ -78,7 +79,7 @@ public class PluginParser extends DefaultHandler implements IModel {
 				requiresExpanded = true;
 				if (requires == null) {
 					requires = new ArrayList(1);
-					requires.add(new Prerequisite(PluginConverterImpl.PI_RUNTIME, TARGET21, false, false, IModel.PLUGIN_REQUIRES_MATCH_GREATER_OR_EQUAL));
+					requires.add(new Prerequisite(PluginConverterImpl.PI_RUNTIME, TARGET21_STRING, false, false, IModel.PLUGIN_REQUIRES_MATCH_GREATER_OR_EQUAL));
 					requires.add(new Prerequisite(PluginConverterImpl.PI_RUNTIME_COMPATIBILITY, null, false, false, null));
 				} else {
 					//Add elements on the requirement list of ui and help.
@@ -103,7 +104,7 @@ public class PluginParser extends DefaultHandler implements IModel {
 					//This is used to recognize the version for which the given plugin was initially targeted.
 					Prerequisite runtimePrereq = new Prerequisite(PluginConverterImpl.PI_RUNTIME, null, false, false, null);
 					requires.remove(runtimePrereq);
-					requires.add(new Prerequisite(PluginConverterImpl.PI_RUNTIME, TARGET21, false, false, IModel.PLUGIN_REQUIRES_MATCH_GREATER_OR_EQUAL));
+					requires.add(new Prerequisite(PluginConverterImpl.PI_RUNTIME, TARGET21_STRING, false, false, IModel.PLUGIN_REQUIRES_MATCH_GREATER_OR_EQUAL));
 				}
 			}
 			if (requires == null)
@@ -205,9 +206,10 @@ public class PluginParser extends DefaultHandler implements IModel {
 	private static final int PLUGIN_REQUIRES_IMPORT_STATE = 9;
 	private static final int FRAGMENT_STATE = 11;
 
-	public PluginParser(BundleContext context, String target) {
+	public PluginParser(FrameworkAdaptor adaptor, BundleContext context, Version target) {
 		super();
 		this.context = context;
+		this.adaptor = adaptor;
 		this.target = target;
 	}
 
@@ -446,15 +448,15 @@ public class PluginParser extends DefaultHandler implements IModel {
 		else
 			msg = NLS.bind(EclipseAdaptorMsg.parse_errorNameLineColumn, new String[] {name, Integer.toString(ex.getLineNumber()), Integer.toString(ex.getColumnNumber()), ex.getMessage()});
 
-		FrameworkLogEntry entry = new FrameworkLogEntry(FrameworkAdaptor.FRAMEWORK_SYMBOLICNAME, msg, 0, ex, null);
-		EclipseAdaptor.getDefault().getFrameworkLog().log(entry);
+		FrameworkLogEntry entry = new FrameworkLogEntry(FrameworkAdaptor.FRAMEWORK_SYMBOLICNAME, FrameworkLogEntry.ERROR, 0, msg, 0, ex, null);
+		adaptor.getFrameworkLog().log(entry);
 	}
 
 	synchronized public PluginInfo parsePlugin(InputStream in) throws Exception {
 		SAXParserFactory factory = acquireXMLParsing(context);
 		if (factory == null) {
-			FrameworkLogEntry entry = new FrameworkLogEntry(FrameworkAdaptor.FRAMEWORK_SYMBOLICNAME, EclipseAdaptorMsg.ECLIPSE_CONVERTER_NO_SAX_FACTORY, 0, null, null);
-			EclipseAdaptor.getDefault().getFrameworkLog().log(entry);
+			FrameworkLogEntry entry = new FrameworkLogEntry(FrameworkAdaptor.FRAMEWORK_SYMBOLICNAME, FrameworkLogEntry.ERROR, 0, EclipseAdaptorMsg.ECLIPSE_CONVERTER_NO_SAX_FACTORY, 0, null, null);
+			adaptor.getFrameworkLog().log(entry);
 			return null;
 		}
 
@@ -475,7 +477,11 @@ public class PluginParser extends DefaultHandler implements IModel {
 			xmlTracker = new ServiceTracker(context, "javax.xml.parsers.SAXParserFactory", null); //$NON-NLS-1$
 			xmlTracker.open();
 		}
-		return (SAXParserFactory) xmlTracker.getService();
+		SAXParserFactory result = (SAXParserFactory) xmlTracker.getService();
+		if (result != null)
+			return result;
+		// backup to using jaxp to create a new instance
+		return SAXParserFactory.newInstance();
 	}
 
 	public static void releaseXMLParsing() {
@@ -591,7 +597,7 @@ public class PluginParser extends DefaultHandler implements IModel {
 		String plugin = attributes.getValue("", PLUGIN_REQUIRES_PLUGIN); //$NON-NLS-1$ 
 		if (plugin == null)
 			return;
-		if (plugin.equals(PluginConverterImpl.PI_BOOT)) //$NON-NLS-1$//$NON-NLS-2$
+		if (plugin.equals(PluginConverterImpl.PI_BOOT)) 
 			return;
 		if (plugin.equals(PluginConverterImpl.PI_RUNTIME_COMPATIBILITY))
 			manifestInfo.compatibilityFound = true;
@@ -663,9 +669,9 @@ public class PluginParser extends DefaultHandler implements IModel {
 
 	private void internalError(String elementName) {
 		FrameworkLogEntry error;
-		String message = NLS.bind(EclipseAdaptorMsg.ECLIPSE_CONVERTER_PARSE_UNKNOWNTOP_ELEMENT, elementName); //$NON-NLS-1$
-		error = new FrameworkLogEntry(FrameworkAdaptor.FRAMEWORK_SYMBOLICNAME, (manifestInfo.pluginId == null ? message : "Plug-in : " + manifestInfo.pluginId + ", " + message), 0, null, null); //$NON-NLS-1$ //$NON-NLS-2$
-		EclipseAdaptor.getDefault().getFrameworkLog().log(error);
+		String message = NLS.bind(EclipseAdaptorMsg.ECLIPSE_CONVERTER_PARSE_UNKNOWNTOP_ELEMENT, elementName); 
+		error = new FrameworkLogEntry(FrameworkAdaptor.FRAMEWORK_SYMBOLICNAME, FrameworkLogEntry.ERROR, 0, (manifestInfo.pluginId == null ? message : "Plug-in : " + manifestInfo.pluginId + ", " + message), 0, null, null); //$NON-NLS-1$ //$NON-NLS-2$
+		adaptor.getFrameworkLog().log(error);
 	}
 
 	public void processingInstruction(String target, String data) throws SAXException {
