@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
  * http://www.eclipse.org/legal/epl-v10.html
- * 
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *     Tim Tromey - update method signature syntax (bug 31507)
@@ -18,7 +18,7 @@ import java.io.DataOutputStream;
 import java.io.IOException;
 import java.lang.reflect.Field;
 import java.lang.reflect.Modifier;
-import java.text.MessageFormat;
+import com.ibm.icu.text.MessageFormat;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Collections;
@@ -143,7 +143,7 @@ public class MethodImpl extends TypeComponentImpl implements Method, Locatable {
 		}
 		if (fCodeIndexToLine != null) {
 			if (fCodeIndexToLine.isEmpty()) {
-				throw new AbsentInformationException(JDIMessages.MethodImpl_Got_empty_line_number_table_for_this_method_1); //$NON-NLS-1$
+				throw new AbsentInformationException(JDIMessages.MethodImpl_Got_empty_line_number_table_for_this_method_1); 
 			}
 			return;
 		}
@@ -157,9 +157,9 @@ public class MethodImpl extends TypeComponentImpl implements Method, Locatable {
 			JdwpReplyPacket replyPacket = requestVM(JdwpCommandPacket.M_LINE_TABLE, outBytes);
 			switch (replyPacket.errorCode()) {
 				case JdwpReplyPacket.ABSENT_INFORMATION:
-					throw new AbsentInformationException(JDIMessages.MethodImpl_No_line_number_information_available_2); //$NON-NLS-1$
+					throw new AbsentInformationException(JDIMessages.MethodImpl_No_line_number_information_available_2); 
 				case JdwpReplyPacket.NATIVE_METHOD:
-					throw new AbsentInformationException(JDIMessages.MethodImpl_No_line_number_information_available_2); //$NON-NLS-1$
+					throw new AbsentInformationException(JDIMessages.MethodImpl_No_line_number_information_available_2); 
 			}
 			defaultReplyErrorHandler(replyPacket.errorCode());
 			
@@ -170,7 +170,7 @@ public class MethodImpl extends TypeComponentImpl implements Method, Locatable {
 			fCodeIndexToLine = new HashMap();
 			fLineToCodeIndexes = new HashMap();
 			if (nrOfElements == 0) {
-				throw new AbsentInformationException(JDIMessages.MethodImpl_Got_empty_line_number_table_for_this_method_3); //$NON-NLS-1$
+				throw new AbsentInformationException(JDIMessages.MethodImpl_Got_empty_line_number_table_for_this_method_3); 
 			}
 			fCodeIndexTable= new long[nrOfElements];
 			fJavaStratumLineNumberTable= new int[nrOfElements];
@@ -211,7 +211,7 @@ public class MethodImpl extends TypeComponentImpl implements Method, Locatable {
 		}
 		getLineTable();
 		if (lineCodeIndex > fHighestValidCodeIndex) {
-			throw new InvalidCodeIndexException (JDIMessages.MethodImpl_Invalid_code_index_of_a_location_given_4); //$NON-NLS-1$
+			throw new InvalidCodeIndexException (JDIMessages.MethodImpl_Invalid_code_index_of_a_location_given_4); 
 		}
 
 		Long lineCodeIndexObj;
@@ -233,7 +233,7 @@ public class MethodImpl extends TypeComponentImpl implements Method, Locatable {
 					return lineNrObj.intValue();
 				}
 			}
-			throw new InvalidCodeIndexException (JDIMessages.MethodImpl_Invalid_code_index_of_a_location_given_4); //$NON-NLS-1$
+			throw new InvalidCodeIndexException (JDIMessages.MethodImpl_Invalid_code_index_of_a_location_given_4); 
 		}
 		return lineNrObj.intValue();
 	}
@@ -251,7 +251,7 @@ public class MethodImpl extends TypeComponentImpl implements Method, Locatable {
 	 */
 	public List arguments() throws AbsentInformationException {
 		if (isNative() || isAbstract()) {
-			throw new AbsentInformationException(JDIMessages.MethodImpl_No_local_variable_information_available_9); //$NON-NLS-1$
+			throw new AbsentInformationException(JDIMessages.MethodImpl_No_local_variable_information_available_9); 
 		}
 		if (fArguments != null) {
 			return fArguments;
@@ -371,7 +371,7 @@ public class MethodImpl extends TypeComponentImpl implements Method, Locatable {
 	 */
 	public int compareTo(Object object) {
 		if (object == null || !object.getClass().equals(this.getClass()))
-			throw new ClassCastException(JDIMessages.MethodImpl_Can__t_compare_method_to_given_object_6); //$NON-NLS-1$
+			throw new ClassCastException(JDIMessages.MethodImpl_Can__t_compare_method_to_given_object_6); 
 		
 		// See if declaring types are the same, if not return comparison between declaring types.
 		Method type2 = (Method)object;
@@ -437,7 +437,7 @@ public class MethodImpl extends TypeComponentImpl implements Method, Locatable {
 		try {
 			Integer lineNrInt = (Integer)javaStratumCodeIndexToLine().get(new Long(index));
 			if (lineNrInt == null) {
-				throw new InvalidCodeIndexException(MessageFormat.format(JDIMessages.MethodImpl_No_valid_location_at_the_specified_code_index__0__2, new Object[]{Long.toString(index)})); //$NON-NLS-1$
+				throw new InvalidCodeIndexException(MessageFormat.format(JDIMessages.MethodImpl_No_valid_location_at_the_specified_code_index__0__2, new Object[]{Long.toString(index)})); 
 			}
 		} catch (AbsentInformationException e ) {
 		}
@@ -478,7 +478,7 @@ public class MethodImpl extends TypeComponentImpl implements Method, Locatable {
 	 */
 	public List variables() throws AbsentInformationException {
 		if (isNative() || isAbstract()) {
-			throw new AbsentInformationException(JDIMessages.MethodImpl_No_local_variable_information_available_9); //$NON-NLS-1$
+			throw new AbsentInformationException(JDIMessages.MethodImpl_No_local_variable_information_available_9); 
 		}
 		
 		if (fVariables != null) {
@@ -575,7 +575,7 @@ public class MethodImpl extends TypeComponentImpl implements Method, Locatable {
 			}
 			return fVariables;
 		}
-		throw new AbsentInformationException(JDIMessages.MethodImpl_No_local_variable_information_available_9); //$NON-NLS-1$
+		throw new AbsentInformationException(JDIMessages.MethodImpl_No_local_variable_information_available_9); 
 
 	}
 
@@ -664,7 +664,7 @@ public class MethodImpl extends TypeComponentImpl implements Method, Locatable {
 		// The method must be part of a known reference type.
 		MethodImpl method = referenceType.findMethod(ID);
 		if (method == null) {
-			throw new InternalError(JDIMessages.MethodImpl_Got_MethodID_of_ReferenceType_that_is_not_a_member_of_the_ReferenceType_10); //$NON-NLS-1$
+			throw new InternalError(JDIMessages.MethodImpl_Got_MethodID_of_ReferenceType_that_is_not_a_member_of_the_ReferenceType_10); 
 		}
 		return method;
 	}
