@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -22,8 +22,8 @@ import org.eclipse.jdt.core.IJavaElement;
 import org.eclipse.jdt.core.IJavaProject;
 import org.eclipse.jdt.core.JavaModelException;
 import org.eclipse.jdt.core.WorkingCopyOwner;
+import org.eclipse.jdt.core.compiler.CategorizedProblem;
 import org.eclipse.jdt.core.compiler.CharOperation;
-import org.eclipse.jdt.core.compiler.IProblem;
 import org.eclipse.jdt.internal.compiler.CompilationResult;
 import org.eclipse.jdt.internal.compiler.Compiler;
 import org.eclipse.jdt.internal.compiler.DefaultErrorHandlingPolicies;
@@ -37,7 +37,7 @@ import org.eclipse.jdt.internal.compiler.env.INameEnvironment;
 import org.eclipse.jdt.internal.compiler.env.ISourceType;
 import org.eclipse.jdt.internal.compiler.impl.CompilerOptions;
 import org.eclipse.jdt.internal.compiler.lookup.Binding;
-import org.eclipse.jdt.internal.compiler.lookup.CompilerModifiers;
+import org.eclipse.jdt.internal.compiler.lookup.ExtraCompilerModifiers;
 import org.eclipse.jdt.internal.compiler.lookup.PackageBinding;
 import org.eclipse.jdt.internal.compiler.parser.Parser;
 import org.eclipse.jdt.internal.compiler.problem.AbortCompilation;
@@ -109,7 +109,7 @@ class CompilationUnitResolver extends Compiler {
 	 *      in UI when compiling interactively.
 	 *      @see org.eclipse.jdt.internal.compiler.DefaultErrorHandlingPolicies
 	 * 
-	 *	@param settings The settings to use for the resolution.
+	 *	@param compilerOptions The compiler options to use for the resolution.
 	 *      
 	 *  @param requestor org.eclipse.jdt.internal.compiler.api.ICompilerRequestor
 	 *      Component which will receive and persist all compilation results and is intended
@@ -127,12 +127,12 @@ class CompilationUnitResolver extends Compiler {
 	public CompilationUnitResolver(
 		INameEnvironment environment,
 		IErrorHandlingPolicy policy,
-		Map settings,
+		CompilerOptions compilerOptions,
 		ICompilerRequestor requestor,
 		IProblemFactory problemFactory,
 		IProgressMonitor monitor) {
 
-		super(environment, policy, settings, requestor, problemFactory, false);
+		super(environment, policy, compilerOptions, requestor, problemFactory);
 		this.hasCompilationAborted = false;
 		this.monitor =monitor;
 	}
@@ -168,7 +168,7 @@ class CompilationUnitResolver extends Compiler {
 				new CompilationResult(sourceUnit, index++, maxUnits, this.options.maxProblemsPerUnit);
 			try {
 				if (options.verbose) {
-					System.out.println(
+					this.out.println(
 						Messages.bind(Messages.compilation_request,
 						new String[] {
 							String.valueOf(index++ + 1),
@@ -197,7 +197,9 @@ class CompilationUnitResolver extends Compiler {
 		for (int i = 0; i < keyLength; i++) {
 			BindingKeyResolver resolver = new BindingKeyResolver(bindingKeys[i], this, this.lookupEnvironment);
 			resolver.parse(true/*pause after fully qualified name*/);
-			CompilationUnitDeclaration parsedUnit = resolver.getCompilationUnitDeclaration();
+			// If it doesn't have a type name, then it is either an array type, package or base type, which will definitely not have a compilation unit. 
+			// Skipping it will speed up performance because the call will open jars. (theodora)
+			CompilationUnitDeclaration parsedUnit = resolver.hasTypeName() ? resolver.getCompilationUnitDeclaration() : null;
 			if (parsedUnit != null) {
 				char[] fileName = parsedUnit.compilationResult.getFileName();
 				Object existing = this.requestedKeys.get(fileName);
@@ -249,12 +251,19 @@ class CompilationUnitResolver extends Compiler {
 		ast.setBindingResolver(resolver);
 		converter.setAST(ast);
 		compilationUnit = converter.convert(compilationUnitDeclaration, source);
-		compilationUnit.setLineEndTable(compilationUnitDeclaration.compilationResult.lineSeparatorPositions);
+		compilationUnit.setLineEndTable(compilationUnitDeclaration.compilationResult.getLineSeparatorPositions());
 		ast.setDefaultNodeFlag(0);
 		ast.setOriginalModificationCount(ast.modificationCount());
 		return compilationUnit;
 	}
 	
+	protected static CompilerOptions getCompilerOptions(Map options, boolean statementsRecovery) {
+		CompilerOptions compilerOptions = new CompilerOptions(options);
+		compilerOptions.performStatementsRecovery = statementsRecovery;
+		compilerOptions.parseLiteralExpressionsAsConstants = false;
+		compilerOptions.storeAnnotations = true /*store annotations in the bindings*/;
+		return compilerOptions;
+	}
 	/*
 	 *  Low-level API performing the actual compilation
 	 */
@@ -344,7 +353,6 @@ class CompilationUnitResolver extends Compiler {
 				
 				//fill the methods bodies in order for the code to be generated
 				//real parse of the method....
-				parser.scanner.setSource(compilationResult);
 				org.eclipse.jdt.internal.compiler.ast.TypeDeclaration[] types = compilationUnitDeclaration.types;
 				if (types != null) {
 					for (int j = types.length; --j >= 0;)
@@ -352,7 +360,7 @@ class CompilationUnitResolver extends Compiler {
 				}
 				
 				// convert AST
-				CompilationUnit node = convert(compilationUnitDeclaration, sourceUnit.getContents(), apiLevel, options, false/*don't resolve binding*/, null/*no owner needed*/, null/*no binding table needed*/, monitor);
+				CompilationUnit node = convert(compilationUnitDeclaration, parser.scanner.getSource(), apiLevel, options, false/*don't resolve binding*/, null/*no owner needed*/, null/*no binding table needed*/, monitor);
 				node.setJavaElement(compilationUnits[i]);
 				
 				// accept AST
@@ -365,11 +373,12 @@ class CompilationUnitResolver extends Compiler {
 		}
 	}
 	
-	public static CompilationUnitDeclaration parse(org.eclipse.jdt.internal.compiler.env.ICompilationUnit sourceUnit, NodeSearcher nodeSearcher, Map settings) {
+	public static CompilationUnitDeclaration parse(org.eclipse.jdt.internal.compiler.env.ICompilationUnit sourceUnit, NodeSearcher nodeSearcher, Map settings, boolean statementsRecovery) {
 		if (sourceUnit == null) {
 			throw new IllegalStateException();
 		}
 		CompilerOptions compilerOptions = new CompilerOptions(settings);
+		compilerOptions.performStatementsRecovery = statementsRecovery;
 		Parser parser = new CommentRecorderParser(
 			new ProblemReporter(
 					DefaultErrorHandlingPolicies.proceedWithAllProblems(), 
@@ -386,7 +395,7 @@ class CompilationUnitResolver extends Compiler {
 		}
 		
 		if (nodeSearcher != null) {
-			char[] source = sourceUnit.getContents();
+			char[] source = parser.scanner.getSource();
 			int searchPosition = nodeSearcher.position;
 			if (searchPosition < 0 || searchPosition > source.length) {
 				// the position is out of range. There is no need to search for a node.
@@ -414,7 +423,6 @@ class CompilationUnitResolver extends Compiler {
 		} else {
 			//fill the methods bodies in order for the code to be generated
 			//real parse of the method....
-			parser.scanner.setSource(compilationResult);
 			org.eclipse.jdt.internal.compiler.ast.TypeDeclaration[] types = compilationUnitDeclaration.types;
 			if (types != null) {
 				for (int i = types.length; --i >= 0;)
@@ -432,6 +440,7 @@ class CompilationUnitResolver extends Compiler {
 		Map options,
 		IJavaProject javaProject,
 		WorkingCopyOwner owner,
+		boolean statementsRecovery,
 		IProgressMonitor monitor) {
 	
 		CancelableNameEnvironment environment = null;
@@ -447,7 +456,7 @@ class CompilationUnitResolver extends Compiler {
 				new CompilationUnitResolver(
 					environment,
 					getHandlingPolicy(),
-					options,
+					getCompilerOptions(options, statementsRecovery),
 					getRequestor(),
 					problemFactory, 
 					monitor);
@@ -476,6 +485,7 @@ class CompilationUnitResolver extends Compiler {
 		NodeSearcher nodeSearcher,
 		Map options,
 		WorkingCopyOwner owner,
+		boolean statementsRecovery,
 		IProgressMonitor monitor) throws JavaModelException {
 	
 		CompilationUnitDeclaration unit = null;
@@ -489,9 +499,9 @@ class CompilationUnitResolver extends Compiler {
 				new CompilationUnitResolver(
 					environment,
 					getHandlingPolicy(),
-					options,
+					getCompilerOptions(options, statementsRecovery),
 					getRequestor(),
-					problemFactory, 
+					problemFactory,
 					monitor);
 
 			unit = 
@@ -505,10 +515,10 @@ class CompilationUnitResolver extends Compiler {
 			if (resolver.hasCompilationAborted) {
 				// the bindings could not be resolved due to missing types in name environment
 				// see https://bugs.eclipse.org/bugs/show_bug.cgi?id=86541
-				CompilationUnitDeclaration unitDeclaration = parse(sourceUnit, nodeSearcher, options);
+				CompilationUnitDeclaration unitDeclaration = parse(sourceUnit, nodeSearcher, options, statementsRecovery);
 				final int problemCount = unit.compilationResult.problemCount;
 				if (problemCount != 0) {
-					unitDeclaration.compilationResult.problems = new IProblem[problemCount];
+					unitDeclaration.compilationResult.problems = new CategorizedProblem[problemCount];
 					System.arraycopy(unit.compilationResult.problems, 0, unitDeclaration.compilationResult.problems, 0, problemCount);
 					unitDeclaration.compilationResult.problemCount = problemCount;
 				}
@@ -543,6 +553,7 @@ class CompilationUnitResolver extends Compiler {
 		Map compilerOptions,
 		IJavaProject javaProject,
 		WorkingCopyOwner owner,
+		boolean statementsRecovery,
 		IProgressMonitor monitor) {
 
 		final int length = elements.length;
@@ -602,7 +613,7 @@ class CompilationUnitResolver extends Compiler {
 			}
 		}
 		Requestor requestor = new Requestor();
-		resolve(cus, bindingKeys, requestor, apiLevel, compilerOptions, javaProject, owner, monitor);
+		resolve(cus, bindingKeys, requestor, apiLevel, compilerOptions, javaProject, owner, statementsRecovery, monitor);
 		return requestor.bindings;
 	}
 	/*
@@ -624,14 +635,14 @@ class CompilationUnitResolver extends Compiler {
 				removeUnresolvedBindings(memberTypes[i]);
 			}
 		}
-		if (type.binding != null && (type.binding.modifiers & CompilerModifiers.AccUnresolved) != 0) {
+		if (type.binding != null && (type.binding.modifiers & ExtraCompilerModifiers.AccUnresolved) != 0) {
 			type.binding = null;
 		}
 		
 		final org.eclipse.jdt.internal.compiler.ast.FieldDeclaration[] fields = type.fields;
 		if (fields != null) {
 			for (int i = 0, max = fields.length; i < max; i++){
-				if (fields[i].binding != null && (fields[i].binding.modifiers & CompilerModifiers.AccUnresolved) != 0) {
+				if (fields[i].binding != null && (fields[i].binding.modifiers & ExtraCompilerModifiers.AccUnresolved) != 0) {
 					fields[i].binding = null;
 				}
 			}
@@ -640,7 +651,7 @@ class CompilationUnitResolver extends Compiler {
 		final AbstractMethodDeclaration[] methods = type.methods;
 		if (methods != null) {
 			for (int i = 0, max = methods.length; i < max; i++){
-				if (methods[i].binding !=  null && (methods[i].binding.modifiers & CompilerModifiers.AccUnresolved) != 0) {
+				if (methods[i].binding !=  null && (methods[i].binding.modifiers & ExtraCompilerModifiers.AccUnresolved) != 0) {
 					methods[i].binding = null;
 				}
 			}
@@ -661,57 +672,105 @@ class CompilationUnitResolver extends Compiler {
 			beginToCompile(sourceUnits, bindingKeys);
 			// process all units (some more could be injected in the loop by the lookup environment)
 			for (; i < this.totalUnits; i++) {
+				if (this.requestedSources.size() == 0 && this.requestedKeys.size() == 0) {
+					// no need to keep resolving if no more ASTs and no more binding keys are needed
+					// see https://bugs.eclipse.org/bugs/show_bug.cgi?id=114935
+					// cleanup remaining units
+					for (; i < this.totalUnits; i++) {
+						this.unitsToProcess[i].cleanUp();
+						this.unitsToProcess[i] = null;
+					}
+					break;
+				}
 				unit = this.unitsToProcess[i];
 				try {
-					char[] fileName = unit.compilationResult.getFileName();
+					super.process(unit, i); // this.process(...) is optimized to not process already known units
+
+					// requested AST
+					char[] fileName = unit.compilationResult.getFileName();					
+					ICompilationUnit source = (ICompilationUnit) this.requestedSources.get(fileName);
+					if (source != null) {
+						// convert AST
+						CompilationResult compilationResult = unit.compilationResult;
+						org.eclipse.jdt.internal.compiler.env.ICompilationUnit sourceUnit = compilationResult.compilationUnit;
+						char[] contents = sourceUnit.getContents();
+						AST ast = AST.newAST(apiLevel);
+						ast.setDefaultNodeFlag(ASTNode.ORIGINAL);
+						ASTConverter converter = new ASTConverter(compilerOptions, true/*need to resolve bindings*/, this.monitor);
+						BindingResolver resolver = new DefaultBindingResolver(unit.scope, owner, this.bindingTables);
+						ast.setBindingResolver(resolver);
+						converter.setAST(ast);
+						CompilationUnit compilationUnit = converter.convert(unit, contents);
+						compilationUnit.setJavaElement(source);
+						compilationUnit.setLineEndTable(compilationResult.getLineSeparatorPositions());
+						ast.setDefaultNodeFlag(0);
+						ast.setOriginalModificationCount(ast.modificationCount());
+						
+						// pass it to requestor
+						astRequestor.acceptAST(source, compilationUnit);
+						
+						worked(1);
+					} 
 					
-					// only process requested units
-					if (this.requestedKeys.containsKey(fileName) || this.requestedSources.containsKey(fileName)) {
-						super.process(unit, i); // this.process(...) is optimized to not process already known units
-						
-						ICompilationUnit source = (ICompilationUnit) this.requestedSources.get(fileName);
-						if (source != null) {
-							// convert AST
-							CompilationResult compilationResult = unit.compilationResult;
-							org.eclipse.jdt.internal.compiler.env.ICompilationUnit sourceUnit = compilationResult.compilationUnit;
-							char[] contents = sourceUnit.getContents();
-							AST ast = AST.newAST(apiLevel);
-							ast.setDefaultNodeFlag(ASTNode.ORIGINAL);
-							ASTConverter converter = new ASTConverter(compilerOptions, true/*need to resolve bindings*/, this.monitor);
-							BindingResolver resolver = new DefaultBindingResolver(unit.scope, owner, this.bindingTables);
-							ast.setBindingResolver(resolver);
-							converter.setAST(ast);
-							CompilationUnit compilationUnit = converter.convert(unit, contents);
-							compilationUnit.setJavaElement(source);
-							compilationUnit.setLineEndTable(compilationResult.lineSeparatorPositions);
-							ast.setDefaultNodeFlag(0);
-							ast.setOriginalModificationCount(ast.modificationCount());
-							
-							// pass it to requestor
-							astRequestor.acceptAST(source, compilationUnit);
-							
+					// requested binding
+					Object key = this.requestedKeys.get(fileName);
+					if (key instanceof BindingKeyResolver) {
+						reportBinding(key, astRequestor, owner, unit);
+						worked(1);
+					} else if (key instanceof ArrayList) {
+						Iterator iterator = ((ArrayList) key).iterator();
+						while (iterator.hasNext()) {
+							reportBinding(iterator.next(), astRequestor, owner, unit);
 							worked(1);
-						} 
-						
-						Object key = this.requestedKeys.get(fileName);
-						if (key instanceof BindingKeyResolver) {
-							reportBinding(key, astRequestor, owner, unit);
-							worked(1);
-						} else if (key instanceof ArrayList) {
-							Iterator iterator = ((ArrayList) key).iterator();
-							while (iterator.hasNext()) {
-								reportBinding(iterator.next(), astRequestor, owner, unit);
-								worked(1);
-							}
 						}
-						
-						// remove at the end so that we don't resolve twice if a source and a key for the same file name have been requested
-						this.requestedSources.removeKey(fileName);
-						this.requestedKeys.removeKey(fileName);
-					} else {
-						if (unit.scope != null)
-							unit.scope.faultInTypes(); // still force resolution of signatures, so clients can query DOM AST
 					}
+					
+					// remove at the end so that we don't resolve twice if a source and a key for the same file name have been requested
+					this.requestedSources.removeKey(fileName);
+					this.requestedKeys.removeKey(fileName);
+						
+/*	Code used to fault in types and resolve which is no longer necessary as all questions asked to forward references are
+ * lazily resolved.
+ * Code used to be:
+					if (this.requestedKeys.containsKey(fileName) || this.requestedSources.containsKey(fileName)) {
+					   ...
+					} else {
+
+						if (unit.scope != null)
+							unit.scope.faultInTypes();// still force resolution of signatures, so clients can query DOM AST
+				
+						// the following ensures that all type, method and field bindings are correctly initialized
+						// as they may be needed by further units
+						// (see https://bugs.eclipse.org/bugs/show_bug.cgi?id=111822)
+						unit.resolve();
+						
+						// note that if this has a performance penalty on clients, the above code should be removed
+						// the following patch would workaround bug 111822:
+
+Index: FieldReference.java
+===================================================================
+RCS file: /cvsroot/eclipse/org.eclipse.jdt.core/compiler/org/eclipse/jdt/internal/compiler/ast/FieldReference.java,v
+retrieving revision 1.87
+diff -u -r1.87 FieldReference.java
+--- FieldReference.java	24 Sep 2005 15:23:46 -0000	1.87
++++ FieldReference.java	7 Oct 2005 13:46:12 -0000
+@@ -407,7 +407,14 @@
+ 
+ 		FieldBinding originalField = binding.original();
+ 		SourceTypeBinding sourceType = (SourceTypeBinding) originalField.declaringClass;
+-		TypeDeclaration typeDecl = sourceType.scope.referenceContext;
++		ClassScope classScope = sourceType.scope;
++		if (classScope == null) {
++			// Non compiler clients may not have resolved enough of the unit when processing it, and
++			// scopes got cleaned. Assuming these clients thus do not care about constant info, will simply
++			// pretend it is not a constant.
++			return NotAConstant;
++		}
++		TypeDeclaration typeDecl = classScope.referenceContext;
+ 		FieldDeclaration fieldDecl = typeDecl.declarationOf(originalField);
+ 
+ 		fieldDecl.resolve(originalField.isStatic() //side effect on binding 
+*/					
 				} finally {
 					// cleanup compilation unit result
 					unit.cleanUp();
@@ -787,12 +846,14 @@ class CompilationUnitResolver extends Compiler {
 				this.parser.getMethodBodies(unit); // no-op if method bodies have already been parsed
 			} else {
 				int searchPosition = nodeSearcher.position;
-				if (searchPosition >= 0 && searchPosition <= sourceUnit.getContents().length) {
+				char[] source = sourceUnit.getContents();
+				int length = source.length;
+				if (searchPosition >= 0 && searchPosition <= length) {
 					unit.traverse(nodeSearcher, unit.scope);
 					
 					org.eclipse.jdt.internal.compiler.ast.ASTNode node = nodeSearcher.found;
 					
-					this.parser.scanner.setSource(unit.compilationResult);
+					this.parser.scanner.setSource(source, unit.compilationResult);
 					
 		 			if (node != null) {
 						org.eclipse.jdt.internal.compiler.ast.TypeDeclaration enclosingTypeDeclaration = nodeSearcher.enclosingType;
