@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -15,6 +15,7 @@ import java.util.*;
 import org.eclipse.jdt.core.compiler.CharOperation;
 import org.eclipse.jdt.internal.compiler.ASTVisitor;
 import org.eclipse.jdt.internal.compiler.impl.*;
+import org.eclipse.jdt.internal.compiler.classfmt.ClassFileConstants;
 import org.eclipse.jdt.internal.compiler.codegen.*;
 import org.eclipse.jdt.internal.compiler.flow.*;
 import org.eclipse.jdt.internal.compiler.lookup.*;
@@ -23,10 +24,7 @@ import org.eclipse.jdt.internal.compiler.util.Messages;
 
 public abstract class Expression extends Statement {
 	
-	public static final boolean isConstantValueRepresentable(
-		Constant constant,
-		int constantTypeID,
-		int targetTypeID) {
+	public static final boolean isConstantValueRepresentable(Constant constant, int constantTypeID, int targetTypeID) {
 
 		//true if there is no loss of precision while casting.
 		// constantTypeID == constant.typeID
@@ -180,6 +178,8 @@ public abstract class Expression extends Statement {
 	
 	public Constant constant;
 	
+	public int statementEnd = -1;
+	
 	//Some expression may not be used - from a java semantic point
 	//of view only - as statements. Other may. In order to avoid the creation
 	//of wrappers around expression in order to tune them as expression
@@ -205,11 +205,7 @@ public abstract class Expression extends Statement {
 	/**
 	 * Returns false if cast is not legal. 
 	 */
-	public final boolean checkCastTypesCompatibility(
-		Scope scope,
-		TypeBinding castType,
-		TypeBinding expressionType,
-		Expression expression) {
+	public final boolean checkCastTypesCompatibility(Scope scope, TypeBinding castType, TypeBinding expressionType, Expression expression) {
 	
 		// see specifications 5.5
 		// handle errors and process constant when needed
@@ -222,7 +218,7 @@ public abstract class Expression extends Statement {
 	
 		// identity conversion cannot be performed upfront, due to side-effects
 		// like constant propagation
-		boolean use15specifics = scope.compilerOptions().sourceLevel >= JDK1_5;
+		boolean use15specifics = scope.compilerOptions().sourceLevel >= ClassFileConstants.JDK1_5;
 		if (castType.isBaseType()) {
 			if (expressionType.isBaseType()) {
 				if (expressionType == castType) {
@@ -261,7 +257,7 @@ public abstract class Expression extends Statement {
 		switch(expressionType.kind()) {
 			case Binding.BASE_TYPE :
 				//-----------cast to something which is NOT a base type--------------------------	
-				if (expressionType == NullBinding) {
+				if (expressionType == TypeBinding.NULL) {
 					tagAsUnnecessaryCast(scope, castType);
 					return true; //null is compatible with every thing
 				}
@@ -289,13 +285,9 @@ public abstract class Expression extends Statement {
 						
 					case Binding.TYPE_PARAMETER : 
 						// ( TYPE_PARAMETER ) ARRAY
-						if (expressionType instanceof ReferenceBinding) {
-							ReferenceBinding match = ((ReferenceBinding)expressionType).findSuperTypeWithSameErasure(castType);
-							if (match == null) {
-								checkUnsafeCast(scope, castType, expressionType, match, true);
-							}
-						} else {
-							checkUnsafeCast(scope, castType, expressionType, null, true);
+						TypeBinding match = expressionType.findSuperTypeWithSameErasure(castType);
+						if (match == null) {
+							checkUnsafeCast(scope, castType, expressionType, match, true);
 						}
 						// recurse on the type variable upper bound
 						return checkCastTypesCompatibility(scope, ((TypeVariableBinding)castType).upperBound(), expressionType, expression);
@@ -316,21 +308,17 @@ public abstract class Expression extends Statement {
 				}
 						
 			case Binding.TYPE_PARAMETER :
-				if (castType instanceof ReferenceBinding) {
-					TypeBinding match = ((ReferenceBinding)expressionType).findSuperTypeWithSameErasure(castType);
-					if (match != null) {
-						return checkUnsafeCast(scope, castType, expressionType, match, false);
-					}					
+				TypeBinding match = expressionType.findSuperTypeWithSameErasure(castType);
+				if (match != null) {
+					return checkUnsafeCast(scope, castType, expressionType, match, false);
 				}
 				// recursively on the type variable upper bound
 				return checkCastTypesCompatibility(scope, castType, ((TypeVariableBinding)expressionType).upperBound(), expression);
 				
 			case Binding.WILDCARD_TYPE : // intersection type
-				if (castType instanceof ReferenceBinding) {
-					TypeBinding match = ((ReferenceBinding)expressionType).findSuperTypeWithSameErasure(castType);
-					if (match != null) {
-						return checkUnsafeCast(scope, castType, expressionType, match, false);
-					}						
+				match = expressionType.findSuperTypeWithSameErasure(castType);
+				if (match != null) {
+					return checkUnsafeCast(scope, castType, expressionType, match, false);
 				}
 				// recursively on the type variable upper bound
 				return checkCastTypesCompatibility(scope, castType, ((WildcardBinding)expressionType).bound, expression);
@@ -351,7 +339,7 @@ public abstract class Expression extends Statement {
 
 						case Binding.TYPE_PARAMETER :
 							// ( INTERFACE ) TYPE_PARAMETER
-							TypeBinding match = ((ReferenceBinding)expressionType).findSuperTypeWithSameErasure(castType);
+							match = expressionType.findSuperTypeWithSameErasure(castType);
 							if (match == null) {
 								checkUnsafeCast(scope, castType, expressionType, match, true);
 							}
@@ -367,7 +355,7 @@ public abstract class Expression extends Statement {
 									return checkUnsafeCast(scope, castType, interfaceType, match, false);
 								}
 								tagAsNeedCheckCast();
-								match = ((ReferenceBinding)castType).findSuperTypeWithSameErasure(interfaceType);
+								match = castType.findSuperTypeWithSameErasure(interfaceType);
 								if (match != null) {
 									return checkUnsafeCast(scope, castType, interfaceType, match, true);
 								}
@@ -400,16 +388,16 @@ public abstract class Expression extends Statement {
 								}
 								if (((ReferenceBinding) castType).isFinal()) {
 									// no subclass for castType, thus compile-time check is valid
-									match = ((ReferenceBinding)castType).findSuperTypeWithSameErasure(expressionType);
-									if (match == null /*|| !match.isCompatibleWith(expressionType)*/) {
-										// potential runtime error
+									match = castType.findSuperTypeWithSameErasure(expressionType);
+									if (match == null) {
 										return false;
 									}
 								}
 								if (use15specifics) {
 									// ensure there is no collision between both interfaces: i.e. I1 extends List<String>, I2 extends List<Object>
-									if (((ReferenceBinding)castType).hasIncompatibleSuperType((ReferenceBinding) expressionType))
+									if (((ReferenceBinding)castType).hasIncompatibleSuperType((ReferenceBinding) expressionType)) {
 										return false;
+									}
 								}
 							}
 					}
@@ -428,7 +416,7 @@ public abstract class Expression extends Statement {
 							
 						case Binding.TYPE_PARAMETER :
 							// ( TYPE_PARAMETER ) CLASS
-							TypeBinding match = ((ReferenceBinding)expressionType).findSuperTypeWithSameErasure(castType);
+							match = expressionType.findSuperTypeWithSameErasure(castType);
 							if (match == null) {
 								checkUnsafeCast(scope, castType, expressionType, match, true);
 							}
@@ -442,7 +430,7 @@ public abstract class Expression extends Statement {
 								match = refExprType.findSuperTypeWithSameErasure(castType);
 								if (refExprType.isFinal()) {
 									// unless final a subclass may implement the interface ==> no check at compile time
-									if (match == null || !match.isCompatibleWith(castType)) {
+									if (match == null) {
 										return false;
 									}
 									return checkUnsafeCast(scope, castType, expressionType, match, false);
@@ -452,7 +440,7 @@ public abstract class Expression extends Statement {
 									}
 								}
 								tagAsNeedCheckCast();
-								match = ((ReferenceBinding)castType).findSuperTypeWithSameErasure(expressionType);
+								match = castType.findSuperTypeWithSameErasure(expressionType);
 								if (match != null) {
 									return checkUnsafeCast(scope, castType, expressionType, match, true);
 								}
@@ -464,12 +452,12 @@ public abstract class Expression extends Statement {
 								return true;
 							} else {
 								// ( CLASS ) CLASS
-								match = ((ReferenceBinding)expressionType).findSuperTypeWithSameErasure(castType);
+								match = expressionType.findSuperTypeWithSameErasure(castType);
 								if (match != null) {
 									if (expression != null && castType.id == T_JavaLangString) this.constant = expression.constant; // (String) cst is still a constant
 									return checkUnsafeCast(scope, castType, expressionType, match, false);
 								}
-								match = ((ReferenceBinding)castType).findSuperTypeWithSameErasure(expressionType);
+								match = castType.findSuperTypeWithSameErasure(expressionType);
 								if (match != null) {
 									tagAsNeedCheckCast();
 									return checkUnsafeCast(scope, castType, expressionType, match, true);
@@ -481,57 +469,39 @@ public abstract class Expression extends Statement {
 		}
 	}	
 	
-	public FlowInfo checkNullStatus(BlockScope scope, FlowContext flowContext, FlowInfo flowInfo, int nullStatus) {
-
-		LocalVariableBinding local = this.localVariableBinding();
-		if (local != null) {
-			switch(nullStatus) {
-				case FlowInfo.NULL :
-					flowContext.recordUsingNullReference(scope, local, this, FlowInfo.NULL, flowInfo);
-					flowInfo.markAsDefinitelyNull(local); // from thereon it is set
-					break;
-				case FlowInfo.NON_NULL :
-					flowContext.recordUsingNullReference(scope, local, this, FlowInfo.NON_NULL, flowInfo);
-					flowInfo.markAsDefinitelyNonNull(local); // from thereon it is set
-					break;
-				case FlowInfo.UNKNOWN :
-					break;
-			}
+/**
+ * Check the local variable of this expression, if any, against potential NPEs 
+ * given a flow context and an upstream flow info. If so, report the risk to
+ * the context. Marks the local as checked, which affects the flow info.
+ * @param scope the scope of the analysis
+ * @param flowContext the current flow context
+ * @param flowInfo the upstream flow info; caveat: may get modified
+ */
+public void checkNPE(BlockScope scope, FlowContext flowContext, 
+		FlowInfo flowInfo) {
+	LocalVariableBinding local = this.localVariableBinding();
+	if (local != null && 
+			(local.type.tagBits & TagBits.IsBaseType) == 0) {
+		if ((this.bits & IsNonNull) == 0) {
+			flowContext.recordUsingNullReference(scope, local, this, 
+					FlowContext.MAY_NULL, flowInfo);
 		}
-		return flowInfo;
-	}
-
-	private MethodBinding[] getAllInheritedMethods(ReferenceBinding binding) {
-		ArrayList collector = new ArrayList();
-		getAllInheritedMethods0(binding, collector);
-		return (MethodBinding[]) collector.toArray(new MethodBinding[collector.size()]);
-	}
-	
-	private void getAllInheritedMethods0(ReferenceBinding binding, ArrayList collector) {
-		if (!binding.isInterface()) return;
-		MethodBinding[] methodBindings = binding.methods();
-		for (int i = 0, max = methodBindings.length; i < max; i++) {
-			collector.add(methodBindings[i]);
-		}
-		ReferenceBinding[] superInterfaces = binding.superInterfaces();
-		for (int i = 0, max = superInterfaces.length; i < max; i++) {
-			getAllInheritedMethods0(superInterfaces[i], collector);
+		flowInfo.markAsComparedEqualToNonNull(local); 
+			// from thereon it is set
+		if (flowContext.initsOnFinally != null) {
+			flowContext.initsOnFinally.markAsComparedEqualToNonNull(local); 
 		}
 	}
-	public void checkNullComparison(BlockScope scope, FlowContext flowContext, FlowInfo flowInfo, FlowInfo initsWhenTrue, FlowInfo initsWhenFalse) {
-		// do nothing by default - see EqualExpression
-	}
+}
 
-	public boolean checkUnsafeCast(Scope scope, TypeBinding castType, TypeBinding expressionType, TypeBinding match, boolean isNarrowing) {
+public boolean checkUnsafeCast(Scope scope, TypeBinding castType, TypeBinding expressionType, TypeBinding match, boolean isNarrowing) {
 		if (match == castType) {
 			if (!isNarrowing) tagAsUnnecessaryCast(scope, castType);
 			return true;
 		}
 		if (match != null && (
 				castType.isBoundParameterizedType() 
-				|| castType.isGenericType() 
-				|| 	expressionType.isBoundParameterizedType() 
-				|| expressionType.isGenericType())) {
+				|| 	expressionType.isBoundParameterizedType())) {
 			
 			if (match.isProvablyDistinctFrom(isNarrowing ? expressionType : castType, 0)) {
 				return false; 
@@ -540,43 +510,46 @@ public abstract class Expression extends Statement {
 		if (!isNarrowing) tagAsUnnecessaryCast(scope, castType);
 		return true;
 	}
-	
 	/**
 	 * Base types need that the widening is explicitly done by the compiler using some bytecode like i2f.
 	 * Also check unsafe type operations.
 	 */ 
-	public void computeConversion(Scope scope, TypeBinding runtimeTimeType, TypeBinding compileTimeType) {
+	public void computeConversion(Scope scope, TypeBinding runtimeType, TypeBinding compileTimeType) {
 
-		if (runtimeTimeType == null || compileTimeType == null)
+		if (runtimeType == null || compileTimeType == null)
 			return;
 		if (this.implicitConversion != 0) return; // already set independantly
-
+		
 		// it is possible for a Byte to be unboxed to a byte & then converted to an int
 		// but it is not possible for a byte to become Byte & then assigned to an Integer,
 		// or to become an int before boxed into an Integer
-		if (runtimeTimeType != NullBinding && runtimeTimeType.isBaseType()) {
+		if (runtimeType != TypeBinding.NULL && runtimeType.isBaseType()) {
 			if (!compileTimeType.isBaseType()) {
 				TypeBinding unboxedType = scope.environment().computeBoxingType(compileTimeType);
 				this.implicitConversion = UNBOXING;
-				scope.problemReporter().autoboxing(this, compileTimeType, runtimeTimeType);
+				scope.problemReporter().autoboxing(this, compileTimeType, runtimeType);
 				compileTimeType = unboxedType;
 			}
-		} else {
-			if (compileTimeType != NullBinding && compileTimeType.isBaseType()) {
-				TypeBinding boxedType = scope.environment().computeBoxingType(runtimeTimeType);
-				if (boxedType == runtimeTimeType) // Object o = 12;
-					boxedType = compileTimeType; 
-				this.implicitConversion = BOXING | (boxedType.id << 4) + compileTimeType.id;
-				scope.problemReporter().autoboxing(this, compileTimeType, scope.environment().computeBoxingType(boxedType));
-				return;
-			}
+		} else if (compileTimeType != TypeBinding.NULL && compileTimeType.isBaseType()) {
+			TypeBinding boxedType = scope.environment().computeBoxingType(runtimeType);
+			if (boxedType == runtimeType) // Object o = 12;
+				boxedType = compileTimeType; 
+			this.implicitConversion = BOXING | (boxedType.id << 4) + compileTimeType.id;
+			scope.problemReporter().autoboxing(this, compileTimeType, scope.environment().computeBoxingType(boxedType));
+			return;
+		} else if (this.constant != Constant.NotAConstant && this.constant.typeID() != T_JavaLangString) {
+			this.implicitConversion = BOXING;
+			return;
 		}
-
-		switch (runtimeTimeType.id) {
+		int compileTimeTypeID, runtimeTypeID;
+		if ((compileTimeTypeID = compileTimeType.id) == NoId) { // e.g. ? extends String  ==> String (103227)
+			compileTimeTypeID = compileTimeType.erasure().id == T_JavaLangString ? T_JavaLangString : T_JavaLangObject;
+		}		
+		switch (runtimeTypeID = runtimeType.id) {
 			case T_byte :
 			case T_short :
 			case T_char :
-				this.implicitConversion |= (T_int << 4) + compileTimeType.id;
+				this.implicitConversion |= (T_int << 4) + compileTimeTypeID;
 				break;
 			case T_JavaLangString :
 			case T_float :
@@ -584,14 +557,15 @@ public abstract class Expression extends Statement {
 			case T_double :
 			case T_int : //implicitConversion may result in i2i which will result in NO code gen
 			case T_long :
-				this.implicitConversion |= (runtimeTimeType.id << 4) + compileTimeType.id;
+				this.implicitConversion |= (runtimeTypeID << 4) + compileTimeTypeID;
 				break;
 			default : // regular object ref
 //				if (compileTimeType.isRawType() && runtimeTimeType.isBoundParameterizedType()) {
 //				    scope.problemReporter().unsafeRawExpression(this, compileTimeType, runtimeTimeType);
 //				}		
 		}
-	}	
+	}
+
 	/**
 	 * Expression statements are plain expressions, however they generate like
 	 * normal expressions with no value required.
@@ -601,12 +575,12 @@ public abstract class Expression extends Statement {
 	 */
 	public void generateCode(BlockScope currentScope, CodeStream codeStream) {
 
-		if ((bits & IsReachableMASK) == 0) {
+		if ((bits & IsReachable) == 0) {
 			return;
 		}
 		generateCode(currentScope, codeStream, false);
 	}
-
+	
 	/**
 	 * Every expression is responsible for generating its implicit conversion when necessary.
 	 *
@@ -619,7 +593,7 @@ public abstract class Expression extends Statement {
 		CodeStream codeStream,
 		boolean valueRequired) {
 
-		if (constant != NotAConstant) {
+		if (constant != Constant.NotAConstant) {
 			// generate a constant expression
 			int pc = codeStream.position;
 			codeStream.generateConstant(constant, implicitConversion);
@@ -628,8 +602,8 @@ public abstract class Expression extends Statement {
 			// actual non-constant code generation
 			throw new ShouldNotImplement(Messages.ast_missingCode); 
 		}
-	}
-
+	}	
+	
 	/**
 	 * Default generation of a boolean value
 	 * @param currentScope
@@ -641,16 +615,18 @@ public abstract class Expression extends Statement {
 	public void generateOptimizedBoolean(
 		BlockScope currentScope,
 		CodeStream codeStream,
-		Label trueLabel,
-		Label falseLabel,
+		BranchLabel trueLabel,
+		BranchLabel falseLabel,
 		boolean valueRequired) {
 
 		// a label valued to nil means: by default we fall through the case... 
 		// both nil means we leave the value on the stack
 
-		if ((constant != Constant.NotAConstant) && (constant.typeID() == T_boolean)) {
+		Constant cst = this.optimizedBooleanConstant();
+		generateCode(currentScope, codeStream, valueRequired && cst == Constant.NotAConstant);
+		if ((cst != Constant.NotAConstant) && (cst.typeID() == T_boolean)) {
 			int pc = codeStream.position;
-			if (constant.booleanValue() == true) {
+			if (cst.booleanValue() == true) {
 				// constant == true
 				if (valueRequired) {
 					if (falseLabel == null) {
@@ -673,7 +649,6 @@ public abstract class Expression extends Statement {
 			codeStream.recordPositionsFrom(pc, this.sourceStart);
 			return;
 		}
-		generateCode(currentScope, codeStream, valueRequired);
 		// branching
 		int position = codeStream.position;
 		if (valueRequired) {
@@ -705,7 +680,7 @@ public abstract class Expression extends Statement {
 		CodeStream codeStream,
 		int typeID) {
 
-		if (typeID == T_JavaLangString && this.constant != NotAConstant && this.constant.stringValue().length() == 0) {
+		if (typeID == T_JavaLangString && this.constant != Constant.NotAConstant && this.constant.stringValue().length() == 0) {
 			return; // optimize str + ""
 		}
 		generateCode(blockScope, codeStream, true);
@@ -734,7 +709,7 @@ public abstract class Expression extends Statement {
 				return;
 			case T_JavaLangString :
 			case T_null :
-				if (constant != NotAConstant) {
+				if (constant != Constant.NotAConstant) {
 					String stringValue = constant.stringValue();
 					if (stringValue.length() == 0) {  // optimize ""+<str> 
 						codeStream.invokeStringConcatenationDefaultConstructor();
@@ -754,10 +729,28 @@ public abstract class Expression extends Statement {
 		codeStream.invokeStringConcatenationStringConstructor();
 	}
 
+	private MethodBinding[] getAllInheritedMethods(ReferenceBinding binding) {
+		ArrayList collector = new ArrayList();
+		getAllInheritedMethods0(binding, collector);
+		return (MethodBinding[]) collector.toArray(new MethodBinding[collector.size()]);
+	}
+
+	private void getAllInheritedMethods0(ReferenceBinding binding, ArrayList collector) {
+		if (!binding.isInterface()) return;
+		MethodBinding[] methodBindings = binding.methods();
+		for (int i = 0, max = methodBindings.length; i < max; i++) {
+			collector.add(methodBindings[i]);
+		}
+		ReferenceBinding[] superInterfaces = binding.superInterfaces();
+		for (int i = 0, max = superInterfaces.length; i < max; i++) {
+			getAllInheritedMethods0(superInterfaces[i], collector);
+		}
+	}
+	
 	public boolean isCompactableOperation() {
 
 		return false;
-	}
+	}	
 
 	//Return true if the conversion is done AUTOMATICALLY by the vm
 	//while the javaVM is an int based-machine, thus for example pushing
@@ -765,17 +758,17 @@ public abstract class Expression extends Statement {
 	//(this request some work d be done by the VM on signed numbers)
 	public boolean isConstantValueOfTypeAssignableToType(TypeBinding constantType, TypeBinding targetType) {
 
-		if (constant == Constant.NotAConstant)
+		if (this.constant == Constant.NotAConstant)
 			return false;
 		if (constantType == targetType)
 			return true;
 		if (constantType.isBaseType() && targetType.isBaseType()) {
 			//No free assignment conversion from anything but to integral ones.
-			if ((constantType == IntBinding
+			if ((constantType == TypeBinding.INT
 				|| BaseTypeBinding.isWidening(T_int, constantType.id))
 				&& (BaseTypeBinding.isNarrowing(targetType.id, T_int))) {
 				//use current explicit conversion in order to get some new value to compare with current one
-				return isConstantValueRepresentable(constant, constantType.id, targetType.id);
+				return isConstantValueRepresentable(this.constant, constantType.id, targetType.id);
 			}
 		}
 		return false;
@@ -784,10 +777,28 @@ public abstract class Expression extends Statement {
 	public boolean isTypeReference() {
 		return false;
 	}
+
+	/**
+	 * Returns the local variable referenced by this node. Can be a direct reference (SingleNameReference)
+	 * or thru a cast expression etc...
+	 */
+	public LocalVariableBinding localVariableBinding() {
+		return null;
+	}
 	
+/**
+ * Mark this expression as being non null, per a specific tag in the
+ * source code.
+ */
+// this is no more called for now, waiting for inter procedural null reference analysis
+public void markAsNonNull() {
+	this.bits |= IsNonNull;
+}
+
 	public int nullStatus(FlowInfo flowInfo) {
 		
-		if (this.constant != null && this.constant != NotAConstant)
+		if (/* (this.bits & IsNonNull) != 0 || */ 
+				this.constant != null && this.constant != Constant.NotAConstant)
 			return FlowInfo.NON_NULL; // constant expression cannot be null
 		
 		LocalVariableBinding local = localVariableBinding();
@@ -812,13 +823,55 @@ public abstract class Expression extends Statement {
 		return this.constant;
 	}
 
+	/**
+	 * Returns the type of the expression after required implicit conversions. When expression type gets promoted
+	 * or inserted a generic cast, the converted type will differ from the resolved type (surface side-effects from
+	 * #computeConversion(...)).
+	 * @return the type after conversion
+	 */
+	public TypeBinding postConversionType(Scope scope) {
+		TypeBinding convertedType = this.resolvedType;
+		int runtimeType = (this.implicitConversion & IMPLICIT_CONVERSION_MASK) >> 4;
+		switch (runtimeType) {
+			case T_boolean :
+				convertedType = TypeBinding.BOOLEAN;
+				break;
+			case T_byte :
+				convertedType = TypeBinding.BYTE;
+				break;
+			case T_short :
+				convertedType = TypeBinding.SHORT;
+				break;
+			case T_char :
+				convertedType = TypeBinding.CHAR;
+				break;
+			case T_int :
+				convertedType = TypeBinding.INT;
+				break;
+			case T_float :
+				convertedType = TypeBinding.FLOAT;
+				break;
+			case T_long :
+				convertedType = TypeBinding.LONG;
+				break;
+			case T_double :
+				convertedType = TypeBinding.DOUBLE;
+				break;
+			default :
+		}		
+		if ((this.implicitConversion & BOXING) != 0) {
+			convertedType = scope.environment().computeBoxingType(convertedType);
+		}
+		return convertedType;
+	}
+
 	public StringBuffer print(int indent, StringBuffer output) {
 		printIndent(indent, output);
 		return printExpression(indent, output);
 	}
-
-	public abstract StringBuffer printExpression(int indent, StringBuffer output);
 	
+	public abstract StringBuffer printExpression(int indent, StringBuffer output);
+
 	public StringBuffer printStatement(int indent, StringBuffer output) {
 		return print(indent, output).append(";"); //$NON-NLS-1$
 	}
@@ -860,6 +913,17 @@ public abstract class Expression extends Statement {
 		return expressionType;
 	}
 
+	/**
+	 * Returns an object which can be used to identify identical JSR sequence targets
+	 * (see TryStatement subroutine codegen)
+	 * or <code>null</null> if not reusable
+	 */
+	public Object reusableJSRTarget() {
+		if (this.constant != Constant.NotAConstant)
+			return this.constant;
+		return null;
+	}
+	
 	/**
 	 * Record the type expectation before this expression is typechecked.
 	 * e.g. String s = foo();, foo() will be tagged as being expected of type String
@@ -868,15 +932,15 @@ public abstract class Expression extends Statement {
 	public void setExpectedType(TypeBinding expectedType) {
 	    // do nothing by default
 	}
-
-	public void tagAsUnnecessaryCast(Scope scope, TypeBinding castType) {
-	    // do nothing by default
-	}
 	
 	public void tagAsNeedCheckCast() {
 	    // do nothing by default		
 	}
 	
+	public void tagAsUnnecessaryCast(Scope scope, TypeBinding castType) {
+	    // do nothing by default
+	}
+	
 	public Expression toTypeReference() {
 		//by default undefined
 
@@ -887,7 +951,6 @@ public abstract class Expression extends Statement {
 
 		return this;
 	}
-	
 	public void traverse(ASTVisitor visitor, BlockScope scope) {
 		// do nothing by default
 	}
@@ -897,11 +960,4 @@ public abstract class Expression extends Statement {
 	public void traverse(ASTVisitor visitor, CompilationUnitScope scope) {
 		// do nothing by default
 	}
-	/**
-	 * Returns the local variable referenced by this node. Can be a direct reference (SingleNameReference)
-	 * or thru a cast expression etc...
-	 */
-	public LocalVariableBinding localVariableBinding() {
-		return null;
-	}
 }
