@@ -34,7 +34,9 @@ public class QualifiedTypeReference extends TypeReference {
 	}
 
 	protected TypeBinding findNextTypeBinding(int tokenIndex, Scope scope, PackageBinding packageBinding) {
+		LookupEnvironment env = scope.environment();
 		try {
+			env.missingClassFileLocation = this;
 		    if (this.resolvedType == null) {
 				this.resolvedType = scope.getType(this.tokens[tokenIndex], packageBinding);
 		    } else {
@@ -51,6 +53,8 @@ public class QualifiedTypeReference extends TypeReference {
 		} catch (AbortCompilation e) {
 			e.updateContext(this, scope.referenceCompilationUnit().compilationResult);
 			throw e;
+		} finally {
+			env.missingClassFileLocation = null;
 		}
 	}
 
@@ -73,7 +77,7 @@ public class QualifiedTypeReference extends TypeReference {
 			findNextTypeBinding(i, scope, packageBinding);
 			if (!this.resolvedType.isValidBinding())
 				return this.resolvedType;
-			if (i == 0 && this.resolvedType.isTypeVariable()) { // cannot select from a type variable
+			if (i == 0 && this.resolvedType.isTypeVariable() && ((TypeVariableBinding) this.resolvedType).firstBound == null) { // cannot select from a type variable
 				scope.problemReporter().illegalAccessFromTypeVariable((TypeVariableBinding) this.resolvedType, this);
 				return this.resolvedType = null;
 			}
