@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -20,10 +20,10 @@ import org.eclipse.jdt.internal.compiler.problem.ProblemSeverities;
 
 public class SingleNameReference extends NameReference implements OperatorIds {
     
-	public char[] token;
-	public MethodBinding[] syntheticAccessors; // [0]=read accessor [1]=write accessor
 	public static final int READ = 0;
 	public static final int WRITE = 1;
+	public char[] token;
+	public MethodBinding[] syntheticAccessors; // [0]=read accessor [1]=write accessor
 	public TypeBinding genericCast;
 	
 	public SingleNameReference(char[] source, long pos) {
@@ -34,7 +34,7 @@ public class SingleNameReference extends NameReference implements OperatorIds {
 	}
 	public FlowInfo analyseAssignment(BlockScope currentScope, FlowContext flowContext, FlowInfo flowInfo, Assignment assignment, boolean isCompound) {
 	
-		boolean isReachable = flowInfo.isReachable();
+		boolean isReachable = (flowInfo.tagBits & FlowInfo.UNREACHABLE) == 0;
 		// compound assignment extra work
 		if (isCompound) { // check the variable part is initialized if blank final
 			switch (bits & RestrictiveFlagMASK) {
@@ -69,9 +69,22 @@ public class SingleNameReference extends NameReference implements OperatorIds {
 			case Binding.FIELD : // assigning to a field
 				manageSyntheticAccessIfNecessary(currentScope, flowInfo, false /*write-access*/);
 	
+				FieldBinding fieldBinding = (FieldBinding) binding;
+				ReferenceBinding declaringClass = fieldBinding.declaringClass;
+				// check if accessing enum static field in initializer
+				if (declaringClass.isEnum()) {
+					MethodScope methodScope = currentScope.methodScope();
+					SourceTypeBinding sourceType = currentScope.enclosingSourceType();
+					if (fieldBinding.isStatic()
+							&& this.constant == Constant.NotAConstant
+							&& !methodScope.isStatic
+							&& (sourceType == declaringClass || sourceType.superclass == declaringClass) // enum constant body
+							&& methodScope.isInsideInitializerOrConstructor()) {
+						currentScope.problemReporter().enumStaticFieldUsedDuringInitialization(fieldBinding, this);
+					}
+				}					
 				// check if assigning a final field
-				FieldBinding fieldBinding;
-				if ((fieldBinding = (FieldBinding) binding).isFinal()) {
+				if (fieldBinding.isFinal()) {
 					// inside a context where allowed
 					if (!isCompound && fieldBinding.isBlankFinal() && currentScope.allowBlankFinalFieldAssignment(fieldBinding)) {
 						if (flowInfo.isPotentiallyAssigned(fieldBinding)) {
@@ -88,9 +101,9 @@ public class SingleNameReference extends NameReference implements OperatorIds {
 			case Binding.LOCAL : // assigning to a local variable 
 				LocalVariableBinding localBinding = (LocalVariableBinding) binding;
 				if (!flowInfo.isDefinitelyAssigned(localBinding)){// for local variable debug attributes
-					bits |= FirstAssignmentToLocalMASK;
+					bits |= FirstAssignmentToLocal;
 				} else {
-					bits &= ~FirstAssignmentToLocalMASK;
+					bits &= ~FirstAssignmentToLocal;
 				}
 				if (localBinding.isFinal()) {
 					if ((bits & DepthMASK) == 0) {
@@ -106,6 +119,9 @@ public class SingleNameReference extends NameReference implements OperatorIds {
 						currentScope.problemReporter().cannotAssignToFinalOuterLocal(localBinding, this);
 					}
 				}
+				else /* avoid double diagnostic */ if ((localBinding.tagBits & TagBits.IsArgument) != 0) {
+					currentScope.problemReporter().parameterAssignment(localBinding, this);
+				}
 				flowInfo.markAsDefinitelyAssigned(localBinding);
 		}
 		manageEnclosingInstanceAccessIfNecessary(currentScope, flowInfo);
@@ -121,10 +137,22 @@ public class SingleNameReference extends NameReference implements OperatorIds {
 				if (valueRequired) {
 					manageSyntheticAccessIfNecessary(currentScope, flowInfo, true /*read-access*/);
 				}
+				FieldBinding fieldBinding = (FieldBinding) binding;
+				ReferenceBinding declaringClass = fieldBinding.declaringClass;
+				// check if accessing enum static field in initializer
+				if (declaringClass.isEnum()) {
+					MethodScope methodScope = currentScope.methodScope();
+					SourceTypeBinding sourceType = currentScope.enclosingSourceType();
+					if (fieldBinding.isStatic()
+							&& this.constant == Constant.NotAConstant
+							&& !methodScope.isStatic
+							&& (sourceType == declaringClass || sourceType.superclass == declaringClass) // enum constant body
+							&& methodScope.isInsideInitializerOrConstructor()) {
+						currentScope.problemReporter().enumStaticFieldUsedDuringInitialization(fieldBinding, this);
+					}
+				}				
 				// check if reading a final blank field
-				FieldBinding fieldBinding;
-				if ((fieldBinding = (FieldBinding) binding).isBlankFinal() 
-						&& currentScope.allowBlankFinalFieldAssignment(fieldBinding)) {
+				if (fieldBinding.isBlankFinal() && currentScope.allowBlankFinalFieldAssignment(fieldBinding)) {
 					if (!flowInfo.isDefinitelyAssigned(fieldBinding)) {
 						currentScope.problemReporter().uninitializedBlankFinalField(fieldBinding, this);
 					}
@@ -135,7 +163,7 @@ public class SingleNameReference extends NameReference implements OperatorIds {
 				if (!flowInfo.isDefinitelyAssigned(localBinding = (LocalVariableBinding) binding)) {
 					currentScope.problemReporter().uninitializedLocalVariable(localBinding, this);
 				}
-				if (flowInfo.isReachable()) {
+				if ((flowInfo.tagBits & FlowInfo.UNREACHABLE) == 0)	{
 					localBinding.useFlag = LocalVariableBinding.USED;
 				} else if (localBinding.useFlag == LocalVariableBinding.UNUSED) {
 					localBinding.useFlag = LocalVariableBinding.FAKE_USED;
@@ -153,30 +181,28 @@ public class SingleNameReference extends NameReference implements OperatorIds {
 		
 		bits &= ~RestrictiveFlagMASK; // clear bits
 		bits |= Binding.FIELD;
-		if (!((FieldBinding) binding).isStatic()) {
+		MethodScope methodScope = scope.methodScope();
+		boolean isStatic = fieldBinding.isStatic();
+		if (!isStatic) {
 			// must check for the static status....
-			if (scope.methodScope().isStatic) {
+			if (methodScope.isStatic) {
 				scope.problemReporter().staticFieldAccessToNonStaticVariable(this, fieldBinding);
-				constant = NotAConstant;
+				this.constant = Constant.NotAConstant;
 				return fieldBinding.type;
 			}
 		}
-		constant = FieldReference.getConstantFor(fieldBinding, this, true, scope);
+		this.constant = fieldBinding.constant();
 	
-		if (isFieldUseDeprecated(fieldBinding, scope, (this.bits & IsStrictlyAssignedMASK) !=0))
+		if (isFieldUseDeprecated(fieldBinding, scope, (this.bits & IsStrictlyAssigned) !=0))
 			scope.problemReporter().deprecatedField(fieldBinding, this);
 	
-		MethodScope ms = scope.methodScope();
-		if ((this.bits & IsStrictlyAssignedMASK) == 0
-			&& ms.enclosingSourceType() == fieldBinding.declaringClass
-			&& ms.lastVisibleFieldID >= 0
-			&& fieldBinding.id >= ms.lastVisibleFieldID) {
-			//if the field is static and ms is not .... then it is valid
-			if (!fieldBinding.isStatic() || ms.isStatic)
-				scope.problemReporter().forwardReference(this, 0, scope.enclosingSourceType());
+		if ((this.bits & IsStrictlyAssigned) == 0
+				&& methodScope.enclosingSourceType() == fieldBinding.original().declaringClass
+				&& methodScope.lastVisibleFieldID >= 0
+				&& fieldBinding.id >= methodScope.lastVisibleFieldID
+				&& (!fieldBinding.isStatic() || methodScope.isStatic)) {
+			scope.problemReporter().forwardReference(this, 0, methodScope.enclosingSourceType());
 		}
-		//====================================================
-	
 		return fieldBinding.type;
 	
 	}
@@ -189,15 +215,18 @@ public class SingleNameReference extends NameReference implements OperatorIds {
 			return;				
 		if ((bits & Binding.FIELD) != 0 && this.binding != null && this.binding.isValidBinding()) {
 			// set the generic cast after the fact, once the type expectation is fully known (no need for strict cast)
-			FieldBinding originalBinding = ((FieldBinding)this.binding).original();
-			if (originalBinding != this.binding) {
-			    // extra cast needed if method return type has type variable
-			    if ((originalBinding.type.tagBits & TagBits.HasTypeVariable) != 0 && runtimeTimeType.id != T_JavaLangObject) {
-			    	TypeBinding targetType = (!compileTimeType.isBaseType() && runtimeTimeType.isBaseType()) 
-			    		? compileTimeType  // unboxing: checkcast before conversion
-			    		: runtimeTimeType;
-			        this.genericCast = originalBinding.type.genericCast(scope.boxing(targetType));
-			    }
+			FieldBinding field = (FieldBinding) this.binding;
+			FieldBinding originalBinding = field.original();
+			TypeBinding originalType = originalBinding.type;
+		    // extra cast needed if method return type is type variable
+			if (originalBinding != field 
+					&& originalType != field.type
+					&& runtimeTimeType.id != T_JavaLangObject
+					&& (originalType.tagBits & TagBits.HasTypeVariable) != 0) {
+		    	TypeBinding targetType = (!compileTimeType.isBaseType() && runtimeTimeType.isBaseType()) 
+		    		? compileTimeType  // unboxing: checkcast before conversion
+		    		: runtimeTimeType;
+		        this.genericCast = originalType.genericCast(scope.boxing(targetType));
 			} 	
 		}
 		super.computeConversion(scope, runtimeTimeType, compileTimeType);
@@ -221,7 +250,7 @@ public class SingleNameReference extends NameReference implements OperatorIds {
 			if ((operation.right instanceof SingleNameReference)
 					&& ((operator == PLUS) || (operator == MULTIPLY)) // only commutative operations
 					&& ((variableReference = (SingleNameReference) operation.right).binding == binding)
-					&& (operation.left.constant != NotAConstant) // exclude non constant expressions, since could have side-effect
+					&& (operation.left.constant != Constant.NotAConstant) // exclude non constant expressions, since could have side-effect
 					&& (((operation.left.implicitConversion & IMPLICIT_CONVERSION_MASK) >> 4) != T_JavaLangString) // exclude string concatenation which would occur backwards
 					&& (((operation.right.implicitConversion & IMPLICIT_CONVERSION_MASK) >> 4) != T_JavaLangString)) { // exclude string concatenation which would occur backwards
 				// i = value + i, then use the variable on the right hand side, since it has the correct implicit conversion
@@ -258,7 +287,7 @@ public class SingleNameReference extends NameReference implements OperatorIds {
 				if (localBinding.resolvedPosition != -1) {
 					assignment.expression.generateCode(currentScope, codeStream, true);
 				} else {
-					if (assignment.expression.constant != NotAConstant) {
+					if (assignment.expression.constant != Constant.NotAConstant) {
 						// assigning an unused local to a constant value = no actual assignment is necessary
 						if (valueRequired) {
 							codeStream.generateConstant(assignment.expression.constant, assignment.implicitConversion);
@@ -270,7 +299,7 @@ public class SingleNameReference extends NameReference implements OperatorIds {
 						if (valueRequired) {
 							codeStream.generateImplicitConversion(assignment.implicitConversion); // implicit conversion
 						} else {
-							if ((localBinding.type == LongBinding) || (localBinding.type == DoubleBinding)) {
+							if ((localBinding.type == TypeBinding.LONG) || (localBinding.type == TypeBinding.DOUBLE)) {
 								codeStream.pop2();
 							} else {
 								codeStream.pop();
@@ -281,15 +310,15 @@ public class SingleNameReference extends NameReference implements OperatorIds {
 				}
 				// 26903, need extra cast to store null in array local var	
 				if (localBinding.type.isArrayType() 
-					&& (assignment.expression.resolvedType == NullBinding	// arrayLoc = null
+					&& (assignment.expression.resolvedType == TypeBinding.NULL	// arrayLoc = null
 						|| ((assignment.expression instanceof CastExpression)	// arrayLoc = (type[])null
-							&& (((CastExpression)assignment.expression).innermostCastedExpression().resolvedType == NullBinding)))){
+							&& (((CastExpression)assignment.expression).innermostCastedExpression().resolvedType == TypeBinding.NULL)))){
 					codeStream.checkcast(localBinding.type); 
 				}
 				
 				// normal local assignment (since cannot store in outer local which are final locations)
 				codeStream.store(localBinding, valueRequired);
-				if ((bits & FirstAssignmentToLocalMASK) != 0) { // for local variable debug attributes
+				if ((bits & FirstAssignmentToLocal) != 0) { // for local variable debug attributes
 					localBinding.recordInitializationStartPC(codeStream.position);
 				}
 				// implicit conversion
@@ -300,7 +329,7 @@ public class SingleNameReference extends NameReference implements OperatorIds {
 	}
 	public void generateCode(BlockScope currentScope, CodeStream codeStream, boolean valueRequired) {
 		int pc = codeStream.position;
-		if (constant != NotAConstant) {
+		if (constant != Constant.NotAConstant) {
 			if (valueRequired) {
 				codeStream.generateConstant(constant, implicitConversion);
 			}
@@ -308,48 +337,63 @@ public class SingleNameReference extends NameReference implements OperatorIds {
 			switch (bits & RestrictiveFlagMASK) {
 				case Binding.FIELD : // reading a field
 					FieldBinding fieldBinding = (FieldBinding) this.codegenBinding;
-					if (fieldBinding.isConstantValue()) {
+					Constant fieldConstant = fieldBinding.constant();
+					if (fieldConstant != Constant.NotAConstant) {
 						// directly use inlined value for constant fields
 						if (valueRequired) {
-							codeStream.generateConstant(fieldBinding.constant(), implicitConversion);
+							codeStream.generateConstant(fieldConstant, implicitConversion);
 						}
+						break;
+					}
+					if (fieldBinding.isStatic()) {
+						if (!valueRequired) {
+							// if no valueRequired, still need possible side-effects of <clinit> invocation, if field belongs to different class
+							if (((FieldBinding)binding).original().declaringClass == this.actualReceiverType.erasure()) {
+								break;
+							}
+						}
+						// managing private access							
+						if ((syntheticAccessors == null) || (syntheticAccessors[READ] == null)) {
+							codeStream.getstatic(fieldBinding);
+						} else {
+							codeStream.invokestatic(syntheticAccessors[READ]);
+						}
+						if (valueRequired) {
+							if (this.genericCast != null) codeStream.checkcast(this.genericCast);			
+							codeStream.generateImplicitConversion(implicitConversion);
+						} else {
+							// could occur if !valueRequired but static field belongs to different class
+							switch (fieldBinding.type.id) {
+								case T_long :
+								case T_double :
+									codeStream.pop2();
+									break;
+								default :
+									codeStream.pop();
+							}
+						}							
 					} else {
-						if (valueRequired || currentScope.compilerOptions().complianceLevel >= ClassFileConstants.JDK1_4) {
-							boolean isStatic = fieldBinding.isStatic();
-							if (!isStatic) {
-								if ((bits & DepthMASK) != 0) {
-									ReferenceBinding targetType = currentScope.enclosingSourceType().enclosingTypeAt((bits & DepthMASK) >> DepthSHIFT);
-									Object[] emulationPath = currentScope.getEmulationPath(targetType, true /*only exact match*/, false/*consider enclosing arg*/);
-									codeStream.generateOuterAccess(emulationPath, this, targetType, currentScope);
-								} else {
-									generateReceiver(codeStream);
-								}
-							}
-							// managing private access							
-							if ((syntheticAccessors == null) || (syntheticAccessors[READ] == null)) {
-								if (isStatic) {
-									codeStream.getstatic(fieldBinding);
-								} else {
-									codeStream.getfield(fieldBinding);
-								}
-							} else {
-								codeStream.invokestatic(syntheticAccessors[READ]);
-							}
-							if (valueRequired) {
-								if (this.genericCast != null) codeStream.checkcast(this.genericCast);			
-								codeStream.generateImplicitConversion(implicitConversion);
-							} else {
-								// could occur if !valueRequired but compliance >= 1.4
-								switch (fieldBinding.type.id) {
-									case T_long :
-									case T_double :
-										codeStream.pop2();
-										break;
-									default :
-										codeStream.pop();
-								}
-							}							
+						if (!valueRequired) {
+							// if no valueRequired, optimize out entire gen
+							break;
 						}
+						// managing enclosing instance access
+						if ((bits & DepthMASK) != 0) {
+							ReferenceBinding targetType = currentScope.enclosingSourceType().enclosingTypeAt((bits & DepthMASK) >> DepthSHIFT);
+							Object[] emulationPath = currentScope.getEmulationPath(targetType, true /*only exact match*/, false/*consider enclosing arg*/);
+							codeStream.generateOuterAccess(emulationPath, this, targetType, currentScope);
+						} else {
+							generateReceiver(codeStream);
+						}
+						// managing private access							
+						if ((syntheticAccessors == null) || (syntheticAccessors[READ] == null)) {
+							codeStream.getfield(fieldBinding);
+						} else {
+							codeStream.invokestatic(syntheticAccessors[READ]);
+						}
+						// managing generic cast
+						if (this.genericCast != null) codeStream.checkcast(this.genericCast);			
+						codeStream.generateImplicitConversion(implicitConversion);
 					}
 					break;
 				case Binding.LOCAL : // reading a local
@@ -431,7 +475,7 @@ public class SingleNameReference extends NameReference implements OperatorIds {
 						codeStream.store(localBinding, false);
 						return;
 					case T_int :
-						if (((assignConstant = expression.constant) != NotAConstant) 
+						if (((assignConstant = expression.constant) != Constant.NotAConstant) 
 							&& (assignConstant.typeID() != T_float) // only for integral types
 							&& (assignConstant.typeID() != T_double)
 							&& ((increment = assignConstant.intValue()) == (short) increment)) { // 16 bits value
@@ -468,10 +512,12 @@ public class SingleNameReference extends NameReference implements OperatorIds {
 				break;
 			default :
 				// promote the array reference to the suitable operation type
-				codeStream.generateImplicitConversion(implicitConversion);
+				if (this.genericCast != null)
+					codeStream.checkcast(this.genericCast);
+				codeStream.generateImplicitConversion(this.implicitConversion);
 				// generate the increment value (will by itself  be promoted to the operation value)
 				if (expression == IntLiteral.One){ // prefix operation
-					codeStream.generateConstant(expression.constant, implicitConversion);			
+					codeStream.generateConstant(expression.constant, this.implicitConversion);			
 				} else {
 					expression.generateCode(currentScope, codeStream, true);
 				}		
@@ -489,7 +535,7 @@ public class SingleNameReference extends NameReference implements OperatorIds {
 			case Binding.LOCAL : // assigning to a local variable
 				LocalVariableBinding localBinding = (LocalVariableBinding) this.codegenBinding;
 				if (valueRequired) {
-					if ((localBinding.type == LongBinding) || (localBinding.type == DoubleBinding)) {
+					if ((localBinding.type == TypeBinding.LONG) || (localBinding.type == TypeBinding.DOUBLE)) {
 						codeStream.dup2();
 					} else {
 						codeStream.dup();
@@ -526,30 +572,32 @@ public class SingleNameReference extends NameReference implements OperatorIds {
 				}
 				if (valueRequired) {
 					if (fieldBinding.isStatic()) {
-						if ((fieldBinding.type == LongBinding) || (fieldBinding.type == DoubleBinding)) {
+						if ((fieldBinding.type == TypeBinding.LONG) || (fieldBinding.type == TypeBinding.DOUBLE)) {
 							codeStream.dup2();
 						} else {
 							codeStream.dup();
 						}
 					} else { // Stack:  [owner][old field value]  ---> [old field value][owner][old field value]
-						if ((fieldBinding.type == LongBinding) || (fieldBinding.type == DoubleBinding)) {
+						if ((fieldBinding.type == TypeBinding.LONG) || (fieldBinding.type == TypeBinding.DOUBLE)) {
 							codeStream.dup2_x1();
 						} else {
 							codeStream.dup_x1();
 						}
 					}
 				}
-				codeStream.generateImplicitConversion(implicitConversion);		
-				codeStream.generateConstant(postIncrement.expression.constant, implicitConversion);
+				if (this.genericCast != null) 
+					codeStream.checkcast(this.genericCast);
+				codeStream.generateImplicitConversion(this.implicitConversion);		
+				codeStream.generateConstant(postIncrement.expression.constant, this.implicitConversion);
 				codeStream.sendOperator(postIncrement.operator, this.implicitConversion & COMPILE_TYPE_MASK);
 				codeStream.generateImplicitConversion(postIncrement.preAssignImplicitConversion);
-				fieldStore(codeStream, fieldBinding, syntheticAccessors == null ? null : syntheticAccessors[WRITE], false);
+				fieldStore(codeStream, fieldBinding, this.syntheticAccessors == null ? null : this.syntheticAccessors[WRITE], false);
 				// no need for generic cast 
 				return;
 			case Binding.LOCAL : // assigning to a local variable
 				LocalVariableBinding localBinding = (LocalVariableBinding) this.codegenBinding;
 				// using incr bytecode if possible
-				if (localBinding.type == IntBinding) {
+				if (localBinding.type == TypeBinding.INT) {
 					if (valueRequired) {
 						codeStream.load(localBinding);
 					}
@@ -561,7 +609,7 @@ public class SingleNameReference extends NameReference implements OperatorIds {
 				} else {
 					codeStream.load(localBinding);
 					if (valueRequired){
-						if ((localBinding.type == LongBinding) || (localBinding.type == DoubleBinding)) {
+						if ((localBinding.type == TypeBinding.LONG) || (localBinding.type == TypeBinding.DOUBLE)) {
 							codeStream.dup2();
 						} else {
 							codeStream.dup();
@@ -575,35 +623,50 @@ public class SingleNameReference extends NameReference implements OperatorIds {
 					codeStream.store(localBinding, false);
 				}
 		}
-	}
+	}	
 	
 	public void generateReceiver(CodeStream codeStream) {
 		codeStream.aload_0();
 	}
-
+	
 	/**
 	 * @see org.eclipse.jdt.internal.compiler.lookup.InvocationSite#genericTypeArguments()
 	 */
 	public TypeBinding[] genericTypeArguments() {
 		return null;
 	}
+
+	/**
+	 * Returns the local variable referenced by this node. Can be a direct reference (SingleNameReference)
+	 * or thru a cast expression etc...
+	 */
+	public LocalVariableBinding localVariableBinding() {
+		switch (bits & RestrictiveFlagMASK) {
+			case Binding.FIELD : // reading a field
+				break;
+			case Binding.LOCAL : // reading a local variable
+				return (LocalVariableBinding) this.binding;
+		}
+		return null;
+	}
 	
 	public void manageEnclosingInstanceAccessIfNecessary(BlockScope currentScope, FlowInfo flowInfo) {
 	
-		if (!flowInfo.isReachable()) return;
+		if ((flowInfo.tagBits & FlowInfo.UNREACHABLE) == 0)	{
 		//If inlinable field, forget the access emulation, the code gen will directly target it
-		if (((bits & DepthMASK) == 0) || (constant != NotAConstant)) return;
+		if (((bits & DepthMASK) == 0) || (constant != Constant.NotAConstant)) return;
 	
 		if ((bits & RestrictiveFlagMASK) == Binding.LOCAL) {
 			currentScope.emulateOuterAccess((LocalVariableBinding) binding);
 		}
+		}
 	}
 	public void manageSyntheticAccessIfNecessary(BlockScope currentScope, FlowInfo flowInfo, boolean isReadAccess) {
 	
-		if (!flowInfo.isReachable()) return;
+		if ((flowInfo.tagBits & FlowInfo.UNREACHABLE) != 0)	return;
 	
 		//If inlinable field, forget the access emulation, the code gen will directly target it
-		if (constant != NotAConstant)
+		if (constant != Constant.NotAConstant)
 			return;
 	
 		if ((bits & Binding.FIELD) != 0) {
@@ -629,7 +692,7 @@ public class SingleNameReference extends NameReference implements OperatorIds {
 			if (fieldBinding.declaringClass != this.actualReceiverType
 					&& !this.actualReceiverType.isArrayType()
 					&& fieldBinding.declaringClass != null // array.length
-					&& !fieldBinding.isConstantValue()) {
+					&& fieldBinding.constant() == Constant.NotAConstant) {
 				CompilerOptions options = currentScope.compilerOptions();
 				if ((options.targetJDK >= ClassFileConstants.JDK1_2
 						&& (options.complianceLevel >= ClassFileConstants.JDK1_4 || !fieldBinding.isStatic())
@@ -644,11 +707,72 @@ public class SingleNameReference extends NameReference implements OperatorIds {
 			}					
 		}
 	}
+
+public int nullStatus(FlowInfo flowInfo) {
+	if (this.constant != null && this.constant != Constant.NotAConstant) {
+		return FlowInfo.NON_NULL; // constant expression cannot be null
+	}
+	switch (bits & RestrictiveFlagMASK) {
+		case Binding.FIELD : // reading a field
+			return FlowInfo.UNKNOWN;
+		case Binding.LOCAL : // reading a local variable
+			LocalVariableBinding local = (LocalVariableBinding) this.binding;
+			if (local != null) {
+				if (flowInfo.isDefinitelyNull(local))
+					return FlowInfo.NULL;
+				if (flowInfo.isDefinitelyNonNull(local))
+					return FlowInfo.NON_NULL;
+				return FlowInfo.UNKNOWN;
+			}
+	}
+	return FlowInfo.NON_NULL; // never get there 
+}
+
+	/**
+	 * @see org.eclipse.jdt.internal.compiler.ast.Expression#postConversionType(Scope)
+	 */
+	public TypeBinding postConversionType(Scope scope) {
+		TypeBinding convertedType = this.resolvedType;
+		if (this.genericCast != null) 
+			convertedType = this.genericCast;
+		int runtimeType = (this.implicitConversion & IMPLICIT_CONVERSION_MASK) >> 4;
+		switch (runtimeType) {
+			case T_boolean :
+				convertedType = TypeBinding.BOOLEAN;
+				break;
+			case T_byte :
+				convertedType = TypeBinding.BYTE;
+				break;
+			case T_short :
+				convertedType = TypeBinding.SHORT;
+				break;
+			case T_char :
+				convertedType = TypeBinding.CHAR;
+				break;
+			case T_int :
+				convertedType = TypeBinding.INT;
+				break;
+			case T_float :
+				convertedType = TypeBinding.FLOAT;
+				break;
+			case T_long :
+				convertedType = TypeBinding.LONG;
+				break;
+			case T_double :
+				convertedType = TypeBinding.DOUBLE;
+				break;
+			default :
+		}		
+		if ((this.implicitConversion & BOXING) != 0) {
+			convertedType = scope.environment().computeBoxingType(convertedType);
+		}
+		return convertedType;
+	}
+	
 	public StringBuffer printExpression(int indent, StringBuffer output){
 	
 		return output.append(token);
 	}
-	
 	public TypeBinding reportError(BlockScope scope) {
 		
 		//=====error cases=======
@@ -662,6 +786,7 @@ public class SingleNameReference extends NameReference implements OperatorIds {
 		}
 		return null;
 	}
+		
 	public TypeBinding resolveType(BlockScope scope) {
 		// for code gen, harm the restrictiveFlag 	
 	
@@ -685,12 +810,12 @@ public class SingleNameReference extends NameReference implements OperatorIds {
 								scope.problemReporter().cannotReferToNonFinalOuterLocal((LocalVariableBinding)variable, this);
 							}
 							TypeBinding fieldType = variable.type;
-							if ((this.bits & IsStrictlyAssignedMASK) == 0) {
+							if ((this.bits & IsStrictlyAssigned) == 0) {
 								constant = variable.constant();
 								if (fieldType != null) 
 									fieldType = fieldType.capture(scope, this.sourceEnd); // perform capture conversion if read access
 							} else {
-								constant = NotAConstant;
+								constant = Constant.NotAConstant;
 							}
 							return this.resolvedType = fieldType;
 						}
@@ -702,7 +827,7 @@ public class SingleNameReference extends NameReference implements OperatorIds {
 						// perform capture conversion if read access
 						TypeBinding fieldType = checkFieldAccess(scope);
 						return this.resolvedType = 
-							(((this.bits & IsStrictlyAssignedMASK) == 0) 
+							(((this.bits & IsStrictlyAssigned) == 0) 
 								? fieldType.capture(scope, this.sourceEnd)
 								: fieldType);
 					}
@@ -716,14 +841,15 @@ public class SingleNameReference extends NameReference implements OperatorIds {
 					TypeBinding type = (TypeBinding)binding;
 					if (isTypeUseDeprecated(type, scope))
 						scope.problemReporter().deprecatedType(type, this);
-					return this.resolvedType = scope.environment().convertToRawType(type);
+					type = scope.environment().convertToRawType(type);
+					return this.resolvedType = type;
 			}
 		}
 	
 		// error scenarii
 		return this.resolvedType = this.reportError(scope);
 	}
-		
+	
 	public void traverse(ASTVisitor visitor, BlockScope scope) {
 		
 		visitor.visit(this, scope);
@@ -733,19 +859,5 @@ public class SingleNameReference extends NameReference implements OperatorIds {
 	public String unboundReferenceErrorName(){
 	
 		return new String(token);
-	}
-	
-	/**
-	 * Returns the local variable referenced by this node. Can be a direct reference (SingleNameReference)
-	 * or thru a cast expression etc...
-	 */
-	public LocalVariableBinding localVariableBinding() {
-		switch (bits & RestrictiveFlagMASK) {
-			case Binding.FIELD : // reading a field
-				break;
-			case Binding.LOCAL : // reading a local variable
-				return (LocalVariableBinding) this.binding;
-		}
-		return null;
 	}	
 }
