@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -10,10 +10,12 @@
  *******************************************************************************/
 package org.eclipse.jdt.internal.corext.dom;
 
+import org.eclipse.core.runtime.Assert;
+
 import org.eclipse.jdt.core.dom.ASTMatcher;
 import org.eclipse.jdt.core.dom.ASTNode;
+import org.eclipse.jdt.core.dom.IBinding;
 import org.eclipse.jdt.core.dom.SimpleName;
-import org.eclipse.jdt.internal.corext.Assert;
 
 public class JdtASTMatcher extends ASTMatcher {
 
@@ -22,8 +24,17 @@ public class JdtASTMatcher extends ASTMatcher {
 		if (! isomorphic || !(other instanceof SimpleName))
 			return false;
 		SimpleName name= (SimpleName)other;
-		if (node.resolveBinding() != name.resolveBinding())
-			return false;
+		IBinding nodeBinding= node.resolveBinding();
+		IBinding otherBinding= name.resolveBinding();
+		if (nodeBinding == null) {
+			if (otherBinding != null) {
+				return false;
+			}
+		} else {
+			if (! Bindings.equals(nodeBinding, otherBinding)) { // workaround for bug 104293 (see bug 108115)
+				return false;
+			}
+		}
 		if (node.resolveTypeBinding() != name.resolveTypeBinding())
 			return false;
 		return true;	
