@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -30,6 +30,7 @@ import org.eclipse.swt.widgets.Button;
 import org.eclipse.swt.widgets.Composite;
 import org.eclipse.swt.widgets.Control;
 import org.eclipse.swt.widgets.Event;
+import org.eclipse.swt.widgets.Label;
 import org.eclipse.swt.widgets.Listener;
 import org.eclipse.swt.widgets.Shell;
 import org.eclipse.swt.widgets.Table;
@@ -55,17 +56,19 @@ import org.eclipse.jface.viewers.ITableLabelProvider;
 import org.eclipse.jface.viewers.LabelProvider;
 import org.eclipse.jface.viewers.SelectionChangedEvent;
 import org.eclipse.jface.viewers.Viewer;
+import org.eclipse.jface.window.Window;
 
 import org.eclipse.ui.IWorkbench;
 import org.eclipse.ui.IWorkbenchPreferencePage;
 import org.eclipse.ui.PlatformUI;
-import org.eclipse.ui.dialogs.ElementListSelectionDialog;
 import org.eclipse.ui.dialogs.SelectionDialog;
 import org.eclipse.ui.model.WorkbenchViewerSorter;
+import org.eclipse.ui.progress.IProgressService;
 
 import org.eclipse.jdt.core.IJavaElement;
 import org.eclipse.jdt.core.IType;
 import org.eclipse.jdt.core.JavaModelException;
+import org.eclipse.jdt.core.search.IJavaSearchScope;
 import org.eclipse.jdt.core.search.SearchEngine;
 
 import org.eclipse.jdt.ui.IJavaElementSearchConstants;
@@ -75,7 +78,10 @@ import org.eclipse.jdt.ui.JavaUI;
 import org.eclipse.jdt.internal.ui.util.SWTUtil;
 import org.eclipse.jdt.internal.ui.util.TableLayoutComposite;
 
+
+import org.eclipse.jdt.internal.junit.launcher.AssertionVMArg;
 import org.eclipse.jdt.internal.junit.util.ExceptionHandler;
+import org.eclipse.jdt.internal.junit.util.LayoutUtil;
 
 /**
  * Preference page for JUnit settings. Supports to define the failure
@@ -87,7 +93,11 @@ public class JUnitPreferencePage extends PreferencePage implements IWorkbenchPre
 	private static final Image IMG_CUNIT= JavaUI.getSharedImages().getImage(ISharedImages.IMG_OBJS_CLASS);
 	private static final Image IMG_PKG= JavaUI.getSharedImages().getImage(ISharedImages.IMG_OBJS_PACKAGE);
 	
+	// enable assertions widget
+	private Button fEnableAssertionsCheckBox;
+
 	// Step filter widgets
+	private Label fFilterViewerLabel;
 	private CheckboxTableViewer fFilterViewer;
 	private Table fFilterTable;
 
@@ -303,7 +313,7 @@ public class JUnitPreferencePage extends PreferencePage implements IWorkbenchPre
 		
 	public JUnitPreferencePage() {
 		super();
-		setDescription(JUnitMessages.JUnitPreferencePage_filter_label/*JUnitMessages.JUnitPreferencePage_description*/); 
+		setDescription(JUnitMessages.JUnitPreferencePage_description); 
 		setPreferenceStore(JUnitPlugin.getDefault().getPreferenceStore());
 	}
 
@@ -321,15 +331,41 @@ public class JUnitPreferencePage extends PreferencePage implements IWorkbenchPre
 		data.horizontalAlignment= GridData.FILL;
 		composite.setLayoutData(data);
 
+		createEnableAssertionsCheckbox(composite);
 		createStackFilterPreferences(composite);
 		Dialog.applyDialogFont(composite);
 		return composite;
 	}
 
+	private void createEnableAssertionsCheckbox(Composite container) {
+		fEnableAssertionsCheckBox= new Button(container, SWT.CHECK | SWT.WRAP);
+		fEnableAssertionsCheckBox.setText(JUnitMessages.JUnitPreferencePage_enableassertionscheckbox_label); 
+		fEnableAssertionsCheckBox.setToolTipText(JUnitMessages.JUnitPreferencePage_enableassertionscheckbox_tooltip); 
+		GridData gd= getButtonGridData(fEnableAssertionsCheckBox);
+		fEnableAssertionsCheckBox.setLayoutData(gd);
+		SWTUtil.setButtonDimensionHint(fEnableAssertionsCheckBox);
+		setAssertionCheckBoxSelection(AssertionVMArg.getEnableAssertionsPreference());
+	}
+	
 	/**
-	 * Create a group to contain the step filter related widgetry
+	 * Programatic access to enable assertions checkbox
+	 * @return boolean indicating check box selected or not
+	 */
+	public boolean getAssertionCheckBoxSelection() {
+		return fEnableAssertionsCheckBox.getSelection();
+	}
+
+	public void setAssertionCheckBoxSelection(boolean selected) {
+		fEnableAssertionsCheckBox.setSelection(selected);
+	}
+
+	/*
+	 * Create a group to contain the step filter related widgets
 	 */
 	private void createStackFilterPreferences(Composite composite) {
+		fFilterViewerLabel= new Label(composite, SWT.SINGLE | SWT.LEFT);
+		fFilterViewerLabel.setText(JUnitMessages.JUnitPreferencePage_filter_label);
+		
 		Composite container= new Composite(composite, SWT.NONE);
 		GridLayout layout= new GridLayout();
 		layout.numColumns= 2;
@@ -388,7 +424,7 @@ public class JUnitPreferencePage extends PreferencePage implements IWorkbenchPre
 		fAddFilterButton.setToolTipText(JUnitMessages.JUnitPreferencePage_addfilterbutton_tooltip); 
 		gd= new GridData(GridData.FILL_HORIZONTAL | GridData.VERTICAL_ALIGN_BEGINNING);
 		fAddFilterButton.setLayoutData(gd);
-		SWTUtil.setButtonDimensionHint(fAddFilterButton);
+		LayoutUtil.setButtonDimensionHint(fAddFilterButton);
 		fAddFilterButton.addListener(SWT.Selection, new Listener() {
 			public void handleEvent(Event e) {
 				editFilter();
@@ -400,7 +436,7 @@ public class JUnitPreferencePage extends PreferencePage implements IWorkbenchPre
 		fAddTypeButton.setToolTipText(JUnitMessages.JUnitPreferencePage_addtypebutton_tooltip); 
 		gd= getButtonGridData(fAddTypeButton);
 		fAddTypeButton.setLayoutData(gd);
-		SWTUtil.setButtonDimensionHint(fAddTypeButton);
+		LayoutUtil.setButtonDimensionHint(fAddTypeButton);
 		fAddTypeButton.addListener(SWT.Selection, new Listener() {
 			public void handleEvent(Event e) {
 				addType();
@@ -462,7 +498,6 @@ public class JUnitPreferencePage extends PreferencePage implements IWorkbenchPre
 		GridData gd= new GridData(GridData.FILL_HORIZONTAL | GridData.VERTICAL_ALIGN_BEGINNING);
 		int widthHint= convertHorizontalDLUsToPixels(IDialogConstants.BUTTON_WIDTH);
 		gd.widthHint= Math.max(widthHint, button.computeSize(SWT.DEFAULT, SWT.DEFAULT, true).x);
-		gd.heightHint= convertVerticalDLUsToPixels(IDialogConstants.BUTTON_HEIGHT);
 		return gd;
 	}
 
@@ -572,8 +607,8 @@ public class JUnitPreferencePage extends PreferencePage implements IWorkbenchPre
 		cleanupEditor();
 	}
 
-	/**
-	 * Cleanup all widgetry & resources used by the in-place editing
+	/*
+	 * Cleanup all widgets & resources used by the in-place editing
 	 */
 	private void cleanupEditor() {
 		if (fEditorText == null)
@@ -590,7 +625,7 @@ public class JUnitPreferencePage extends PreferencePage implements IWorkbenchPre
 		fStackFilterContentProvider.removeFilters(new Object[] { fNewStackFilter });
 	}
 
-	/**
+	/*
 	 * A valid step filter is simply one that is a valid Java identifier.
 	 * and, as defined in the JDI spec, the regular expressions used for
 	 * step filtering must be limited to exact matches or patterns that
@@ -650,20 +685,12 @@ public class JUnitPreferencePage extends PreferencePage implements IWorkbenchPre
 
 	private void addPackage() {
 		Shell shell= getShell();
-		ElementListSelectionDialog dialog= null;
-		try {
-			dialog= JUnitPlugin.createAllPackagesDialog(shell, null, true);
-		} catch (JavaModelException jme) {
-			String title= JUnitMessages.JUnitPreferencePage_addpackagedialog_title; 
-			String message= JUnitMessages.JUnitPreferencePage_addpackagedialog_error_message; 
-			ExceptionHandler.handle(jme, shell, title, message);
-			return;
-		}
-
+		IProgressService context= PlatformUI.getWorkbench().getProgressService();
+		IJavaSearchScope createWorkspaceScope= SearchEngine.createWorkspaceScope();
+		SelectionDialog dialog= JavaUI.createPackageDialog(shell, context, createWorkspaceScope, true, true, ""); //$NON-NLS-1$
 		dialog.setTitle(JUnitMessages.JUnitPreferencePage_addpackagedialog_title); 
 		dialog.setMessage(JUnitMessages.JUnitPreferencePage_addpackagedialog_message); 
-		dialog.setMultipleSelection(true);
-		if (dialog.open() == IDialogConstants.CANCEL_ID)
+		if (dialog.open() != Window.OK)
 			return;
 
 		Object[] packages= dialog.getResult();
@@ -696,7 +723,7 @@ public class JUnitPreferencePage extends PreferencePage implements IWorkbenchPre
 	}
 	
 	public boolean performOk() {
-		IPreferenceStore store= getPreferenceStore();
+		AssertionVMArg.setEnableAssertionsPreference(getAssertionCheckBoxSelection());
 		fStackFilterContentProvider.saveFilters();
 		return true;
 	}
@@ -707,7 +734,7 @@ public class JUnitPreferencePage extends PreferencePage implements IWorkbenchPre
 	}
 
 	private void setDefaultValues() {
-		IPreferenceStore store= getPreferenceStore();
+		fEnableAssertionsCheckBox.setSelection(false);
 		fStackFilterContentProvider.setDefaults();
 	}
 
@@ -755,10 +782,8 @@ public class JUnitPreferencePage extends PreferencePage implements IWorkbenchPre
 		store.setValue(JUnitPreferencesConstants.DO_FILTER_STACK, filter);
 	}
 
-	/**
+	/*
 	 * Parses the comma separated string into an array of strings
-	 * 
-	 * @return list
 	 */
 	private static String[] parseList(String listString) {
 		List list= new ArrayList(10);
