@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -84,8 +84,10 @@ public interface IJavaEditorActionDefinitionIds extends ITextEditorActionDefinit
 	/**
 	 * Action definition ID of the edit -> correction assist proposal action
 	 * (value <code>"org.eclipse.jdt.ui.edit.text.java.correction.assist.proposals"</code>).
+	 * 
+	 * @deprecated As of 3.2, replaced by {@link ITextEditorActionDefinitionIds#QUICK_ASSIST}
 	 */
-	public static final String CORRECTION_ASSIST_PROPOSALS= "org.eclipse.jdt.ui.edit.text.java.correction.assist.proposals"; //$NON-NLS-1$
+	public static final String CORRECTION_ASSIST_PROPOSALS= QUICK_ASSIST;
 		
 
 	/**
@@ -231,6 +233,13 @@ public interface IJavaEditorActionDefinitionIds extends ITextEditorActionDefinit
 	 */
 	public static final String GENERATE_CONSTRUCTOR_USING_FIELDS= "org.eclipse.jdt.ui.edit.text.java.generate.constructor.using.fields"; //$NON-NLS-1$
 
+	/**
+	 * Action definition ID of the source ->generate hashcode() and equals() action
+	 * (value <code>"org.eclipse.jdt.ui.edit.text.java.generate.hashcode.equals"</code>).
+	 * @since 3.2
+	 */
+	public static final String GENERATE_HASHCODE_EQUALS= "org.eclipse.jdt.ui.edit.text.java.generate.hashcode.equals"; //$NON-NLS-1$
+
 	/**
 	 * Action definition ID of the source -> generate setter/getter action
 	 * (value <code>"org.eclipse.jdt.ui.edit.text.java.create.getter.setter"</code>).
@@ -361,6 +370,22 @@ public interface IJavaEditorActionDefinitionIds extends ITextEditorActionDefinit
 	 * @since 2.1
 	 */
 	public static final String INLINE= "org.eclipse.jdt.ui.edit.text.java.inline"; //$NON-NLS-1$
+	
+	/**
+	 * Action definition ID of the refactor -> replace invocations action
+	 * (value <code>"org.eclipse.jdt.ui.edit.text.java.replace.invocations"</code>).
+	 *
+	 * @since 3.2
+	 */
+	public static final String REPLACE_INVOCATIONS= "org.eclipse.jdt.ui.edit.text.java.replace.invocations"; //$NON-NLS-1$
+	
+	/**
+	 * Action definition ID of the refactor -> introduce indirection action
+	 * (value <code>"org.eclipse.jdt.ui.edit.text.java.create.indirection"</code>).
+	 *
+	 * @since 3.2
+	 */
+	public static final String INTRODUCE_INDIRECTION= "org.eclipse.jdt.ui.edit.text.java.introduce.indirection"; //$NON-NLS-1$
 
 	/**
 	 * Action definition ID of the refactor -> extract interface action
@@ -631,4 +656,25 @@ public interface IJavaEditorActionDefinitionIds extends ITextEditorActionDefinit
 	 * @since 3.0
 	 */
 	public static final String TOGGLE_MARK_OCCURRENCES= "org.eclipse.jdt.ui.edit.text.java.toggleMarkOccurrences"; //$NON-NLS-1$
+
+	/**
+	 * Action definition id of the collapse members action
+	 * (value: <code>"org.eclipse.jdt.ui.edit.text.java.folding.collapseMembers"</code>).
+	 * @since 3.2
+	 */
+	public static final String FOLDING_COLLAPSE_MEMBERS= "org.eclipse.jdt.ui.edit.text.java.folding.collapseMembers"; //$NON-NLS-1$
+
+	/**
+	 * Action definition id of the collapse comments action
+	 * (value: <code>"org.eclipse.jdt.ui.edit.text.java.folding.collapseComments"</code>).
+	 * @since 3.2
+	 */
+	public static final String FOLDING_COLLAPSE_COMMENTS= "org.eclipse.jdt.ui.edit.text.java.folding.collapseComments"; //$NON-NLS-1$
+
+	/**
+	 * Action definition id of the code clean up action
+	 * (value: <code>"org.eclipse.jdt.ui.edit.text.java.clean.up"</code>).
+	 * @since 3.2
+	 */
+	public static final String CLEAN_UP= "org.eclipse.jdt.ui.edit.text.java.clean.up"; //$NON-NLS-1$
 }
