@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -12,15 +12,19 @@ package org.eclipse.jdt.ui.refactoring;
 
 import java.lang.reflect.InvocationTargetException;
 
-import org.eclipse.swt.widgets.Shell;
-
 import org.eclipse.core.runtime.CoreException;
 import org.eclipse.core.runtime.IStatus;
 import org.eclipse.core.runtime.Status;
 
+import org.eclipse.swt.widgets.Shell;
+
 import org.eclipse.jface.dialogs.MessageDialog;
 import org.eclipse.jface.operation.IRunnableContext;
 
+import org.eclipse.ltk.core.refactoring.RefactoringCore;
+import org.eclipse.ltk.core.refactoring.RefactoringStatus;
+import org.eclipse.ltk.core.refactoring.participants.RenameRefactoring;
+
 import org.eclipse.jdt.core.ICompilationUnit;
 import org.eclipse.jdt.core.IField;
 import org.eclipse.jdt.core.IJavaProject;
@@ -32,6 +36,7 @@ import org.eclipse.jdt.core.IType;
 import org.eclipse.jdt.core.ITypeParameter;
 
 import org.eclipse.jdt.internal.corext.refactoring.rename.JavaRenameProcessor;
+import org.eclipse.jdt.internal.corext.refactoring.rename.JavaRenameRefactoring;
 import org.eclipse.jdt.internal.corext.refactoring.rename.MethodChecks;
 import org.eclipse.jdt.internal.corext.refactoring.rename.RenameCompilationUnitProcessor;
 import org.eclipse.jdt.internal.corext.refactoring.rename.RenameEnumConstProcessor;
@@ -55,11 +60,6 @@ import org.eclipse.jdt.internal.ui.refactoring.RefactoringExecutionHelper;
 import org.eclipse.jdt.internal.ui.refactoring.UserInterfaceStarter;
 import org.eclipse.jdt.internal.ui.refactoring.reorg.RenameUserInterfaceManager;
 
-import org.eclipse.ltk.core.refactoring.RefactoringCore;
-import org.eclipse.ltk.core.refactoring.RefactoringStatus;
-import org.eclipse.ltk.core.refactoring.RefactoringStatusEntry;
-import org.eclipse.ltk.core.refactoring.participants.RenameRefactoring;
-
 /**
  * Central access point to execute rename refactorings.
  * <p>
@@ -92,7 +92,7 @@ public class RenameSupport {
 	public IStatus preCheck() throws CoreException {
 		ensureChecked();
 		if (fPreCheckStatus.hasFatalError())
-			return asStatus(fPreCheckStatus.getEntryMatchingSeverity(RefactoringStatus.FATAL));
+			return fPreCheckStatus.getEntryMatchingSeverity(RefactoringStatus.FATAL).toStatus();
 		else
 			return new Status(IStatus.OK, JavaPlugin.getPluginId(), 0, "", null); //$NON-NLS-1$
 	}
@@ -149,7 +149,7 @@ public class RenameSupport {
 			getJavaRenameProcessor().needsSavedEditors(),
 			parent,
 			context);
-		helper.perform();
+		helper.perform(false);
 	}
 	
 	/** Flag indication that no additional update is to be performed. */
@@ -188,7 +188,7 @@ public class RenameSupport {
 	public static final int UPDATE_SETTER_METHOD= 1 << 5;
 
 	private RenameSupport(JavaRenameProcessor processor, String newName, int flags) throws CoreException {
-		fRefactoring= new RenameRefactoring(processor);
+		fRefactoring= new JavaRenameRefactoring(processor);
 		initialize(fRefactoring, newName, flags);
 	}
 
@@ -417,27 +417,4 @@ public class RenameSupport {
 		String message= status.getMessageMatchingSeverity(RefactoringStatus.FATAL);
 		MessageDialog.openInformation(parent, JavaUIMessages.RenameSupport_dialog_title, message); 
 	}
-	
-	private static IStatus asStatus(RefactoringStatusEntry entry) {
-		int statusSeverity= IStatus.ERROR;
-		switch (entry.getSeverity()) {
-			case RefactoringStatus.OK :
-				statusSeverity= IStatus.OK;
-				break;
-			case RefactoringStatus.INFO :
-				statusSeverity= IStatus.INFO;
-				break;
-			case RefactoringStatus.WARNING :
-			case RefactoringStatus.ERROR :
-				statusSeverity= IStatus.WARNING;
-				break;
-		}
-		String pluginId= entry.getPluginId();
-		int code= entry.getCode();
-		if (pluginId == null) {
-			pluginId= JavaPlugin.getPluginId();
-			code= IStatus.ERROR;
-		}
-		return new Status(statusSeverity, pluginId, code, entry.getMessage(), null);
-	}
 }
