@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -102,12 +102,22 @@ public abstract class NewContainerWizardPage extends NewElementWizardPage {
 		
 		fContainerDialogField= new StringButtonDialogField(adapter);
 		fContainerDialogField.setDialogFieldListener(adapter);
-		fContainerDialogField.setLabelText(NewWizardMessages.NewContainerWizardPage_container_label); 
+		fContainerDialogField.setLabelText(getContainerLabel()); 
 		fContainerDialogField.setButtonLabel(NewWizardMessages.NewContainerWizardPage_container_button); 
 		
 		fContainerStatus= new StatusInfo();
 		fCurrRoot= null;
 	}
+
+	/**
+	 * Returns the label that is used for the container input field.
+	 * 
+	 * @return the label that is used for the container input field.
+	 * @since 3.2
+	 */
+	protected String getContainerLabel() {
+		return NewWizardMessages.NewContainerWizardPage_container_label;
+	}
 			
 	/**
 	 * Initializes the source folder field with a valid package fragment root.
@@ -203,9 +213,9 @@ public abstract class NewContainerWizardPage extends NewElementWizardPage {
 	}
 	
 	/**
-	 * Returns the test selection of the current editor. <code>null</code> is returned
+	 * Returns the text selection of the current editor. <code>null</code> is returned
 	 * when the current editor does not have focus or does not return a text selection.
-	 * @return Returns the test selection of the current editor or <code>null</code>.
+	 * @return Returns the text selection of the current editor or <code>null</code>.
      *
      * @since 3.0 
 	 */
@@ -276,8 +286,7 @@ public abstract class NewContainerWizardPage extends NewElementWizardPage {
 	
 	private void containerChangeControlPressed(DialogField field) {
 		// take the current jproject as init element of the dialog
-		IPackageFragmentRoot root= getPackageFragmentRoot();
-		root= chooseSourceContainer(root);
+		IPackageFragmentRoot root= chooseContainer();
 		if (root != null) {
 			setPackageFragmentRoot(root, true);
 		}
@@ -424,7 +433,19 @@ public abstract class NewContainerWizardPage extends NewElementWizardPage {
 		
 	// ------------- choose source container dialog
 	
-	private IPackageFragmentRoot chooseSourceContainer(IJavaElement initElement) {
+	/**
+	 * Opens a selection dialog that allows to select a source container. 
+	 * 
+	 * @return returns the selected package fragment root  or <code>null</code> if the dialog has been canceled.
+	 * The caller typically sets the result to the container input field.
+	 * <p>
+	 * Clients can override this method if they want to offer a different dialog.
+	 * </p>
+	 * 
+	 * @since 3.2
+	 */
+	protected IPackageFragmentRoot chooseContainer() {
+		IJavaElement initElement= getPackageFragmentRoot();
 		Class[] acceptedClasses= new Class[] { IPackageFragmentRoot.class, IJavaProject.class };
 		TypedElementSelectionValidator validator= new TypedElementSelectionValidator(acceptedClasses, false) {
 			public boolean isSelectedValid(Object element) {
@@ -469,6 +490,7 @@ public abstract class NewContainerWizardPage extends NewElementWizardPage {
 		dialog.addFilter(filter);
 		dialog.setInput(JavaCore.create(fWorkspaceRoot));
 		dialog.setInitialSelection(initElement);
+		dialog.setHelpAvailable(false);
 		
 		if (dialog.open() == Window.OK) {
 			Object element= dialog.getFirstResult();
