@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2004 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -10,9 +10,9 @@
  *******************************************************************************/
 package org.eclipse.pde.internal.core.feature;
 
-import org.eclipse.core.runtime.*;
-import org.eclipse.pde.core.*;
-import org.w3c.dom.*;
+import org.eclipse.core.runtime.CoreException;
+import org.eclipse.pde.core.IIdentifiable;
+import org.w3c.dom.Node;
 
 public class IdentifiableObject extends FeatureObject implements IIdentifiable {
 	private static final long serialVersionUID = 1L;
