@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2005 IBM Corporation and others.
+ * Copyright (c) 2005, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -11,52 +11,68 @@
 
 package org.eclipse.pde.internal.ui.editor.plugin;
 
+import java.util.ArrayList;
+import java.util.ListIterator;
 import java.util.Vector;
 
 import org.eclipse.core.resources.IProject;
 import org.eclipse.core.runtime.CoreException;
+import org.eclipse.core.runtime.IPath;
+import org.eclipse.core.runtime.Path;
+import org.eclipse.jdt.core.IJavaProject;
 import org.eclipse.jdt.core.IPackageFragment;
+import org.eclipse.jdt.core.IPackageFragmentRoot;
 import org.eclipse.jdt.core.JavaCore;
+import org.eclipse.jdt.core.JavaModelException;
 import org.eclipse.jdt.ui.ISharedImages;
 import org.eclipse.jdt.ui.JavaElementLabelProvider;
 import org.eclipse.jdt.ui.JavaUI;
+import org.eclipse.jdt.ui.actions.FindReferencesAction;
+import org.eclipse.jdt.ui.actions.ShowInPackageViewAction;
 import org.eclipse.jface.action.Action;
 import org.eclipse.jface.action.IMenuManager;
 import org.eclipse.jface.action.Separator;
 import org.eclipse.jface.viewers.ILabelProvider;
+import org.eclipse.jface.viewers.ISelection;
 import org.eclipse.jface.viewers.IStructuredSelection;
-import org.eclipse.jface.viewers.ITableLabelProvider;
 import org.eclipse.jface.viewers.LabelProvider;
 import org.eclipse.jface.viewers.StructuredSelection;
 import org.eclipse.jface.viewers.TableViewer;
 import org.eclipse.jface.viewers.Viewer;
 import org.eclipse.jface.viewers.ViewerSorter;
+import org.eclipse.jface.window.Window;
 import org.eclipse.osgi.service.resolver.ExportPackageDescription;
-import org.eclipse.osgi.util.NLS;
+import org.eclipse.pde.core.IBaseModel;
 import org.eclipse.pde.core.IModelChangedEvent;
 import org.eclipse.pde.core.IModelChangedListener;
+import org.eclipse.pde.core.plugin.IPluginLibrary;
 import org.eclipse.pde.core.plugin.IPluginModelBase;
+import org.eclipse.pde.internal.core.ClasspathUtilCore;
 import org.eclipse.pde.internal.core.ICoreConstants;
+import org.eclipse.pde.internal.core.SearchablePluginsManager;
 import org.eclipse.pde.internal.core.bundle.BundlePluginBase;
 import org.eclipse.pde.internal.core.ibundle.IBundle;
 import org.eclipse.pde.internal.core.ibundle.IBundleModel;
+import org.eclipse.pde.internal.core.text.bundle.Bundle;
+import org.eclipse.pde.internal.core.text.bundle.ExportPackageHeader;
+import org.eclipse.pde.internal.core.text.bundle.ExportPackageObject;
+import org.eclipse.pde.internal.core.text.bundle.PackageObject;
 import org.eclipse.pde.internal.ui.PDEPlugin;
 import org.eclipse.pde.internal.ui.PDEUIMessages;
 import org.eclipse.pde.internal.ui.editor.PDEFormPage;
 import org.eclipse.pde.internal.ui.editor.TableSection;
 import org.eclipse.pde.internal.ui.editor.context.InputContextManager;
 import org.eclipse.pde.internal.ui.elements.DefaultTableProvider;
-import org.eclipse.pde.internal.ui.model.bundle.Bundle;
-import org.eclipse.pde.internal.ui.model.bundle.ExportPackageHeader;
-import org.eclipse.pde.internal.ui.model.bundle.ExportPackageObject;
-import org.eclipse.pde.internal.ui.model.bundle.PackageObject;
 import org.eclipse.pde.internal.ui.parts.TablePart;
 import org.eclipse.pde.internal.ui.util.SWTUtil;
+import org.eclipse.search.ui.NewSearchUI;
 import org.eclipse.swt.SWT;
 import org.eclipse.swt.graphics.Image;
 import org.eclipse.swt.layout.GridData;
 import org.eclipse.swt.widgets.Composite;
 import org.eclipse.swt.widgets.Table;
+import org.eclipse.ui.IViewPart;
+import org.eclipse.ui.PartInitException;
 import org.eclipse.ui.actions.ActionFactory;
 import org.eclipse.ui.forms.widgets.FormToolkit;
 import org.eclipse.ui.forms.widgets.Section;
@@ -78,17 +94,6 @@ public class ExportPackageSection extends TableSection implements IModelChangedL
             return fHeader == null ? new Object[0] : fHeader.getPackages();
 		}
     }
-    
-	class ExportPackageLabelProvider extends LabelProvider implements
-			ITableLabelProvider {
-		public String getColumnText(Object obj, int index) {
-			return obj.toString();
-		}
-
-		public Image getColumnImage(Object obj, int index) {
-			return JavaUI.getSharedImages().getImage(ISharedImages.IMG_OBJS_PACKAGE);
-		}
-	}
 
 	class ExportPackageDialogLabelProvider extends LabelProvider {
 		public Image getImage(Object element) {
@@ -111,6 +116,7 @@ public class ExportPackageSection extends TableSection implements IModelChangedL
     private TableViewer fPackageViewer;
 
     private Action fAddAction;
+    private Action fGoToAction;
     private Action fRemoveAction;
     private Action fPropertiesAction;
     private ExportPackageHeader fHeader;
@@ -136,14 +142,17 @@ public class ExportPackageSection extends TableSection implements IModelChangedL
 	 */
 	protected void createClient(Section section, FormToolkit toolkit) {
         section.setText(PDEUIMessages.ExportPackageSection_title); 
-        section.setDescription(NLS.bind(PDEUIMessages.ExportPackageSection_desc, isFragment() ? "fragment" : "plug-in"));  //$NON-NLS-1$ //$NON-NLS-2$
+        if (isFragment())
+			section.setDescription(PDEUIMessages.ExportPackageSection_descFragment);
+		else
+			section.setDescription(PDEUIMessages.ExportPackageSection_desc);
 
         Composite container = createClientContainer(section, 2, toolkit);
 		createViewerPartControl(container, SWT.MULTI, 2, toolkit);
 		TablePart tablePart = getTablePart();
 		fPackageViewer = tablePart.getTableViewer();
 		fPackageViewer.setContentProvider(new ExportPackageContentProvider());
-		fPackageViewer.setLabelProvider(new ExportPackageLabelProvider());
+		fPackageViewer.setLabelProvider(PDEPlugin.getDefault().getLabelProvider());
 		fPackageViewer.setSorter(new ViewerSorter() {
             public int compare(Viewer viewer, Object e1, Object e2) {
                 String s1 = e1.toString();
@@ -158,8 +167,10 @@ public class ExportPackageSection extends TableSection implements IModelChangedL
 		toolkit.paintBordersFor(container);
 		section.setClient(container);
         GridData gd = new GridData(GridData.FILL_BOTH);
-        gd.verticalSpan = 2;
-        gd.minimumWidth = 300;
+        if (((ManifestEditor)getPage().getEditor()).isEquinox()) {
+	        gd.verticalSpan = 2;
+	        gd.minimumWidth = 300;
+        }
 		section.setLayoutData(gd);
         makeActions();
         
@@ -233,7 +244,117 @@ public class ExportPackageSection extends TableSection implements IModelChangedL
 	}
     
     protected void handleDoubleClick(IStructuredSelection selection) {
-        handleOpenProperties();
+        handleGoToPackage(selection);
+    }
+    
+    private IPackageFragment getPackageFragment(ISelection sel) {
+    	if (sel instanceof IStructuredSelection) {
+    		IStructuredSelection selection = (IStructuredSelection) sel;
+    		if (selection.size() != 1) 
+    			return null;
+    		PackageObject importObject = (PackageObject)selection.getFirstElement();
+    		String packageName = importObject.getName();
+
+    		IBaseModel base = getPage().getModel();
+    		if (!(base instanceof IPluginModelBase)) 
+    			return null;
+    		IPluginModelBase model = (IPluginModelBase)base;
+    		if (model.getUnderlyingResource() == null) 
+    			return getExternalPackageFragment(packageName, model);
+    		IJavaProject jp = JavaCore.create(getPage().getPDEEditor().getCommonProject());
+    		if (jp != null)
+    			try {
+    				IPackageFragmentRoot[] roots = jp.getPackageFragmentRoots();
+    				for (int i = 0; i < roots.length; i++) {
+    					IPackageFragment frag = roots[i].getPackageFragment(packageName);
+    					if (frag.exists()) {
+    						return frag;
+    					}
+    				}
+    			} catch (JavaModelException e) {
+    			}
+    	}
+    	return null;
+    }
+    
+    private IPackageFragment getExternalPackageFragment(String packageName, IPluginModelBase model) {
+    	try {
+    		IProject proj = PDEPlugin.getWorkspace().getRoot().getProject(SearchablePluginsManager.PROXY_PROJECT_NAME);
+    		if (proj == null)
+    			return searchWorkspaceForPackage(packageName, model);
+    		IJavaProject jp = JavaCore.create(proj);
+    		IPath path = new Path(model.getInstallLocation());
+    		// if model is in jar form
+    		if (!path.toFile().isDirectory()) {
+    			IPackageFragmentRoot root = jp.findPackageFragmentRoot(path);
+    			if (root != null) {
+    				IPackageFragment frag = root.getPackageFragment(packageName);
+    				if (frag.exists())
+    					return frag;
+    			}
+    			// else model is in folder form, try to find model's libraries on filesystem
+    		} else {
+    			IPluginLibrary[] libs = model.getPluginBase().getLibraries();
+    			for (int i = 0; i < libs.length; i++) {
+    				if (IPluginLibrary.RESOURCE.equals(libs[i].getType()))
+    					continue;
+    				String libName = ClasspathUtilCore.expandLibraryName(libs[i].getName());
+    				IPackageFragmentRoot root = jp.findPackageFragmentRoot(path.append(libName));
+    				if (root != null) {
+    					IPackageFragment frag = root.getPackageFragment(packageName);
+    					if (frag.exists())
+    						return frag;
+    				}
+    			}
+    		}
+    	} catch (JavaModelException e){
+    	}
+    	return searchWorkspaceForPackage(packageName, model);
+    }
+    
+    private IPackageFragment searchWorkspaceForPackage(String packageName, IPluginModelBase base) {
+    	IPluginLibrary[] libs = base.getPluginBase().getLibraries();
+    	ArrayList libPaths = new ArrayList();
+    	IPath path = new Path(base.getInstallLocation());
+    	if (libs.length == 0) {
+    		libPaths.add(path);
+    	}
+		for (int i = 0; i < libs.length; i++) {
+			if (IPluginLibrary.RESOURCE.equals(libs[i].getType()))
+				continue;
+			String libName = ClasspathUtilCore.expandLibraryName(libs[i].getName());
+			libPaths.add(path.append(libName));
+		}
+		IProject[] projects = PDEPlugin.getWorkspace().getRoot().getProjects();
+		for (int i = 0; i < projects.length; i++) {
+			try {
+				if(!projects[i].hasNature(JavaCore.NATURE_ID) || !projects[i].isOpen())
+					continue;
+				IJavaProject jp = JavaCore.create(projects[i]);
+				ListIterator li = libPaths.listIterator();
+				while (li.hasNext()) {
+					IPackageFragmentRoot root = jp.findPackageFragmentRoot((IPath)li.next());
+					if (root != null) {
+						IPackageFragment frag = root.getPackageFragment(packageName);
+						if (frag.exists())
+							return frag;
+					}
+				}
+			} catch (CoreException e) {
+			}
+		}
+		return null;
+    }
+    
+    private void handleGoToPackage(ISelection selection) {
+    	IPackageFragment frag = getPackageFragment(selection);
+    	if (frag != null)
+    		try {
+    			IViewPart part = PDEPlugin.getActivePage().showView(JavaUI.ID_PACKAGES);
+    			ShowInPackageViewAction action = new ShowInPackageViewAction(part.getSite());
+    			action.run(frag);
+    		} catch (PartInitException e) {
+    		}
     }
 
     protected void buttonSelected(int index) {
@@ -259,7 +380,7 @@ public class ExportPackageSection extends TableSection implements IModelChangedL
         	dialog.setTitle(((ExportPackageObject)selected[0]).getName());
         else 
         	dialog.setTitle(PDEUIMessages.ExportPackageSection_props);
-        if (dialog.open() == DependencyPropertiesDialog.OK && isEditable()) {
+        if (dialog.open() == Window.OK && isEditable()) {
         	String newVersion = dialog.getVersion();
         	for (int i = 0; i < selected.length; i++) {
         		ExportPackageObject object = (ExportPackageObject)selected[i];
@@ -286,8 +407,9 @@ public class ExportPackageSection extends TableSection implements IModelChangedL
                         PDEPlugin.getActiveWorkbenchShell(),
                         labelProvider, 
                         JavaCore.create(project), 
-                        fHeader == null ? new Vector() : fHeader.getPackageNames());
-                if (dialog.open() == PackageSelectionDialog.OK) {
+                        fHeader == null ? new Vector() : fHeader.getPackageNames(),
+                        "true".equals(getBundle().getHeader(ICoreConstants.ECLIPSE_JREBUNDLE))); //$NON-NLS-1$
+                if (dialog.open() == Window.OK) {
                     Object[] selected = dialog.getResult();
                     if (fHeader != null) {
                         for (int i = 0; i < selected.length; i++) {
@@ -356,13 +478,18 @@ public class ExportPackageSection extends TableSection implements IModelChangedL
 	}
     
     private void makeActions() {
-        fAddAction = new Action(PDEUIMessages.RequiresSection_add) { //$NON-NLS-1$
+        fAddAction = new Action(PDEUIMessages.RequiresSection_add) { 
             public void run() {
                 handleAdd();
             }
         };
         fAddAction.setEnabled(isEditable());
-        fRemoveAction = new Action(PDEUIMessages.RequiresSection_delete) { //$NON-NLS-1$
+        fGoToAction = new Action(PDEUIMessages.ImportPackageSection_goToPackage) {
+        	public void run() {
+        		handleGoToPackage(fPackageViewer.getSelection());
+        	}
+        };
+        fRemoveAction = new Action(PDEUIMessages.RequiresSection_delete) { 
             public void run() {
                 handleRemove();
             }
@@ -377,15 +504,39 @@ public class ExportPackageSection extends TableSection implements IModelChangedL
     }
 
 	protected void fillContextMenu(IMenuManager manager) {
+		ISelection selection = fPackageViewer.getSelection();
         manager.add(fAddAction);
+        boolean singleSelection = selection instanceof IStructuredSelection && 
+			((IStructuredSelection)selection).size() == 1;
+        if (singleSelection)
+        	manager.add(fGoToAction);
         manager.add(new Separator());
-        manager.add(fRemoveAction);
+        if (!selection.isEmpty())
+        	manager.add(fRemoveAction);
 		getPage().getPDEEditor().getContributor().contextMenuAboutToShow(manager);
+		if (singleSelection)
+			manager.add(new Action(PDEUIMessages.ExportPackageSection_findReferences) {
+				public void run() {
+					doSearch(fPackageViewer.getSelection());
+				}
+			});
         if (shouldEnableProperties(((IStructuredSelection)fPackageViewer.getSelection()).toArray())) {
             manager.add(new Separator());
             manager.add(fPropertiesAction);
         }
 	}
+	
+	private void doSearch(ISelection sel) {
+		IPackageFragment frag = getPackageFragment(sel);
+		if (frag != null) {
+			FindReferencesAction action = new FindReferencesAction(getPage().getEditorSite());
+			action.run(frag);
+		} else if (sel instanceof IStructuredSelection)  {
+    		IStructuredSelection selection = (IStructuredSelection) sel;
+    		PackageObject exportObject = (PackageObject)selection.getFirstElement();
+    		NewSearchUI.runQueryInBackground(new BlankQuery(exportObject));
+		}
+	}
 
     private BundleInputContext getBundleContext() {
         InputContextManager manager = getPage().getPDEEditor().getContextManager();
