@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -18,23 +18,31 @@ import org.eclipse.core.resources.IFile;
 import org.eclipse.core.resources.IProject;
 import org.eclipse.core.resources.IResource;
 import org.eclipse.core.resources.IStorage;
+import org.eclipse.core.resources.ProjectScope;
 import org.eclipse.core.runtime.CoreException;
+import org.eclipse.core.runtime.preferences.IEclipsePreferences;
 import org.eclipse.jface.preference.IPreferenceStore;
-import org.eclipse.jface.viewers.ISelection;
+import org.eclipse.pde.core.IBaseModel;
+import org.eclipse.pde.core.IIdentifiable;
 import org.eclipse.pde.core.build.IBuild;
 import org.eclipse.pde.core.build.IBuildEntry;
 import org.eclipse.pde.core.build.IBuildModel;
-import org.eclipse.pde.core.plugin.IFragment;
 import org.eclipse.pde.core.plugin.IFragmentModel;
 import org.eclipse.pde.core.plugin.IPluginBase;
 import org.eclipse.pde.core.plugin.IPluginExtension;
 import org.eclipse.pde.core.plugin.IPluginExtensionPoint;
 import org.eclipse.pde.core.plugin.IPluginModelBase;
+import org.eclipse.pde.core.plugin.IPluginObject;
+import org.eclipse.pde.core.plugin.ISharedPluginModel;
+import org.eclipse.pde.internal.core.ICoreConstants;
 import org.eclipse.pde.internal.core.PDECore;
+import org.eclipse.pde.internal.core.TargetPlatform;
 import org.eclipse.pde.internal.core.build.IBuildObject;
+import org.eclipse.pde.internal.core.ibundle.IBundlePluginModelProvider;
 import org.eclipse.pde.internal.core.plugin.WorkspaceFragmentModel;
 import org.eclipse.pde.internal.core.plugin.WorkspacePluginModel;
 import org.eclipse.pde.internal.core.plugin.WorkspacePluginModelBase;
+import org.eclipse.pde.internal.core.util.CoreUtility;
 import org.eclipse.pde.internal.ui.IPDEUIConstants;
 import org.eclipse.pde.internal.ui.IPreferenceConstants;
 import org.eclipse.pde.internal.ui.PDEPlugin;
@@ -59,11 +67,94 @@ import org.eclipse.ui.IStorageEditorInput;
 import org.eclipse.ui.PartInitException;
 import org.eclipse.ui.part.FileEditorInput;
 import org.eclipse.ui.views.properties.IPropertySheetPage;
+import org.osgi.service.prefs.BackingStoreException;
 
 public class ManifestEditor extends MultiSourceEditor implements IShowEditorInput {
     
     private static int BUILD_INDEX = 5;
+	private static boolean SHOW_SOURCE;
+    private boolean fEquinox = true;
+    private boolean fShowExtensions = true;
+    private IEclipsePreferences fPrefs;
     
+	public static IEditorPart openPluginEditor(String id) {
+		return openPluginEditor(PDECore.getDefault().getModelManager().findModel(id));
+	}
+	
+	public static IEditorPart openPluginEditor(IPluginModelBase model) {
+		if (model == null) {
+			Display.getDefault().beep();
+			return null;
+		}
+		return openPluginEditor(model, false);
+	}
+	
+	public static IEditorPart openPluginEditor(IPluginModelBase model, boolean source) {
+		return open(model.getPluginBase(), source);
+	}
+	
+	public static IEditorPart open(Object object, boolean source) {
+		SHOW_SOURCE = source;
+		if (object instanceof IPluginObject) {
+			ISharedPluginModel model = ((IPluginObject)object).getModel();
+			if (model instanceof IBundlePluginModelProvider) 
+				model = ((IBundlePluginModelProvider)model).getBundlePluginModel();
+			if (model instanceof IPluginModelBase) {
+				String filename = ((IPluginModelBase)model).isFragmentModel() ? "fragment.xml" : "plugin.xml"; //$NON-NLS-1$ //$NON-NLS-2$
+				if (!(object instanceof IPluginExtension) && !(object instanceof IPluginExtensionPoint)) {
+					File file = new File(model.getInstallLocation());
+					if (file.isFile()) {
+						if (CoreUtility.jarContainsResource(file, "META-INF/MANIFEST.MF", false)) { //$NON-NLS-1$
+							filename = "META-INF/MANIFEST.MF"; //$NON-NLS-1$
+						} 
+					} else if (new File(file, "META-INF/MANIFEST.MF").exists()) { //$NON-NLS-1$
+						filename = "META-INF/MANIFEST.MF"; //$NON-NLS-1$
+					}
+				}
+				IResource resource = model.getUnderlyingResource();
+				if (resource == null) 
+					return openExternalPlugin(new File(model.getInstallLocation()), filename);
+				return openWorkspacePlugin(resource.getProject().getFile(filename));
+			}
+		}
+		return null;
+	}
+		
+	
+	private static IEditorPart openWorkspacePlugin(IFile pluginFile) {
+		return openEditor(new FileEditorInput(pluginFile));
+	}
+	
+	private static IEditorPart openExternalPlugin(File location, String filename) {
+		IEditorInput input = null;
+		if (location.isFile()) {
+			try {
+				ZipFile zipFile = new ZipFile(location);
+				if (zipFile.getEntry(filename) != null) 
+					input = new JarEntryEditorInput(new JarEntryFile(zipFile, filename));
+			} catch (IOException e) {
+			}			
+		} else {
+			File file = new File(location, filename);
+			if (file.exists())
+				input = new SystemFileEditorInput(file);
+		}
+		return openEditor(input);
+	}
+	
+	public static IEditorPart openEditor(IEditorInput input) {
+		if (input != null) {
+			try {
+				return PDEPlugin.getActivePage().openEditor(
+						input,
+						IPDEUIConstants.MANIFEST_EDITOR_ID);
+			} catch (PartInitException e) {
+				PDEPlugin.logException(e);
+			}
+		}
+		return null;		
+	}
+
 	protected void createResourceContexts(InputContextManager manager,
 			IFileEditorInput input) {
 		IFile file = input.getFile();
@@ -106,6 +197,12 @@ public class ManifestEditor extends MultiSourceEditor implements IShowEditorInpu
 		manager.monitorFile(project.getFile("plugin.xml")); //$NON-NLS-1$
 		manager.monitorFile(project.getFile("fragment.xml")); //$NON-NLS-1$
 		manager.monitorFile(buildFile);
+		
+		fPrefs = new ProjectScope(project).getNode(PDECore.PLUGIN_ID);
+		if (fPrefs != null) {
+			fShowExtensions = fPrefs.getBoolean(ICoreConstants.EXTENSIONS_PROPERTY, true);
+			fEquinox = fPrefs.getBoolean(ICoreConstants.EQUINOX_PROPERTY, true);
+		}
 	}
 	
 	protected InputContextManager createInputContextManager() {
@@ -115,40 +212,42 @@ public class ManifestEditor extends MultiSourceEditor implements IShowEditorInpu
 	}
 	
 	public void monitoredFileAdded(IFile file) {
+		if (fInputContextManager == null)
+			return;
 		String name = file.getName();
 		if (name.equalsIgnoreCase("MANIFEST.MF")) { //$NON-NLS-1$
-			if (!inputContextManager.hasContext(BundleInputContext.CONTEXT_ID)) {
+			if (!fInputContextManager.hasContext(BundleInputContext.CONTEXT_ID)) {
 				IEditorInput in = new FileEditorInput(file);
-				inputContextManager.putContext(in, new BundleInputContext(this, in, false));
+				fInputContextManager.putContext(in, new BundleInputContext(this, in, false));
 			}
 		}
 		else if (name.equalsIgnoreCase("plugin.xml")) { //$NON-NLS-1$
-			if (!inputContextManager.hasContext(PluginInputContext.CONTEXT_ID)) {
+			if (!fInputContextManager.hasContext(PluginInputContext.CONTEXT_ID)) {
 				IEditorInput in = new FileEditorInput(file);
-				inputContextManager.putContext(in, new PluginInputContext(this, in, false, false));
+				fInputContextManager.putContext(in, new PluginInputContext(this, in, false, false));
 			}
 		}
 		else if (name.equalsIgnoreCase("fragment.xml")) { //$NON-NLS-1$
-			if (!inputContextManager.hasContext(PluginInputContext.CONTEXT_ID)) {			
+			if (!fInputContextManager.hasContext(PluginInputContext.CONTEXT_ID)) {			
 				IEditorInput in = new FileEditorInput(file);
-				inputContextManager.putContext(in, new PluginInputContext(this, in, false, true));
+				fInputContextManager.putContext(in, new PluginInputContext(this, in, false, true));
 			}
 		}
 		else if (name.equalsIgnoreCase("build.properties")) { //$NON-NLS-1$
-			if (!inputContextManager.hasContext(BuildInputContext.CONTEXT_ID)) {			
+			if (!fInputContextManager.hasContext(BuildInputContext.CONTEXT_ID)) {			
 				IEditorInput in = new FileEditorInput(file);
-				inputContextManager.putContext(in, new BuildInputContext(this, in, false));
+				fInputContextManager.putContext(in, new BuildInputContext(this, in, false));
 			}
 		}
 	}
 	
 	public void ensurePluginContextPresence() {
-		if (inputContextManager.hasContext(PluginInputContext.CONTEXT_ID))
+		if (fInputContextManager.hasContext(PluginInputContext.CONTEXT_ID))
 			return;
-		IProject project = inputContextManager.getCommonProject();
-		String name = (inputContextManager.getAggregateModel() instanceof IFragmentModel)
+		IProject project = fInputContextManager.getCommonProject();
+		String name = (fInputContextManager.getAggregateModel() instanceof IFragmentModel)
 						? "fragment.xml" : "plugin.xml"; //$NON-NLS-1$ //$NON-NLS-2$
-		IFile file = project.getFile(name); //$NON-NLS-1$
+		IFile file = project.getFile(name); 
 		WorkspacePluginModelBase model;
 		if (name.equals("fragment.xml"))  //$NON-NLS-1$
 			model = new WorkspaceFragmentModel(file, false);
@@ -157,20 +256,20 @@ public class ManifestEditor extends MultiSourceEditor implements IShowEditorInpu
 		
 		IPluginBase pluginBase = model.getPluginBase(true);
 		try {
-			pluginBase.setSchemaVersion("3.0"); //$NON-NLS-1$
+			pluginBase.setSchemaVersion(TargetPlatform.getTargetVersion() < 3.2 ? "3.0" : "3.2"); //$NON-NLS-1$ //$NON-NLS-2$
 		}
 		catch (CoreException e) {
 		}
 		model.save();
 		IEditorInput in = new FileEditorInput(file);
-		inputContextManager.putContext(in, new PluginInputContext(this, in, false, false));
+		fInputContextManager.putContext(in, new PluginInputContext(this, in, false, false));
 
 		updateBuildProperties(name);
 	}
 	
     private void updateBuildProperties(String filename) {
         try {
-         InputContext context = inputContextManager.findContext(BuildInputContext.CONTEXT_ID);
+         InputContext context = fInputContextManager.findContext(BuildInputContext.CONTEXT_ID);
          if (context != null) {
                 IBuildModel buildModel = (IBuildModel)context.getModel();
                 IBuild build = buildModel.getBuild();
@@ -192,7 +291,7 @@ public class ManifestEditor extends MultiSourceEditor implements IShowEditorInpu
 		//file that just got removed under us.
 		return true;
 	}
-	public void contextAdded(InputContext context) {
+	public void editorContextAdded(InputContext context) {
 		addSourcePage(context.getId());
 		try {
 			if (context.getId().equals(BuildInputContext.CONTEXT_ID))
@@ -334,18 +433,15 @@ public class ManifestEditor extends MultiSourceEditor implements IShowEditorInpu
 		}
 	}
 	
-	public boolean canCopy(ISelection selection) {
-		return true;
-	}	
-
-	protected void addPages() {
+	protected void addEditorPages() {
 		try {
 			addPage(new OverviewPage(this));
 			addPage(new DependenciesPage(this));
 			addPage(new RuntimePage(this));
-			addPage(new ExtensionsPage(this));
-			addPage(new ExtensionPointsPage(this));
-			if (inputContextManager.hasContext(BuildInputContext.CONTEXT_ID))
+			if (showExtensionTabs()) {
+				addExtensionTabs();
+			}
+			if (fInputContextManager.hasContext(BuildInputContext.CONTEXT_ID))
 				addPage(new BuildPage(this));
 		} catch (PartInitException e) {
 			PDEPlugin.logException(e);
@@ -357,15 +453,22 @@ public class ManifestEditor extends MultiSourceEditor implements IShowEditorInpu
 
 
 	protected String computeInitialPageId() {
+		if (SHOW_SOURCE) {
+			SHOW_SOURCE = false;
+			InputContext primary = fInputContextManager.getPrimaryContext();
+			if (primary != null)
+				return primary.getId();
+		}
+		
 		String firstPageId = super.computeInitialPageId();
 		if (firstPageId == null) {
-			InputContext primary = inputContextManager.getPrimaryContext();
+			InputContext primary = fInputContextManager.getPrimaryContext();
 			if (primary == null)
 				return null;
 			if (BuildInputContext.CONTEXT_ID.equals(primary.getId()))
 				firstPageId = BuildPage.PAGE_ID;
 			else if (PluginInputContext.CONTEXT_ID.equals(primary.getId())) {
-				if (inputContextManager.hasContext(BundleInputContext.CONTEXT_ID))
+				if (fInputContextManager.hasContext(BundleInputContext.CONTEXT_ID))
 					firstPageId = ExtensionsPage.PAGE_ID;
 				else
 					firstPageId = OverviewPage.PAGE_ID;
@@ -392,6 +495,7 @@ public class ManifestEditor extends MultiSourceEditor implements IShowEditorInpu
 	protected ISortableContentOutlinePage createContentOutline() {
 		return new ManifestOutlinePage(this);
 	}
+	
 	public Object getAdapter(Class key) {
 		//No property sheet needed - block super
 		if (key.equals(IPropertySheetPage.class)) {
@@ -399,85 +503,7 @@ public class ManifestEditor extends MultiSourceEditor implements IShowEditorInpu
 		}
 		return super.getAdapter(key);
 	}
-	public static void openPluginEditor(String pluginId) {
-		openPluginEditor(pluginId, null);
-	}
 
-	public static void openPluginEditor(
-		String pluginId,
-		Object object) {
-		IPluginModelBase model = PDECore.getDefault().getModelManager().findModel(pluginId);
-		if (model != null) {
-			openPluginEditor(model.getPluginBase(), object);
-		} else {
-			Display.getCurrent().beep();
-		}
-	}
-
-	public static IEditorPart openPluginEditor(IPluginBase plugin) {
-		return openPluginEditor(plugin, null);
-	}
-	
-	public static IEditorPart openPluginEditor(
-		IPluginBase plugin,
-		Object object) {
-		IEditorPart editor = null;
-		IResource underlyingResource = plugin.getModel().getUnderlyingResource();
-		if (underlyingResource == null) {
-			editor = openExternalPlugin(plugin);
-		} else {
-			editor = openWorkspacePlugin((IFile) underlyingResource, plugin instanceof IFragment);
-		}
-		return editor;
-	}
-	
-	private static IEditorPart openWorkspacePlugin(IFile pluginFile, boolean fragment) {
-		String editorId = PDEPlugin.MANIFEST_EDITOR_ID;
-		try {
-			FileEditorInput input = new FileEditorInput(pluginFile);
-			return PDEPlugin.getActivePage().openEditor(
-				input,
-				editorId);
-		} catch (PartInitException e) {
-			PDEPlugin.logException(e);
-		}
-		return null;
-	}
-	private static ManifestEditor openExternalPlugin(IPluginBase pluginInfo) {
-		boolean isFragment = pluginInfo.getPluginModel().isFragmentModel();
-		String manifest = null;
-		
-		IStorageEditorInput input = null;
-		File pluginLocation = new File(pluginInfo.getModel().getInstallLocation());
-		if (pluginLocation.isFile() && pluginLocation.getName().endsWith(".jar")) { //$NON-NLS-1$
-			try {
-				ZipFile zipFile = new ZipFile(pluginLocation);
-				if (zipFile.getEntry("META-INF/MANIFEST.MF") != null) //$NON-NLS-1$
-					manifest = "META-INF/MANIFEST.MF"; //$NON-NLS-1$
-				else 
-					manifest = isFragment ? "fragment.xml" : "plugin.xml"; //$NON-NLS-1$ //$NON-NLS-2$
-				input = new JarEntryEditorInput(new JarEntryFile(zipFile, manifest));
-			} catch (Exception e) {
-			}			
-		}
-		
-		if (input == null) {
-			File file = new File(pluginLocation, "META-INF/MANIFEST.MF"); //$NON-NLS-1$
-			if (!file.exists())
-				file = new File(pluginLocation, isFragment ? "fragment.xml" : "plugin.xml"); //$NON-NLS-1$ //$NON-NLS-2$
-			if (file.exists())
-				input = new SystemFileEditorInput(file);
-		}
-		try {
-			if (input != null)
-				return (ManifestEditor) PDEPlugin.getActivePage().openEditor(
-					input,
-					IPDEUIConstants.MANIFEST_EDITOR_ID);
-		} catch (PartInitException e) {
-			PDEPlugin.logException(e);
-		}
-		return null;
-	}
 
 	public String getTitle() {
 		IPluginModelBase model = (IPluginModelBase)getAggregateModel();
@@ -493,8 +519,8 @@ public class ManifestEditor extends MultiSourceEditor implements IShowEditorInpu
 		IPreferenceStore store = PDEPlugin.getDefault().getPreferenceStore();
 		String pref = store.getString(IPreferenceConstants.PROP_SHOW_OBJECTS);
 		if (pref!=null && pref.equals(IPreferenceConstants.VALUE_USE_NAMES))
-			return IPluginBase.P_NAME;
-		return IPluginBase.P_ID;
+			return IPluginObject.P_NAME;
+		return IIdentifiable.P_ID;
 	}
 	
 	private String getTitleText(IPluginBase pluginBase) {
@@ -510,14 +536,22 @@ public class ManifestEditor extends MultiSourceEditor implements IShowEditorInpu
 	 */
 	protected InputContext getInputContext(Object object) {
 		InputContext context = null;
-		if (object instanceof IBuildObject) {
-			context = inputContextManager.findContext(BuildInputContext.CONTEXT_ID);
+		if (object instanceof IFile) {
+			String name = ((IFile)object).getName();
+			if (name.equals("plugin.xml") || name.equals("fragment.xml")) //$NON-NLS-1$ //$NON-NLS-2$
+				context = fInputContextManager.findContext(PluginInputContext.CONTEXT_ID);
+			else if (name.equals("MANIFEST.MF")) //$NON-NLS-1$
+				context = fInputContextManager.findContext(BundleInputContext.CONTEXT_ID);
+			else if (name.equals("build.properties")) //$NON-NLS-1$
+				context = fInputContextManager.findContext(BuildInputContext.CONTEXT_ID);
+		} else if (object instanceof IBuildObject) {
+			context = fInputContextManager.findContext(BuildInputContext.CONTEXT_ID);
 		} else if (object instanceof IPluginExtensionPoint || object instanceof IPluginExtension) {
-			context = inputContextManager.findContext(PluginInputContext.CONTEXT_ID);
+			context = fInputContextManager.findContext(PluginInputContext.CONTEXT_ID);
 		} else {
-			context = inputContextManager.findContext(BundleInputContext.CONTEXT_ID);
+			context = fInputContextManager.findContext(BundleInputContext.CONTEXT_ID);
 			if (context == null)
-				context = inputContextManager.findContext(PluginInputContext.CONTEXT_ID);
+				context = fInputContextManager.findContext(PluginInputContext.CONTEXT_ID);
 		}		
 		return context;
 	}
@@ -526,18 +560,47 @@ public class ManifestEditor extends MultiSourceEditor implements IShowEditorInpu
      * @see org.eclipse.ui.IShowEditorInput#showEditorInput(org.eclipse.ui.IEditorInput)
      */
     public void showEditorInput(IEditorInput editorInput) {
-    	String name = editorInput.getName();
-    	if (name.equals("MANIFEST.MF")) { //$NON-NLS-1$
-    		setActivePage(0);
-    	} else if (name.equals("build.properties")) { //$NON-NLS-1$
-    		setActivePage(BUILD_INDEX);
-    	} else {
-    		if (inputContextManager.hasContext(BundleInputContext.CONTEXT_ID))
-    			setActivePage(3);
-    		else
-    			setActivePage(0);
+     	String name = editorInput.getName();
+		String id = getActivePageInstance().getId();
+		if (name.equals("build.properties")) { //$NON-NLS-1$
+    		if (!BuildInputContext.CONTEXT_ID.equals(id))
+    			setActivePage(SHOW_SOURCE ? BuildInputContext.CONTEXT_ID : BuildPage.PAGE_ID);
+    	} else if (name.equals("plugin.xml") || name.equals("fragment.xml")) { //$NON-NLS-1$ //$NON-NLS-2$
+    		if (!PluginInputContext.CONTEXT_ID.equals(id)) {
+    			if (SHOW_SOURCE) {
+    				setActivePage(PluginInputContext.CONTEXT_ID);
+    			} else if (fInputContextManager.hasContext(BundleInputContext.CONTEXT_ID)) {
+    				setActivePage(ExtensionsPage.PAGE_ID);
+    			} else {
+    				setActivePage(OverviewPage.PAGE_ID);
+    			}
+    		}
+    	} else if (!BundleInputContext.CONTEXT_ID.equals(id)) {
+			setActivePage(SHOW_SOURCE ? BundleInputContext.CONTEXT_ID : OverviewPage.PAGE_ID);
     	}
     }
+    
+    public boolean showExtensionTabs() {
+    	if (fInputContextManager.hasContext(PluginInputContext.CONTEXT_ID))
+    		return true;
+    	IBaseModel model = getAggregateModel();
+    	return fShowExtensions && model != null && model.isEditable();
+     }
 
-
+    public boolean isEquinox() {
+    	return fEquinox;
+    }
+    
+    protected void addExtensionTabs() throws PartInitException {
+    	addPage(3, new ExtensionPointsPage(this));
+    	addPage(3, new ExtensionsPage(this));
+    }
+    
+    protected void setShowExtensions(boolean show) throws BackingStoreException {
+    	if (fPrefs != null) {
+	    	fPrefs.putBoolean(ICoreConstants.EXTENSIONS_PROPERTY, show);
+	    	fPrefs.flush();
+    	}
+    	fShowExtensions = show;
+    }
 }
