@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2004 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -10,33 +10,58 @@
  *******************************************************************************/
 package org.eclipse.pde.internal.ui.editor.plugin;
 
-import java.util.*;
+import java.io.ByteArrayInputStream;
+import java.io.InputStream;
+import java.util.ArrayList;
+import java.util.HashSet;
+import java.util.Iterator;
+import java.util.Vector;
 
-import org.eclipse.core.runtime.*;
-import org.eclipse.jface.action.*;
-import org.eclipse.jface.viewers.*;
-import org.eclipse.pde.core.*;
-import org.eclipse.pde.core.plugin.*;
-import org.eclipse.pde.internal.core.*;
+import org.eclipse.core.runtime.CoreException;
+import org.eclipse.jface.action.Action;
+import org.eclipse.jface.action.IMenuManager;
+import org.eclipse.jface.action.Separator;
+import org.eclipse.jface.viewers.ISelection;
+import org.eclipse.jface.viewers.IStructuredSelection;
+import org.eclipse.jface.viewers.StructuredSelection;
+import org.eclipse.jface.viewers.TableViewer;
+import org.eclipse.jface.window.Window;
+import org.eclipse.pde.core.IModel;
+import org.eclipse.pde.core.IModelChangedEvent;
+import org.eclipse.pde.core.IModelChangedListener;
+import org.eclipse.pde.core.plugin.IPlugin;
+import org.eclipse.pde.core.plugin.IPluginBase;
+import org.eclipse.pde.core.plugin.IPluginImport;
+import org.eclipse.pde.core.plugin.IPluginModel;
+import org.eclipse.pde.core.plugin.IPluginModelBase;
+import org.eclipse.pde.internal.core.IPluginModelListener;
+import org.eclipse.pde.internal.core.PDECore;
+import org.eclipse.pde.internal.core.PluginModelDelta;
+import org.eclipse.pde.internal.core.plugin.ExternalPluginModel;
 import org.eclipse.pde.internal.core.plugin.ImportObject;
 import org.eclipse.pde.internal.ui.PDEPlugin;
 import org.eclipse.pde.internal.ui.PDEUIMessages;
 import org.eclipse.pde.internal.ui.editor.TableSection;
 import org.eclipse.pde.internal.ui.elements.DefaultTableProvider;
 import org.eclipse.pde.internal.ui.parts.TablePart;
-import org.eclipse.pde.internal.ui.search.*;
-import org.eclipse.pde.internal.ui.search.dependencies.*;
-import org.eclipse.pde.internal.ui.util.*;
+import org.eclipse.pde.internal.ui.search.PluginSearchActionGroup;
+import org.eclipse.pde.internal.ui.search.dependencies.UnusedDependenciesAction;
+import org.eclipse.pde.internal.ui.util.SWTUtil;
 import org.eclipse.pde.internal.ui.wizards.PluginSelectionDialog;
 import org.eclipse.swt.SWT;
-import org.eclipse.swt.layout.*;
-import org.eclipse.swt.widgets.*;
-import org.eclipse.ui.actions.*;
-import org.eclipse.ui.forms.widgets.*;
+import org.eclipse.swt.layout.GridData;
+import org.eclipse.swt.widgets.Composite;
+import org.eclipse.swt.widgets.Control;
+import org.eclipse.swt.widgets.Table;
+import org.eclipse.swt.widgets.TableItem;
+import org.eclipse.ui.actions.ActionContext;
+import org.eclipse.ui.actions.ActionFactory;
+import org.eclipse.ui.forms.widgets.FormToolkit;
+import org.eclipse.ui.forms.widgets.Section;
 
 public class RequiresSection
 	extends TableSection
-	implements IModelChangedListener, IModelProviderListener {
+	implements IModelChangedListener, IPluginModelListener {
     
     private static final int ADD_INDEX = 0;
     private static final int REMOVE_INDEX = 1;
@@ -53,36 +78,26 @@ public class RequiresSection
 
 	class ImportContentProvider extends DefaultTableProvider {
 		public Object[] getElements(Object parent) {
-			if (fImports == null) {
+			if (fImports == null)
 				createImportObjects();
-			}
 			return fImports.toArray();
 		}
-		private void createImportObjects() {
-			fImports = new Vector();
-			IPluginModelBase model = (IPluginModelBase) getPage().getModel();
-			IPluginImport[] iimports = model.getPluginBase().getImports();
-			for (int i = 0; i < iimports.length; i++) {
-				IPluginImport iimport = iimports[i];
-				fImports.add(new ImportObject(iimport));
-			}
-		}
 	}
 
 	public RequiresSection(DependenciesPage page, Composite parent, String[] labels) {
 		super(page, parent, Section.DESCRIPTION, labels);
-		getSection().setText(PDEUIMessages.RequiresSection_title); //$NON-NLS-1$
+		getSection().setText(PDEUIMessages.RequiresSection_title); 
 		boolean fragment = ((IPluginModelBase)getPage().getModel()).isFragmentModel();
 		if (fragment)
-			getSection().setDescription(PDEUIMessages.RequiresSection_fDesc); //$NON-NLS-1$
+			getSection().setDescription(PDEUIMessages.RequiresSection_fDesc); 
 		else
-			getSection().setDescription(PDEUIMessages.RequiresSection_desc); //$NON-NLS-1$
+			getSection().setDescription(PDEUIMessages.RequiresSection_desc); 
 		getTablePart().setEditable(false);
 	}
 
 	public void createClient(Section section, FormToolkit toolkit) {
 		Composite container = createClientContainer(section, 2, toolkit);
-		createViewerPartControl(container, SWT.SINGLE, 2, toolkit);
+		createViewerPartControl(container, SWT.MULTI, 2, toolkit);
 		TablePart tablePart = getTablePart();
 		fImportViewer = tablePart.getTableViewer();
 
@@ -93,8 +108,7 @@ public class RequiresSection
 		section.setClient(container);
 		GridData gd = new GridData(GridData.FILL_BOTH);
 		gd.minimumWidth = 250;
-		if (!isBundle())
-			gd.verticalSpan = 2;
+		gd.grabExcessVerticalSpace = true;
 		section.setLayoutData(gd);
 		initialize();
 	}
@@ -108,7 +122,7 @@ public class RequiresSection
 		Table table = getTablePart().getTableViewer().getTable();
 		TableItem[] selection = table.getSelection();
 		boolean hasSelection = selection.length > 0;
-		boolean canMove = table.getItemCount() > 1;
+		boolean canMove = table.getItemCount() > 1 && selection.length == 1;
 		TablePart tablePart = getTablePart();
         tablePart.setButtonEnabled(ADD_INDEX, isEditable());
 		tablePart.setButtonEnabled(
@@ -120,7 +134,7 @@ public class RequiresSection
 				&& hasSelection && isEditable()
 				&& table.getSelectionIndex() < table.getItemCount() - 1);
         if (isBundle())
-            tablePart.setButtonEnabled(PROPERTIES_INDEX, hasSelection);
+            tablePart.setButtonEnabled(PROPERTIES_INDEX, selection.length == 1);
         tablePart.setButtonEnabled(REMOVE_INDEX, isEditable() && hasSelection);
 	}
 
@@ -158,7 +172,7 @@ public class RequiresSection
         dialog.create();
         SWTUtil.setDialogSize(dialog, 400, -1);
         dialog.setTitle(importObject.getId());
-        if (dialog.open() == DependencyPropertiesDialog.OK && isEditable()) {
+        if (dialog.open() == Window.OK && isEditable()) {
             try {
                 importObject.setOptional(dialog.isOptional());
                 importObject.setReexported(dialog.isReexported());
@@ -173,10 +187,7 @@ public class RequiresSection
 		IPluginModelBase model = (IPluginModelBase) getPage().getModel();
 		if (model!=null)
 			model.removeModelChangedListener(this);
-		PDECore.getDefault().getWorkspaceModelManager().removeModelProviderListener(
-			this);
-		PDECore.getDefault().getExternalModelManager().removeModelProviderListener(
-			this);
+		PDECore.getDefault().getModelManager().removePluginModelListener(this);
 		super.dispose();
 	}
 	public boolean doGlobalAction(String actionId) {
@@ -247,7 +258,7 @@ public class RequiresSection
                 if (obj instanceof ImportObject) {
                     IPlugin plugin = ((ImportObject) obj).getPlugin();
                     if (plugin != null)
-                         ManifestEditor.openPluginEditor(plugin);
+                         ManifestEditor.open(plugin, false);
                 }
 			}
 		}
@@ -276,7 +287,7 @@ public class RequiresSection
 				getAvailablePlugins(model),
 				true);
 		dialog.create();
-		if (dialog.open() == PluginSelectionDialog.OK) {
+		if (dialog.open() == Window.OK) {
 			Object[] models = dialog.getResult();
 			for (int i = 0; i < models.length; i++) {
 				try {
@@ -328,8 +339,32 @@ public class RequiresSection
 				result.add(plugins[i]);
 			}
 		}
+
+		if (!existingImports.contains("system.bundle")) //$NON-NLS-1$
+			addSystemBundle(result);
 		return (IPluginModelBase[])result.toArray(new IPluginModelBase[result.size()]);
 	}
+	
+	private void addSystemBundle(java.util.List list) {
+		try {
+			ExternalPluginModel model = new ExternalPluginModel();
+			
+			// Need Install Location to load model.  Giving it org.eclipse.osgi's install location
+			IPluginModelBase osgi = PDECore.getDefault().getModelManager().findModel("org.eclipse.osgi"); //$NON-NLS-1$
+			if (osgi == null)
+				return;
+			model.setInstallLocation(osgi.getInstallLocation());
+			
+			// Load model from a String representing the contents of an equivalent plugin.xml file
+			String pluginInfo = "<?xml version=\"1.0\" encoding=\"UTF-8\"?><plugin id=\"system.bundle\" name=\"System Bundle\"></plugin>"; //$NON-NLS-1$
+			InputStream is = new ByteArrayInputStream(pluginInfo.getBytes());
+			model.load(is, false);
+			
+			list.add(model);
+			
+		} catch (CoreException e) {
+		}
+	}
 
 
 	public void initialize() {
@@ -339,25 +374,23 @@ public class RequiresSection
 		fImportViewer.setInput(model.getPluginBase());
         updateButtons();
 		model.addModelChangedListener(this);
-		PDECore.getDefault().getWorkspaceModelManager().addModelProviderListener(
-			this);
-		PDECore.getDefault().getExternalModelManager().addModelProviderListener(this);
+		PDECore.getDefault().getModelManager().addPluginModelListener(this);
 		fAddAction.setEnabled(model.isEditable());
 		fRemoveAction.setEnabled(model.isEditable());
 	}
 
 	private void makeActions() {
-		fAddAction = new Action(PDEUIMessages.RequiresSection_add) { //$NON-NLS-1$
+		fAddAction = new Action(PDEUIMessages.RequiresSection_add) { 
 			public void run() {
 				handleAdd();
 			}
 		};
-		fOpenAction = new Action(PDEUIMessages.RequiresSection_open) { //$NON-NLS-1$
+		fOpenAction = new Action(PDEUIMessages.RequiresSection_open) { 
 			public void run() {
 				handleOpen(fImportViewer.getSelection());
 			}
 		};
-		fRemoveAction = new Action(PDEUIMessages.RequiresSection_delete) { //$NON-NLS-1$
+		fRemoveAction = new Action(PDEUIMessages.RequiresSection_delete) { 
 			public void run() {
 				handleRemove();
 			}
@@ -393,7 +426,11 @@ public class RequiresSection
 			IPluginImport iimport = (IPluginImport) changeObject;
 			if (event.getChangeType() == IModelChangedEvent.INSERT) {
 				ImportObject iobj = new ImportObject(iimport);
-				fImports.add(iobj);
+				if (fImports == null)
+					// creatImportObjects method will find new addition
+					createImportObjects();
+				else
+					fImports.add(iobj);
 				fImportViewer.add(iobj);
 				fImportViewer.setSelection(new StructuredSelection(iobj), true);
 				fImportViewer.getTable().setFocus();
@@ -401,7 +438,11 @@ public class RequiresSection
 				ImportObject iobj = findImportObject(iimport);
 				if (iobj != null) {
 					if (event.getChangeType() == IModelChangedEvent.REMOVE) {
-						fImports.remove(iobj);
+						if (fImports == null)
+							// createImportObjects method will not include the removed import
+							createImportObjects();
+						else
+							fImports.remove(iobj);
                         Table table = fImportViewer.getTable();
                         int index = table.getSelectionIndex();
 						fImportViewer.remove(iobj);
@@ -416,7 +457,7 @@ public class RequiresSection
 		}
 	}
 
-	public void modelsChanged(IModelProviderEvent e) {
+	public void modelsChanged(PluginModelDelta delta) {
 		fImports = null;
 		final Control control = fImportViewer.getControl();
 		if (!control.isDisposed()) {
@@ -440,6 +481,16 @@ public class RequiresSection
 		return null;
 	}
 
+	private void createImportObjects() {
+		fImports = new Vector();
+		IPluginModelBase model = (IPluginModelBase) getPage().getModel();
+		IPluginImport[] iimports = model.getPluginBase().getImports();
+		for (int i = 0; i < iimports.length; i++) {
+			IPluginImport iimport = iimports[i];
+			fImports.add(new ImportObject(iimport));
+		}
+	}
+	
 	public void setFocus() {
 		if (fImportViewer != null)
 			fImportViewer.getTable().setFocus();
