@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2004 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -10,16 +10,20 @@
  *******************************************************************************/
 package org.eclipse.pde.internal.ui.editor.text;
 
-import org.eclipse.jface.text.rules.*;
+import org.eclipse.jface.text.rules.IPredicateRule;
+import org.eclipse.jface.text.rules.MultiLineRule;
+import org.eclipse.jface.text.rules.RuleBasedPartitionScanner;
+import org.eclipse.jface.text.rules.Token;
 
 public class XMLPartitionScanner extends RuleBasedPartitionScanner {
-	public final static String XML_DEFAULT = "__xml_default"; //$NON-NLS-1$
 	public final static String XML_COMMENT = "__xml_comment"; //$NON-NLS-1$
 	public final static String XML_TAG = "__xml_tag"; //$NON-NLS-1$
-
+	
+	public static final String[] PARTITIONS = new String[] {XML_COMMENT, XML_TAG};
+	
 	public XMLPartitionScanner() {
 		IPredicateRule[] rules = new IPredicateRule[2];
-		rules[0] = new MultiLineRule("<!--", "-->", new Token(XML_COMMENT)); //$NON-NLS-1$ //$NON-NLS-2$
+		rules[0] = new MultiLineRule("<!--", "-->", new Token(XML_COMMENT), '\\', true); //$NON-NLS-1$ //$NON-NLS-2$
 		rules[1] = new XMLTagRule(new Token(XML_TAG));
 		setPredicateRules(rules);
 	}
