@@ -476,7 +476,7 @@ public class ExternalizeStringsWizardPage extends WizardPage {
 	
 	private void updatePropertiesLabel(IPluginModelBase model) {
 		ModelChange modelChange = fModelChangeTable.getModelChange(model);
-		fProjectLabel.setText(model.getBundleDescription().getName());
+		fProjectLabel.setText(model.getUnderlyingResource().getProject().getName());
 		fLocalizationText.setEditable(!modelChange.localizationSet());
 		fLocalizationText.setText(modelChange.getBundleLocalization());
 	}
@@ -512,7 +512,7 @@ public class ExternalizeStringsWizardPage extends WizardPage {
 			error = getErrorMessage(PDEUIMessages.ExternalizeStringsWizardPage_keyEmptyError, oldKey);
 		} else if (key.charAt(0) == '#' || key.charAt(0) == '!' || key.charAt(0) == '%') {
 			error = getErrorMessage(PDEUIMessages.ExternalizeStringsWizardPage_keyCommentError, oldKey);
-		} else if (key.indexOf(":") != -1 || key.indexOf("=") != -1 || key.indexOf(" ") != -1) { //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
+		} else if (key.indexOf(':') != -1 || key.indexOf('=') != -1 || key.indexOf(' ') != -1) {
 			error = getErrorMessage(PDEUIMessages.ExternalizeStringsWizardPage_keyError, oldKey);
 		} else if ((!key.equals(oldKey) || fPreErrorKey != null) &&
 				properties.containsKey(key)) {
