@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -37,16 +37,14 @@ import org.eclipse.pde.core.plugin.IPluginModelBase;
 import org.eclipse.pde.internal.core.PDECore;
 import org.eclipse.pde.internal.core.ischema.IDocumentSection;
 import org.eclipse.pde.internal.core.ischema.ISchemaAttribute;
-import org.eclipse.pde.internal.core.ischema.ISchemaCompositor;
 import org.eclipse.pde.internal.core.schema.DocumentSection;
 import org.eclipse.pde.internal.core.schema.EditableSchema;
 import org.eclipse.pde.internal.core.schema.SchemaAttribute;
 import org.eclipse.pde.internal.core.schema.SchemaComplexType;
-import org.eclipse.pde.internal.core.schema.SchemaCompositor;
 import org.eclipse.pde.internal.core.schema.SchemaElement;
+import org.eclipse.pde.internal.core.schema.SchemaRootElement;
 import org.eclipse.pde.internal.core.schema.SchemaSimpleType;
 import org.eclipse.pde.internal.core.util.CoreUtility;
-import org.eclipse.pde.internal.core.util.IdUtil;
 import org.eclipse.pde.internal.ui.IHelpContextIds;
 import org.eclipse.pde.internal.ui.IPDEUIConstants;
 import org.eclipse.pde.internal.ui.PDEPlugin;
@@ -120,8 +118,8 @@ public abstract class BaseExtensionPointMainPage extends WizardPage {
 			gd.horizontalSpan =1;
 			gd.widthHint = 50;
 			fPluginBrowseButton.setLayoutData(gd);
-			fPluginBrowseButton.setText(PDEUIMessages.BaseExtensionPointMainPage_pluginBrowse); //$NON-NLS-1$
-			fPluginBrowseButton.setToolTipText(PDEUIMessages.BaseExtensionPointMainPage_pluginId_tooltip); //$NON-NLS-1$
+			fPluginBrowseButton.setText(PDEUIMessages.BaseExtensionPointMainPage_pluginBrowse); 
+			fPluginBrowseButton.setToolTipText(PDEUIMessages.BaseExtensionPointMainPage_pluginId_tooltip); 
 			fPluginBrowseButton.addSelectionListener(new SelectionAdapter(){
 				public void widgetSelected(SelectionEvent e) {
 					handlePluginBrowse();
@@ -171,8 +169,8 @@ public abstract class BaseExtensionPointMainPage extends WizardPage {
 			gd = new GridData(GridData.HORIZONTAL_ALIGN_END);
 			gd.widthHint = 50;
 			fFindLocationButton.setLayoutData(gd);
-			fFindLocationButton.setText(PDEUIMessages.BaseExtensionPointMainPage_findBrowse); //$NON-NLS-1$
-			fFindLocationButton.setToolTipText(PDEUIMessages.BaseExtensionPointMainPage_schemaLocation_tooltip); //$NON-NLS-1$
+			fFindLocationButton.setText(PDEUIMessages.BaseExtensionPointMainPage_findBrowse); 
+			fFindLocationButton.setToolTipText(PDEUIMessages.BaseExtensionPointMainPage_schemaLocation_tooltip); 
 			fFindLocationButton.addSelectionListener(new SelectionAdapter(){
 				public void widgetSelected(SelectionEvent e) {
 					handleSchemaLocation();
@@ -220,17 +218,14 @@ public abstract class BaseExtensionPointMainPage extends WizardPage {
 		EditableSchema schema = new EditableSchema(pluginId, pointId, name, false);
 		schema.setDescription(PDEUIMessages.BaseExtensionPoint_sections_overview);
 		DocumentSection section;
-		section = new DocumentSection(schema, IDocumentSection.SINCE, PDEUIMessages.BaseExtensionPointMainPage_since); //$NON-NLS-1$
+		section = new DocumentSection(schema, IDocumentSection.SINCE, PDEUIMessages.BaseExtensionPointMainPage_since); 
 		section.setDescription(PDEUIMessages.BaseExtensionPoint_sections_since);
 		schema.addDocumentSection(section);
 		SchemaElement element;
 		if (!shared) {
-			element = new SchemaElement(schema, "extension"); //$NON-NLS-1$
+			element = new SchemaRootElement(schema, "extension"); //$NON-NLS-1$
 			SchemaComplexType complexType = new SchemaComplexType(schema);
 			element.setType(complexType);
-			SchemaCompositor compositor = new SchemaCompositor(element,
-					ISchemaCompositor.SEQUENCE);
-			complexType.setCompositor(compositor);
 			SchemaAttribute attribute = new SchemaAttribute(element, "point"); //$NON-NLS-1$
 			attribute.setType(new SchemaSimpleType(schema, "string")); //$NON-NLS-1$
 			attribute.setUse(ISchemaAttribute.REQUIRED);
@@ -384,9 +379,6 @@ public abstract class BaseExtensionPointMainPage extends WizardPage {
 	}
 	public boolean checkFieldsFilled() {	
 		boolean empty = fIdText.getText().length() == 0 || fNameText.getText().length() == 0;
-        if (!empty) {
-            empty = !IdUtil.isValidExtensionPointId(fIdText.getText());
-        }
 		if (!empty && isPluginIdNeeded()) {
 			empty = getPluginId().length() == 0 || fSchemaText.getText().length() == 0 ;
 		}
@@ -395,24 +387,18 @@ public abstract class BaseExtensionPointMainPage extends WizardPage {
 		return !empty;
 	}
 
-    public boolean isInvalidValidId() {
-        return fIdText.getText().length()>0 && !IdUtil.isValidExtensionPointId(fIdText.getText());
-    }
-
 	private void validatePage() {
 		if (!validateContainer())
 			return;
 		boolean isFilled = checkFieldsFilled();
-		String message = null;
-        if (isInvalidValidId())
-            message = PDEUIMessages.BaseExtensionPoint_malformedId;                
-        else if (!isFilled) {
+		String message = getInvalidIdMessage();              
+        if (message == null && !isFilled) {
             if (isPluginIdNeeded())
 				message = PDEUIMessages.BaseExtensionPoint_missingId;
 			else
 				message = PDEUIMessages.BaseExtensionPoint_noPlugin_missingId;
 		}
-		setPageComplete(isFilled);
+		setPageComplete(isFilled && isComplete());
 		setMessage(message, IMessageProvider.WARNING);
 	}
 	private boolean validateContainer() {
@@ -440,7 +426,7 @@ public abstract class BaseExtensionPointMainPage extends WizardPage {
 		return exists;
 	}
 	private void handleInvalidContainer(){
-		setErrorMessage(PDEUIMessages.BaseExtensionPointMainPage_noContainer); //$NON-NLS-1$
+		setErrorMessage(PDEUIMessages.BaseExtensionPointMainPage_noContainer); 
 		setPageComplete(false);
 	}
 	private void handleValidContainer(){
@@ -460,8 +446,8 @@ public abstract class BaseExtensionPointMainPage extends WizardPage {
 				getShell(),
 				new WorkbenchLabelProvider(),
 				new WorkbenchContentProvider());
-		dialog.setTitle(PDEUIMessages.BaseExtensionPointMainPage_schemaLocation_title); //$NON-NLS-1$
-		dialog.setMessage(PDEUIMessages.BaseExtensionPointMainPage_schemaLocation_desc); //$NON-NLS-1$
+		dialog.setTitle(PDEUIMessages.BaseExtensionPointMainPage_schemaLocation_title); 
+		dialog.setMessage(PDEUIMessages.BaseExtensionPointMainPage_schemaLocation_desc); 
 		dialog.setDoubleClickSelects(false);
 		dialog.setAllowMultiple(false);
 		dialog.addFilter(new ViewerFilter(){
@@ -490,4 +476,11 @@ public abstract class BaseExtensionPointMainPage extends WizardPage {
 			path = path.replaceFirst(workspacePath, ""); //$NON-NLS-1$
 		return path;
 	}
+	public String getInvalidIdMessage() {
+		// No validation done (other than making sure id is not blank)
+		return null;
+	}
+	protected boolean isComplete() {
+		return true;
+	}
 }
