@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -53,9 +53,10 @@ import org.eclipse.swt.events.*;
  * @see List
  */
 public class Combo extends Composite {
-	int /*long*/ arrowHandle, entryHandle, listHandle;
+	long /*int*/ arrowHandle, entryHandle, listHandle;
 	int lastEventTime, visibleCount = 5;
-	int /*long*/ gdkEventKey = 0;
+	long /*int*/ gdkEventKey = 0;
+	int fixStart = -1, fixEnd = -1;
 	String [] items = new String [0];
 	boolean ignoreSelect, lockText;
 
@@ -173,13 +174,13 @@ public void add (String string, int index) {
 	*/
 	ignoreSelect = lockText = true;
 	byte [] buffer = Converter.wcsToMbcs (null, string, true);
-	int /*long*/ item = OS.gtk_list_item_new_with_label (buffer);
-	int /*long*/ label = OS.gtk_bin_get_child (item); 
+	long /*int*/ item = OS.gtk_list_item_new_with_label (buffer);
+	long /*int*/ label = OS.gtk_bin_get_child (item); 
 	OS.gtk_widget_modify_fg (label, OS.GTK_STATE_NORMAL, getForegroundColor ());
 	OS.gtk_widget_modify_font (label, getFontDescription ());
 	OS.gtk_widget_set_direction (label, OS.gtk_widget_get_direction (handle));
 	OS.gtk_widget_show (item);
-	int /*long*/ items = OS.g_list_append (0, item);
+	long /*int*/ items = OS.g_list_append (0, item);
 	OS.gtk_list_insert_items (listHandle, items, index);
 	ignoreSelect = lockText = false;
 }
@@ -330,10 +331,10 @@ public Point computeSize (int wHint, int hHint, boolean changed) {
 	if (wHint != SWT.DEFAULT && wHint < 0) wHint = 0;
 	if (hHint != SWT.DEFAULT && hHint < 0) hHint = 0;
 	int[] w = new int [1], h = new int [1];
-	int /*long*/ layout = OS.gtk_entry_get_layout (entryHandle);
+	long /*int*/ layout = OS.gtk_entry_get_layout (entryHandle);
 	OS.pango_layout_get_size (layout, w, h);
 	int xborder = INNER_BORDER, yborder = INNER_BORDER;
-	int /*long*/ style = OS.gtk_widget_get_style (entryHandle);
+	long /*int*/ style = OS.gtk_widget_get_style (entryHandle);
 	xborder += OS.gtk_style_get_xthickness (style);
 	yborder += OS.gtk_style_get_ythickness (style);
 	int [] property = new int [1];
@@ -349,7 +350,7 @@ public Point computeSize (int wHint, int hHint, boolean changed) {
 	GtkRequisition arrowRequesition = new GtkRequisition ();
 	OS.gtk_widget_size_request (arrowHandle, arrowRequesition);
 	GtkRequisition listRequesition = new GtkRequisition ();
-	int /*long*/ listParent = OS.gtk_widget_get_parent (listHandle);
+	long /*int*/ listParent = OS.gtk_widget_get_parent (listHandle);
 	OS.gtk_widget_size_request (listParent != 0 ? listParent : listHandle, listRequesition);
 	
 	width = Math.max (listRequesition.width, width) + arrowRequesition.width + 4;
@@ -396,11 +397,11 @@ void createHandle (int index) {
 	* to hook events.  The fix is to find the first child that is
 	* not the entry or list and assume this is the arrow handle.
 	*/
-	int /*long*/ list = OS.gtk_container_get_children (handle);
+	long /*int*/ list = OS.gtk_container_get_children (handle);
 	if (list != 0) {
 		int i = 0, count = OS.g_list_length (list);
 		while (i<count) {
-			int /*long*/ childHandle = OS.g_list_nth_data (list, i);
+			long /*int*/ childHandle = OS.g_list_nth_data (list, i);
 			if (childHandle != entryHandle && childHandle != listHandle) {
 				arrowHandle = childHandle;
 				break;
@@ -440,15 +441,15 @@ void deregister () {
 	if (arrowHandle != 0) display.removeWidget (arrowHandle);
 	display.removeWidget (entryHandle);
 	display.removeWidget (listHandle);
-	int /*long*/ imContext = imContext ();
+	long /*int*/ imContext = imContext ();
 	if (imContext != 0) display.removeWidget (imContext);
 }
 
-boolean filterKey (int keyval, int /*long*/ event) {
+boolean filterKey (int keyval, long /*int*/ event) {
 	int time = OS.gdk_event_get_time (event);
 	if (time != lastEventTime) {
 		lastEventTime = time;
-		int /*long*/ imContext = imContext ();
+		long /*int*/ imContext = imContext ();
 		if (imContext != 0) {
 			return OS.gtk_im_context_filter_keypress (imContext, event);
 		}
@@ -467,7 +468,7 @@ void fixIM () {
 	*  filter has to be called by SWT.
 	*/	
 	if (gdkEventKey != 0 && gdkEventKey != -1) {
-		int /*long*/ imContext = imContext ();
+		long /*int*/ imContext = imContext ();
 		if (imContext != 0) {
 			OS.gtk_im_context_filter_keypress (imContext, gdkEventKey);
 			gdkEventKey = -1;
@@ -477,12 +478,12 @@ void fixIM () {
 	gdkEventKey = 0;
 }
 
-int /*long*/ fontHandle () {
+long /*int*/ fontHandle () {
 	if (entryHandle != 0) return entryHandle;
 	return super.fontHandle ();
 }
 
-int /*long*/ focusHandle () {
+long /*int*/ focusHandle () {
 	if (entryHandle != 0) return entryHandle;
 	return super.focusHandle ();
 }
@@ -497,28 +498,24 @@ boolean hasFocus () {
 void hookEvents () {
 	//TODO - fix multiple enter/exit
 	super.hookEvents ();
-	int /*long*/ windowProc2 = display.windowProc2;
-	int /*long*/ windowProc3 = display.windowProc3;
-	int /*long*/ windowProc4 = display.windowProc4;
-	int /*long*/ windowProc5 = display.windowProc5;
-	OS.g_signal_connect_after (entryHandle, OS.changed, windowProc2, CHANGED);
-	OS.g_signal_connect (entryHandle, OS.insert_text, windowProc5, INSERT_TEXT);
-	OS.g_signal_connect (entryHandle, OS.delete_text, windowProc4, DELETE_TEXT);
-	OS.g_signal_connect (entryHandle, OS.activate, windowProc2, ACTIVATE);
+	OS.g_signal_connect_closure (entryHandle, OS.changed, display.closures [CHANGED], true);
+	OS.g_signal_connect_closure (entryHandle, OS.insert_text, display.closures [INSERT_TEXT], false);
+	OS.g_signal_connect_closure (entryHandle, OS.delete_text, display.closures [DELETE_TEXT], false);
+	OS.g_signal_connect_closure (entryHandle, OS.activate, display.closures [ACTIVATE], false);
 	int eventMask =	OS.GDK_POINTER_MOTION_MASK | OS.GDK_BUTTON_PRESS_MASK |
 		OS.GDK_BUTTON_RELEASE_MASK | OS.GDK_ENTER_NOTIFY_MASK |
 		OS.GDK_LEAVE_NOTIFY_MASK;
-	int /*long*/ [] handles = new int /*long*/ [] {arrowHandle, entryHandle, listHandle};
+	long /*int*/ [] handles = new long /*int*/ [] {arrowHandle, entryHandle, listHandle};
 	for (int i=0; i<handles.length; i++) {
-		int /*long*/ eventHandle = handles [i];
+		long /*int*/ eventHandle = handles [i];
 		if (eventHandle != 0) {
 			/* Connect the mouse signals */
 			OS.gtk_widget_add_events (eventHandle, eventMask);
-			OS.g_signal_connect (eventHandle, OS.button_press_event, windowProc3, BUTTON_PRESS_EVENT);
-			OS.g_signal_connect (eventHandle, OS.button_release_event, windowProc3, BUTTON_RELEASE_EVENT);
-			OS.g_signal_connect (eventHandle, OS.motion_notify_event, windowProc3, MOTION_NOTIFY_EVENT);
-			OS.g_signal_connect (eventHandle, OS.enter_notify_event, windowProc3, ENTER_NOTIFY_EVENT);
-			OS.g_signal_connect (eventHandle, OS.leave_notify_event, windowProc3, LEAVE_NOTIFY_EVENT);
+			OS.g_signal_connect_closure_by_id (eventHandle, display.signalIds [BUTTON_PRESS_EVENT], 0, display.closures [BUTTON_PRESS_EVENT], false);
+			OS.g_signal_connect_closure_by_id (eventHandle, display.signalIds [BUTTON_RELEASE_EVENT], 0, display.closures [BUTTON_RELEASE_EVENT], false);
+			OS.g_signal_connect_closure_by_id (eventHandle, display.signalIds [MOTION_NOTIFY_EVENT], 0, display.closures [MOTION_NOTIFY_EVENT], false);
+			OS.g_signal_connect_closure_by_id (eventHandle, display.signalIds [ENTER_NOTIFY_EVENT], 0, display.closures [ENTER_NOTIFY_EVENT], false);
+			OS.g_signal_connect_closure_by_id (eventHandle, display.signalIds [LEAVE_NOTIFY_EVENT], 0, display.closures [LEAVE_NOTIFY_EVENT], false);
 			/*
 			* Feature in GTK.  Events such as mouse move are propagated up
 			* the widget hierarchy and are seen by the parent.  This is the
@@ -526,26 +523,26 @@ void hookEvents () {
 			* hook a signal after and stop the propagation using a negative
 			* event number to distinguish this case.
 			*/
-			OS.g_signal_connect_after (eventHandle, OS.button_press_event, windowProc3, -BUTTON_PRESS_EVENT);
-			OS.g_signal_connect_after (eventHandle, OS.button_release_event, windowProc3, -BUTTON_RELEASE_EVENT);
-			OS.g_signal_connect_after (eventHandle, OS.motion_notify_event, windowProc3, -MOTION_NOTIFY_EVENT);
+			OS.g_signal_connect_closure_by_id (eventHandle, display.signalIds [BUTTON_PRESS_EVENT], 0, display.closures [BUTTON_PRESS_EVENT_INVERSE], true);
+			OS.g_signal_connect_closure_by_id (eventHandle, display.signalIds [BUTTON_RELEASE_EVENT], 0, display.closures [BUTTON_RELEASE_EVENT_INVERSE], true);
+			OS.g_signal_connect_closure_by_id (eventHandle, display.signalIds [MOTION_NOTIFY_EVENT], 0, display.closures [MOTION_NOTIFY_EVENT_INVERSE], true);
 
 			/* Connect the event_after signal for both key and mouse */
 			if (eventHandle != entryHandle) {
-				OS.g_signal_connect (eventHandle, OS.event_after, windowProc3, EVENT_AFTER);
+				OS.g_signal_connect_closure_by_id (eventHandle, display.signalIds [EVENT_AFTER], 0, display.closures [EVENT_AFTER], false);
 			}
 		}
 	}
-	int /*long*/ imContext = imContext ();
+	long /*int*/ imContext = imContext ();
 	if (imContext != 0) {
-		OS.g_signal_connect (imContext, OS.commit, windowProc3, COMMIT);
+		OS.g_signal_connect_closure (imContext, OS.commit, display.closures [COMMIT], false);
 		int id = OS.g_signal_lookup (OS.commit, OS.gtk_im_context_get_type ());
 		int blockMask =  OS.G_SIGNAL_MATCH_DATA | OS.G_SIGNAL_MATCH_ID;
 		OS.g_signal_handlers_block_matched (imContext, blockMask, id, 0, 0, 0, entryHandle);
 	}	
 }
 
-int /*long*/ imContext () {
+long /*int*/ imContext () {
 	return OS.GTK_ENTRY_IM_CONTEXT (entryHandle);
 }
 
@@ -565,8 +562,8 @@ public void deselect (int index) {
 	checkWidget();
 	if (index < 0 || index >= items.length) return;
 	ignoreSelect = true;
-	int /*long*/ children = OS.gtk_container_get_children (listHandle);
-	int /*long*/ item = OS.g_list_nth_data (children, index);
+	long /*int*/ children = OS.gtk_container_get_children (listHandle);
+	long /*int*/ item = OS.g_list_nth_data (children, index);
 	boolean selected = OS.GTK_WIDGET_STATE (item) == OS.GTK_STATE_SELECTED;
 	if (selected) {
 		OS.gtk_list_unselect_all (listHandle);
@@ -742,10 +739,10 @@ public Point getSelection () {
 public int getSelectionIndex () {
 	checkWidget();
 	int index = 0, result = -1;
-	int /*long*/ children = OS.gtk_container_get_children (listHandle);
-	int /*long*/ temp = children;
+	long /*int*/ children = OS.gtk_container_get_children (listHandle);
+	long /*int*/ temp = children;
 	while (temp != 0) {
-		int /*long*/ item = OS.g_list_data (temp);
+		long /*int*/ item = OS.g_list_data (temp);
 		if (OS.GTK_WIDGET_STATE (item) == OS.GTK_STATE_SELECTED) {
 			result = index;
 			break;
@@ -771,7 +768,7 @@ public int getSelectionIndex () {
  */
 public String getText () {
 	checkWidget();
-	int /*long*/ address = OS.gtk_entry_get_text (entryHandle);
+	long /*int*/ address = OS.gtk_entry_get_text (entryHandle);
 	int length = OS.strlen (address);
 	byte [] buffer = new byte [length];
 	OS.memmove (buffer, address, length);
@@ -845,14 +842,14 @@ public int getVisibleItemCount () {
 	return visibleCount;
 }
 
-int /*long*/ gtk_activate (int /*long*/ widget) {
+long /*int*/ gtk_activate (long /*int*/ widget) {
 	postEvent (SWT.DefaultSelection);
 	return 0;
 }
 
-int /*long*/ gtk_changed (int /*long*/ widget) {
+long /*int*/ gtk_changed (long /*int*/ widget) {
 	if (!ignoreSelect) {
-		int /*long*/ ptr = OS.gtk_entry_get_text (entryHandle);
+		long /*int*/ ptr = OS.gtk_entry_get_text (entryHandle);
 		int length = OS.strlen (ptr);
 		byte [] buffer = new byte [length];
 		OS.memmove (buffer, ptr, length);
@@ -864,11 +861,34 @@ int /*long*/ gtk_changed (int /*long*/ widget) {
 			}
 		}
 	}
-	sendEvent (SWT.Modify);
+	/*
+	* Feature in GTK.  When the user types, GTK positions
+	* the caret after sending the changed signal.  This
+	* means that application code that attempts to position
+	* the caret during a changed signal will fail.  The fix
+	* is to post the modify event when the user is typing.
+	*/
+	boolean keyPress = false;
+	long /*int*/ eventPtr = OS.gtk_get_current_event ();
+	if (eventPtr != 0) {
+		GdkEventKey gdkEvent = new GdkEventKey ();
+		OS.memmove (gdkEvent, eventPtr, GdkEventKey.sizeof);
+		switch (gdkEvent.type) {
+			case OS.GDK_KEY_PRESS:
+				keyPress = true;
+				break;
+		}
+		OS.gdk_event_free (eventPtr);
+	}
+	if (keyPress) {
+		postEvent (SWT.Modify);
+	} else {
+		sendEvent (SWT.Modify);
+	}
 	return 0;
 }
 
-int /*long*/ gtk_commit (int /*long*/ imContext, int /*long*/ text) {
+long /*int*/ gtk_commit (long /*int*/ imContext, long /*int*/ text) {
 	if (text == 0) return 0;
 	if (!OS.gtk_editable_get_editable (entryHandle)) return 0;
 	int length = OS.strlen (text);
@@ -878,6 +898,14 @@ int /*long*/ gtk_commit (int /*long*/ imContext, int /*long*/ text) {
 	char [] chars = Converter.mbcsToWcs (null, buffer);
 	char [] newChars = sendIMKeyEvent (SWT.KeyDown, null, chars);
 	if (newChars == null) return 0;
+	/*
+	* Feature in GTK.  For a GtkEntry, during the insert-text signal,
+	* GTK allows the programmer to change only the caret location,
+	* not the selection.  If the programmer changes the selection,
+	* the new selection is lost.  The fix is to detect a selection
+	* change and set it after the insert-text signal has completed.
+	*/
+	fixStart = fixEnd = -1;
 	OS.g_signal_handlers_block_matched (imContext, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, COMMIT);
 	int id = OS.g_signal_lookup (OS.commit, OS.gtk_im_context_get_type ());
 	int mask =  OS.G_SIGNAL_MATCH_DATA | OS.G_SIGNAL_MATCH_ID;
@@ -890,10 +918,15 @@ int /*long*/ gtk_commit (int /*long*/ imContext, int /*long*/ text) {
 	}
 	OS.g_signal_handlers_unblock_matched (imContext, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, COMMIT);
 	OS.g_signal_handlers_block_matched (imContext, mask, id, 0, 0, 0, entryHandle);
+	if (fixStart != -1 && fixEnd != -1) {
+		OS.gtk_editable_set_position (entryHandle, fixStart);
+		OS.gtk_editable_select_region (entryHandle, fixStart, fixEnd);
+	}
+	fixStart = fixEnd = -1;
 	return 0;
 }
 
-int /*long*/ gtk_delete_text (int /*long*/ widget, int /*long*/ start_pos, int /*long*/ end_pos) {
+long /*int*/ gtk_delete_text (long /*int*/ widget, long /*int*/ start_pos, long /*int*/ end_pos) {
 	if (lockText) {
 		OS.gtk_list_unselect_item (listHandle, 0);
 		OS.g_signal_stop_emission_by_name (entryHandle, OS.delete_text);
@@ -919,12 +952,12 @@ int /*long*/ gtk_delete_text (int /*long*/ widget, int /*long*/ start_pos, int /
 	return 0;
 }
 
-int /*long*/ gtk_focus_out_event (int /*long*/ widget, int /*long*/ event) {
+long /*int*/ gtk_focus_out_event (long /*int*/ widget, long /*int*/ event) {
 	fixIM ();
 	return super.gtk_focus_out_event (widget, event);
 }
 
-int /*long*/ gtk_insert_text (int /*long*/ widget, int /*long*/ new_text, int /*long*/ new_text_length, int /*long*/ position) {
+long /*int*/ gtk_insert_text (long /*int*/ widget, long /*int*/ new_text, long /*int*/ new_text_length, long /*int*/ position) {
 	if (lockText) {
 		OS.gtk_list_unselect_item (listHandle, 0);
 		OS.g_signal_stop_emission_by_name (entryHandle, OS.insert_text);
@@ -937,49 +970,48 @@ int /*long*/ gtk_insert_text (int /*long*/ widget, int /*long*/ new_text, int /*
 	String oldText = new String (Converter.mbcsToWcs (null, buffer));
 	int [] pos = new int [1];
 	OS.memmove (pos, position, 4);
-	if (pos [0] == -1)  {
-		int /*long*/ ptr = OS.gtk_entry_get_text (entryHandle);
+	if (pos [0] == -1) {
+		long /*int*/ ptr = OS.gtk_entry_get_text (entryHandle);
 		pos [0] = (int)/*64*/OS.g_utf8_strlen (ptr, -1);
 	}
-	int [] oldStart = new int [1], oldEnd = new int [1];
-	OS.gtk_editable_get_selection_bounds (entryHandle, oldStart, oldEnd);
 	String newText = verifyText (oldText, pos [0], pos [0]);
-	final int [] newStart = new int [1], newEnd = new int [1];
-	OS.gtk_editable_get_selection_bounds (entryHandle, newStart, newEnd);
-	boolean newSelection = oldStart [0] != newStart [0] || oldEnd [0] != newEnd [0];
-	if (newSelection) {
-		if (newText == null) newText = "";
-		pos [0] = newEnd [0];
-	}
-	if (newText == null) {
-		OS.g_signal_stop_emission_by_name (entryHandle, OS.insert_text);
-	} else {
-		if (newText != oldText || newSelection) {
-			byte [] buffer2 = Converter.wcsToMbcs (null, newText, false);
-			OS.g_signal_handlers_block_matched (entryHandle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, INSERT_TEXT);
-			if (newSelection) {
+	if (newText != oldText) {
+		int [] newStart = new int [1], newEnd = new int [1];
+		OS.gtk_editable_get_selection_bounds (entryHandle, newStart, newEnd);
+		if (newText != null) {
+			if (newStart [0] != newEnd [0]) {
+				OS.g_signal_handlers_block_matched (entryHandle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, DELETE_TEXT);
 				OS.g_signal_handlers_block_matched (entryHandle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, CHANGED);
 				OS.gtk_editable_delete_selection (entryHandle);
+				OS.g_signal_handlers_unblock_matched (entryHandle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, DELETE_TEXT);
 				OS.g_signal_handlers_unblock_matched (entryHandle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, CHANGED);
 			}
-			OS.gtk_editable_insert_text (entryHandle, buffer2, buffer2.length, pos);
+			byte [] buffer3 = Converter.wcsToMbcs (null, newText, false);
+			OS.g_signal_handlers_block_matched (entryHandle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, INSERT_TEXT);
+			OS.gtk_editable_insert_text (entryHandle, buffer3, buffer3.length, pos);
 			OS.g_signal_handlers_unblock_matched (entryHandle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, INSERT_TEXT);
-			OS.g_signal_stop_emission_by_name (entryHandle, OS.insert_text);
-			OS.memmove (position, pos, 4);
+			newEnd [0] = pos [0];
 		}
+		pos [0] = newEnd [0];
+		if (newStart [0] != newEnd [0] && newEnd [0] != pos [0]) {
+			fixStart = newStart [0];
+			fixEnd = newEnd [0];
+		}
+		OS.memmove (position, pos, 4);
+		OS.g_signal_stop_emission_by_name (entryHandle, OS.insert_text);
 	}
 	return 0;
 }
 
-int /*long*/ gtk_key_press_event (int /*long*/ widget, int /*long*/ event) {
-	int /*long*/ result = super.gtk_key_press_event (widget, event);
+long /*int*/ gtk_key_press_event (long /*int*/ widget, long /*int*/ event) {
+	long /*int*/ result = super.gtk_key_press_event (widget, event);
 	if (result != 0) fixIM ();
 	if (gdkEventKey == -1) result = 1;
 	gdkEventKey = 0;
 	return result;
 }
 
-int /*long*/ gtk_popup_menu (int /*long*/ widget) {
+long /*int*/ gtk_popup_menu (long /*int*/ widget) {
 	int [] x = new int [1], y = new int [1];
 	OS.gdk_window_get_pointer (0, x, y, null);
 	return showMenu (x [0], y [0]) ? 1 : 0;
@@ -1055,7 +1087,7 @@ public void paste () {
 	OS.gtk_editable_paste_clipboard (entryHandle);
 }
 
-int /*long*/ parentingHandle() {
+long /*int*/ parentingHandle() {
 	return fixedHandle;
 }
 
@@ -1064,7 +1096,7 @@ void register () {
 	if (arrowHandle != 0) display.addWidget (arrowHandle, this);
 	display.addWidget (entryHandle, this);
 	display.addWidget (listHandle, this);
-	int /*long*/ imContext = imContext ();
+	long /*int*/ imContext = imContext ();
 	if (imContext != 0) display.addWidget (imContext, this);
 }
 
@@ -1074,8 +1106,8 @@ void releaseHandle () {
 }
 
 void releaseWidget () {
-	fixIM ();
 	super.releaseWidget ();
+	fixIM ();
 }
 
 /**
@@ -1103,10 +1135,10 @@ public void remove (int index) {
 	System.arraycopy (oldItems, index + 1, newItems, index, oldItems.length - index - 1);
 	items = newItems;
 	ignoreSelect = true;
-	int /*long*/ children = OS.gtk_container_get_children (listHandle);
-	int /*long*/ item = OS.g_list_nth_data (children, index);
+	long /*int*/ children = OS.gtk_container_get_children (listHandle);
+	long /*int*/ item = OS.g_list_nth_data (children, index);
 	boolean selected = OS.GTK_WIDGET_STATE (item) == OS.GTK_STATE_SELECTED;
-	int /*long*/ items = OS.g_list_append (0, item);
+	long /*int*/ items = OS.g_list_append (0, item);
 	OS.gtk_list_remove_items (listHandle, items);
 	OS.g_list_free (items);
 	OS.g_list_free (children);
@@ -1145,10 +1177,10 @@ public void remove (int start, int end) {
 	items = newItems;
 	boolean selected = false;
 	ignoreSelect = true;
-	int /*long*/ items = 0;
-	int /*long*/ children = OS.gtk_container_get_children (listHandle);
+	long /*int*/ items = 0;
+	long /*int*/ children = OS.gtk_container_get_children (listHandle);
 	for (int i = start; i <= end; i++) {
-		int /*long*/ item = OS.g_list_nth_data (children, i);
+		long /*int*/ item = OS.g_list_nth_data (children, i);
 		selected |= OS.GTK_WIDGET_STATE (item) == OS.GTK_STATE_SELECTED;
 		items = OS.g_list_append (items, item);
 	}
@@ -1309,16 +1341,16 @@ int setBounds (int x, int y, int width, int height, boolean move, boolean resize
 	return super.setBounds (x, y, width, newHeight, move, resize);
 }
 
-void setFontDescription (int /*long*/ font) {
+void setFontDescription (long /*int*/ font) {
 	super.setFontDescription (font);
 	if (entryHandle != 0) OS.gtk_widget_modify_font (entryHandle, font);
 	if (listHandle != 0) {
 		OS.gtk_widget_modify_font (listHandle, font);
-		int /*long*/ itemsList = OS.gtk_container_get_children (listHandle);
+		long /*int*/ itemsList = OS.gtk_container_get_children (listHandle);
 		if (itemsList != 0) {
 			int count = OS.g_list_length (itemsList);
 			for (int i=count - 1; i>=0; i--) {
-				int /*long*/ widget = OS.gtk_bin_get_child (OS.g_list_nth_data (itemsList, i));
+				long /*int*/ widget = OS.gtk_bin_get_child (OS.g_list_nth_data (itemsList, i));
 				OS.gtk_widget_modify_font (widget, font);
 			}
 			OS.g_list_free (itemsList);
@@ -1331,11 +1363,11 @@ void setForegroundColor (GdkColor color) {
 	if (entryHandle != 0) OS.gtk_widget_modify_text (entryHandle, 0, color);
 	if (listHandle != 0) {
 		OS.gtk_widget_modify_text (listHandle, 0, color);
-		int /*long*/ itemsList = OS.gtk_container_get_children (listHandle);
+		long /*int*/ itemsList = OS.gtk_container_get_children (listHandle);
 		if (itemsList != 0) {
 			int count = OS.g_list_length (itemsList);
 			for (int i=count - 1; i>=0; i--) {
-				int /*long*/ widget = OS.gtk_bin_get_child (OS.g_list_nth_data (itemsList, i));
+				long /*int*/ widget = OS.gtk_bin_get_child (OS.g_list_nth_data (itemsList, i));
 				OS.gtk_widget_modify_fg (widget,  OS.GTK_STATE_NORMAL, color);
 			}
 			OS.g_list_free (itemsList);
@@ -1370,9 +1402,9 @@ public void setItem (int index, String string) {
 	items [index] = string;
 	ignoreSelect = true;
 	byte [] buffer = Converter.wcsToMbcs (null, string, true);
-	int /*long*/ children = OS.gtk_container_get_children (listHandle);
-	int /*long*/ item = OS.g_list_nth_data (children, index);
-	int /*long*/ label = OS.gtk_bin_get_child (item);
+	long /*int*/ children = OS.gtk_container_get_children (listHandle);
+	long /*int*/ item = OS.g_list_nth_data (children, index);
+	long /*int*/ label = OS.gtk_bin_get_child (item);
 	OS.gtk_label_set_text (label, buffer);
 	OS.g_list_free (children);
 	ignoreSelect = false;
@@ -1400,15 +1432,15 @@ public void setItems (String [] items) {
 	}
 	lockText = ignoreSelect = true;
 	OS.gtk_list_clear_items (listHandle, 0, -1);
-	int /*long*/ font = getFontDescription ();
+	long /*int*/ font = getFontDescription ();
 	GdkColor color = getForegroundColor ();
 	int direction = OS.gtk_widget_get_direction (handle);
 	int i = 0;
 	while (i < items.length) {
 		String string = items [i];
 		byte [] buffer = Converter.wcsToMbcs (null, string, true);
-		int /*long*/ item = OS.gtk_list_item_new_with_label (buffer);
-		int /*long*/ label = OS.gtk_bin_get_child (item); 
+		long /*int*/ item = OS.gtk_list_item_new_with_label (buffer);
+		long /*int*/ label = OS.gtk_bin_get_child (item); 
 		OS.gtk_widget_modify_fg (label, OS.GTK_STATE_NORMAL, color);
 		OS.gtk_widget_modify_font (label, font);
 		OS.gtk_widget_set_direction (label, direction);
@@ -1454,11 +1486,11 @@ public void setOrientation (int orientation) {
 	OS.gtk_widget_set_direction (handle, dir);
 	OS.gtk_widget_set_direction (listHandle, dir);
 	OS.gtk_widget_set_direction (entryHandle, dir);
-	int /*long*/ itemsList = OS.gtk_container_get_children (listHandle);
+	long /*int*/ itemsList = OS.gtk_container_get_children (listHandle);
 	if (itemsList != 0) {
 		int count = OS.g_list_length (itemsList);
 		for (int i=count - 1; i>=0; i--) {
-			int /*long*/ widget = OS.gtk_bin_get_child (OS.g_list_nth_data (itemsList, i));
+			long /*int*/ widget = OS.gtk_bin_get_child (OS.g_list_nth_data (itemsList, i));
 			OS.gtk_widget_set_direction (widget, dir);
 		}
 		OS.g_list_free (itemsList);
@@ -1522,7 +1554,7 @@ public void setText (String string) {
 	* fix is to block the firing of these events and fire them ourselves in a consistent manner. 
 	*/
 	if (hooks (SWT.Verify) || filters (SWT.Verify)) {
-		int /*long*/ ptr = OS.gtk_entry_get_text (entryHandle);
+		long /*int*/ ptr = OS.gtk_entry_get_text (entryHandle);
 		string = verifyText (string, 0, (int)/*64*/OS.g_utf8_strlen (ptr, -1));
 		if (string == null) return;
 	}
@@ -1563,6 +1595,11 @@ public void setTextLimit (int limit) {
 	OS.gtk_entry_set_max_length (entryHandle, limit);
 }
 
+void setToolTipText (Shell shell, String newString, String oldString) {
+	shell.setToolTipText (entryHandle, newString, oldString);
+	shell.setToolTipText (arrowHandle, newString, oldString);
+}
+
 /**
  * Sets the number of items that are visible in the drop
  * down portion of the receiver's list.
@@ -1591,9 +1628,9 @@ boolean translateTraversal (GdkEventKey keyEvent) {
 	switch (key) {
 		case OS.GDK_KP_Enter:
 		case OS.GDK_Return: {
-			int /*long*/ imContext = imContext (); 
+			long /*int*/ imContext = imContext (); 
 			if (imContext != 0) {
-				int /*long*/ [] preeditString = new int /*long*/ [1];
+				long /*int*/ [] preeditString = new long /*int*/ [1];
 				OS.gtk_im_context_get_preedit_string (imContext, preeditString, null, null);
 				if (preeditString [0] != 0) {
 					int length = OS.strlen (preeditString [0]);
@@ -1612,7 +1649,7 @@ String verifyText (String string, int start, int end) {
 	event.text = string;
 	event.start = start;
 	event.end = end;
-	int /*long*/ eventPtr = OS.gtk_get_current_event ();
+	long /*int*/ eventPtr = OS.gtk_get_current_event ();
 	if (eventPtr != 0) {
 		GdkEventKey gdkEvent = new GdkEventKey ();
 		OS.memmove (gdkEvent, eventPtr, GdkEventKey.sizeof);
