@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -31,6 +31,12 @@ import org.eclipse.swt.graphics.*;
  * behavior is undefined if they are used with subclasses of <code>Composite</code> other
  * than <code>Canvas</code>.
  * </p><p>
+ * Note: The <code>CENTER</code> style, although undefined for composites, has the
+ * same value as <code>EMBEDDED</code> (which is used to embed widgets from other
+ * widget toolkits into SWT).  On some operating systems (GTK, Motif), this may cause
+ * the children of this composite to be obscured.  The <code>EMBEDDED</code> style
+ * is for use by other widget toolkits and should normally never be used.
+ * </p><p>
  * This class may be subclassed by custom control implementors
  * who are building controls that are constructed from aggregates
  * of other controls.
@@ -39,11 +45,11 @@ import org.eclipse.swt.graphics.*;
  * @see Canvas
  */
 public class Composite extends Scrollable {
-	public int /*long*/  embeddedHandle;
-	int /*long*/ imHandle, socketHandle;
+	public long /*int*/  embeddedHandle;
+	long /*int*/ imHandle, socketHandle;
 	Layout layout;
 	Control[] tabList;
-	int layoutCount = 0;
+	int layoutCount, backgroundMode;
 
 	static final String NO_INPUT_METHOD = "org.eclipse.swt.internal.gtk.noInputMethod"; //$NON-NLS-1$
 
@@ -86,15 +92,15 @@ public Composite (Composite parent, int style) {
 }
 
 Control [] _getChildren () {
-	int /*long*/ parentHandle = parentingHandle ();
-	int /*long*/ list = OS.gtk_container_get_children (parentHandle);
+	long /*int*/ parentHandle = parentingHandle ();
+	long /*int*/ list = OS.gtk_container_get_children (parentHandle);
 	if (list == 0) return new Control [0];
 	int count = OS.g_list_length (list);
 	Control [] children = new Control [count];
 	int i = 0;
-	int /*long*/ temp = list;
+	long /*int*/ temp = list;
 	while (temp != 0) {
-		int /*long*/ handle = OS.g_list_data (temp);
+		long /*int*/ handle = OS.g_list_data (temp);
 		if (handle != 0) {
 			Widget widget = display.getWidget (handle);
 			if (widget != null && widget != this) {
@@ -177,10 +183,22 @@ public void changed (Control[] changed) {
 	}
 }
 
+void checkBuffered () {
+	if ((style & SWT.DOUBLE_BUFFERED) == 0 && (style & SWT.NO_BACKGROUND) != 0) {
+		return;
+	}
+	super.checkBuffered();
+}
+
 protected void checkSubclass () {
 	/* Do nothing - Subclassing is allowed */
 }
 
+long /*int*/ childStyle () {
+	if (scrolledHandle != 0) return 0;
+	return super.childStyle ();
+}
+
 public Point computeSize (int wHint, int hHint, boolean changed) {
 	checkWidget ();
 	if (wHint != SWT.DEFAULT && wHint < 0) wHint = 0;
@@ -224,18 +242,21 @@ Control [] computeTabList () {
 
 void createHandle (int index) {
 	state |= HANDLE | CANVAS;
-	boolean scrolled = (style & (SWT.H_SCROLL | SWT.V_SCROLL | SWT.BORDER)) != 0;
-	createHandle (index, scrolled);
+	boolean scrolled = (style & (SWT.H_SCROLL | SWT.V_SCROLL)) != 0;
+	if (!scrolled) state |= THEME_BACKGROUND;
+	createHandle (index, true, scrolled || (style & SWT.BORDER) != 0);
 }
 
-void createHandle (int index, boolean scrolled) {
+void createHandle (int index, boolean fixed, boolean scrolled) {
 	if (scrolled) {
-		fixedHandle = OS.g_object_new (display.gtk_fixed_get_type (), 0);
-		if (fixedHandle == 0) error (SWT.ERROR_NO_HANDLES);
-		OS.gtk_fixed_set_has_window (fixedHandle, true);
-		int /*long*/ vadj = OS.gtk_adjustment_new (0, 0, 100, 1, 10, 10);
+		if (fixed) {
+			fixedHandle = OS.g_object_new (display.gtk_fixed_get_type (), 0);
+			if (fixedHandle == 0) error (SWT.ERROR_NO_HANDLES);
+			OS.gtk_fixed_set_has_window (fixedHandle, true);
+		}
+		long /*int*/ vadj = OS.gtk_adjustment_new (0, 0, 100, 1, 10, 10);
 		if (vadj == 0) error (SWT.ERROR_NO_HANDLES);
-		int /*long*/ hadj = OS.gtk_adjustment_new (0, 0, 100, 1, 10, 10);
+		long /*int*/ hadj = OS.gtk_adjustment_new (0, 0, 100, 1, 10, 10);
 		if (hadj == 0) error (SWT.ERROR_NO_HANDLES);
 		scrolledHandle = OS.gtk_scrolled_window_new (hadj, vadj);
 		if (scrolledHandle == 0) SWT.error (SWT.ERROR_NO_HANDLES);
@@ -254,7 +275,7 @@ void createHandle (int index, boolean scrolled) {
 		}
 	}
 	if (scrolled) {
-		OS.gtk_container_add (fixedHandle, scrolledHandle);
+		if (fixed) OS.gtk_container_add (fixedHandle, scrolledHandle);
 		/*
 		* Force the scrolledWindow to have a single child that is
 		* not scrolled automatically.  Calling gtk_container_add()
@@ -286,11 +307,11 @@ void createHandle (int index, boolean scrolled) {
 	* background of the widget rather than the current contents of
 	* the screen.  If nothing is drawn during an expose event,
 	* the pixels are altered.  The fix is to clear double buffering
-	* and the DOUBLE_BUFFERED flag.
+	* when NO_BACKGROUND is set and DOUBLE_BUFFERED
+	* is not explicitly set.
 	*/
-	if ((style & SWT.NO_BACKGROUND) != 0) {
+	if ((style & SWT.DOUBLE_BUFFERED) == 0 && (style & SWT.NO_BACKGROUND) != 0) {
 		OS.gtk_widget_set_double_buffered (handle, false);
-		style &= ~SWT.DOUBLE_BUFFERED;
 	}
 }
 
@@ -299,11 +320,41 @@ void deregister () {
 	if (socketHandle != 0) display.removeWidget (socketHandle);
 }
 
+void drawBackground (GC gc, int x, int y, int width, int height) {
+	Control control = findBackgroundControl ();
+	if (control != null) {
+		long /*int*/ gdkGC = gc.handle;
+		GdkGCValues values = new GdkGCValues ();
+		OS.gdk_gc_get_values (gdkGC, values);
+		if (control.backgroundImage != null) {
+			Point pt = display.map (this, control, 0, 0);
+			OS.gdk_gc_set_fill (gdkGC, OS.GDK_TILED);
+			OS.gdk_gc_set_ts_origin (gdkGC, -pt.x, -pt.y);
+			OS.gdk_gc_set_tile (gdkGC, control.backgroundImage.pixmap);
+			gc.fillRectangle (x, y, width, height);
+			OS.gdk_gc_set_fill (gdkGC, values.fill);
+			OS.gdk_gc_set_ts_origin (gdkGC, values.ts_x_origin, values.ts_y_origin);
+		} else {
+			gc.setBackground (control.getBackground ());
+			gc.fillRectangle (x, y, width, height);
+			GdkColor color = new GdkColor ();
+			color.pixel = values.background_pixel;
+			OS.gdk_gc_set_background (gdkGC, color);
+		}
+	} else {
+		gc.fillRectangle (x, y, width, height);
+	}
+}
+
 void enableWidget (boolean enabled) {
 	if ((state & CANVAS) != 0) return;
 	super.enableWidget (enabled);
 }
 
+Composite findDeferredControl () {
+	return layoutCount > 0 ? this : parent.findDeferredControl ();
+}
+
 Menu [] findMenus (Control control) {
 	if (control == this) return new Menu [0];
 	Menu result [] = super.findMenus (control);
@@ -352,15 +403,15 @@ void fixTabList (Control control) {
 
 void fixZOrder () {
 	if ((state & CANVAS) != 0) return;
-	int /*long*/ parentHandle = parentingHandle ();
-	int /*long*/ parentWindow = OS.GTK_WIDGET_WINDOW (parentHandle);
+	long /*int*/ parentHandle = parentingHandle ();
+	long /*int*/ parentWindow = OS.GTK_WIDGET_WINDOW (parentHandle);
 	if (parentWindow == 0) return;
-	int /*long*/ [] userData = new int /*long*/ [1];
-	int /*long*/ windowList = OS.gdk_window_get_children (parentWindow);
+	long /*int*/ [] userData = new long /*int*/ [1];
+	long /*int*/ windowList = OS.gdk_window_get_children (parentWindow);
 	if (windowList != 0) {
-		int /*long*/ windows = windowList;
+		long /*int*/ windows = windowList;
 		while (windows != 0) {
-			int /*long*/ window = OS.g_list_data (windows);
+			long /*int*/ window = OS.g_list_data (windows);
 			if (window != redrawWindow) {
 				OS.gdk_window_get_user_data (window, userData);
 				if (userData [0] == 0 || OS.G_OBJECT_TYPE (userData [0]) != display.gtk_fixed_get_type ()) {
@@ -373,21 +424,46 @@ void fixZOrder () {
 	}
 }
 
-int /*long*/ focusHandle () {
+long /*int*/ focusHandle () {
 	if (socketHandle != 0) return socketHandle;
 	return super.focusHandle ();
 }
 
-boolean forceFocus (int /*long*/ focusHandle) {
+boolean forceFocus (long /*int*/ focusHandle) {
 	if (socketHandle != 0) OS.GTK_WIDGET_SET_FLAGS (focusHandle, OS.GTK_CAN_FOCUS);
 	boolean result = super.forceFocus (focusHandle);
 	if (socketHandle != 0) OS.GTK_WIDGET_UNSET_FLAGS (focusHandle, OS.GTK_CAN_FOCUS);
 	return result;
 }
 
+/**
+ * Returns the receiver's background drawing mode. This
+ * will be one of the following constants defined in class
+ * <code>SWT</code>:
+ * <code>INHERIT_NONE</code>, <code>INHERIT_DEFAULT</code>,
+ * <code>INHERTIT_FORCE</code>.
+ *
+ * @return the background mode
+ *
+ * @exception SWTException <ul>
+ *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
+ *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
+ * </ul>
+ *
+ * @see SWT
+ * 
+ * @since 3.2
+ */
+public int getBackgroundMode () {
+	checkWidget ();
+	return backgroundMode;
+}
+
 /**
  * Returns a (possibly empty) array containing the receiver's children.
- * Children are returned in the order that they are drawn.
+ * Children are returned in the order that they are drawn.  The topmost
+ * control appears at the beginning of the array.  Subsequent controls
+ * draw beneath this control and appear later in the array.
  * <p>
  * Note: This is not the actual structure used by the receiver
  * to maintain its list of children, so modifying the array will
@@ -414,7 +490,7 @@ int getChildrenCount () {
 	* NOTE: The current implementation will count
 	* non-registered children.
 	*/
-	int /*long*/ list = OS.gtk_container_get_children (handle);
+	long /*int*/ list = OS.gtk_container_get_children (handle);
 	if (list == 0) return 0;
 	int count = OS.g_list_length (list);
 	OS.g_list_free (list);
@@ -424,13 +500,13 @@ int getChildrenCount () {
 public Rectangle getClientArea () {
 	checkWidget();
 	if ((state & CANVAS) != 0) {
-		if ((state & ZERO_SIZED) != 0) {
+		if ((state & ZERO_WIDTH) != 0 && (state & ZERO_HEIGHT) != 0) {
 			return new Rectangle (0, 0, 0, 0);
 		}
 		forceResize ();
-		int /*long*/ clientHandle = clientHandle ();
-		int width = OS.GTK_WIDGET_WIDTH (clientHandle);
-		int height = OS.GTK_WIDGET_HEIGHT (clientHandle);
+		long /*int*/ clientHandle = clientHandle ();
+		int width = (state & ZERO_WIDTH) != 0 ? 0 : OS.GTK_WIDGET_WIDTH (clientHandle);
+		int height = (state & ZERO_HEIGHT) != 0 ? 0 : OS.GTK_WIDGET_HEIGHT (clientHandle);
 		return new Rectangle (0, 0, width, height);
 	}
 	return super.getClientArea();
@@ -505,8 +581,9 @@ public Control [] getTabList () {
 	return tabList;
 }
 
-int /*long*/ gtk_button_press_event (int /*long*/ widget, int /*long*/ event) {
-	int /*long*/ result = super.gtk_button_press_event (widget, event);
+long /*int*/ gtk_button_press_event (long /*int*/ widget, long /*int*/ event) {
+	long /*int*/ result = super.gtk_button_press_event (widget, event);
+	if (result != 0) return result;
 	if ((state & CANVAS) != 0) {
 		if ((style & SWT.NO_FOCUS) == 0 && hooksKeys ()) {
 			GdkEventButton gdkEvent = new GdkEventButton ();
@@ -519,7 +596,7 @@ int /*long*/ gtk_button_press_event (int /*long*/ widget, int /*long*/ event) {
 	return result;
 }
 
-int /*long*/ gtk_expose_event (int /*long*/ widget, int /*long*/ eventPtr) {
+long /*int*/ gtk_expose_event (long /*int*/ widget, long /*int*/ eventPtr) {
 	if ((state & OBSCURED) != 0) return 0;
 	if ((state & CANVAS) == 0) {
 		return super.gtk_expose_event (widget, eventPtr);
@@ -530,7 +607,7 @@ int /*long*/ gtk_expose_event (int /*long*/ widget, int /*long*/ eventPtr) {
 	if (!hooks (SWT.Paint) && !filters (SWT.Paint)) return 0;
 	GdkEventExpose gdkEvent = new GdkEventExpose ();
 	OS.memmove(gdkEvent, eventPtr, GdkEventExpose.sizeof);
-	int /*long*/ [] rectangles = new int /*long*/ [1];
+	long /*int*/ [] rectangles = new long /*int*/ [1];
 	int [] n_rectangles = new int [1];
 	OS.gdk_region_get_rectangles (gdkEvent.region, rectangles, n_rectangles);
 	GdkRectangle rect = new GdkRectangle ();
@@ -541,7 +618,7 @@ int /*long*/ gtk_expose_event (int /*long*/ widget, int /*long*/ eventPtr) {
 		event.y = rect.y;
 		event.width = rect.width;
 		event.height = rect.height;
-		int /*long*/ damageRgn = OS.gdk_region_new ();
+		long /*int*/ damageRgn = OS.gdk_region_new ();
 		OS.gdk_region_union_with_rect (damageRgn, rect);
 		GCData data = new GCData ();
 		data.damageRgn = damageRgn;
@@ -556,8 +633,8 @@ int /*long*/ gtk_expose_event (int /*long*/ widget, int /*long*/ eventPtr) {
 	return 0;
 }
 
-int /*long*/ gtk_key_press_event (int /*long*/ widget, int /*long*/ event) {
-	int /*long*/ result = super.gtk_key_press_event (widget, event);
+long /*int*/ gtk_key_press_event (long /*int*/ widget, long /*int*/ event) {
+	long /*int*/ result = super.gtk_key_press_event (widget, event);
 	if (result != 0) return result;
 	/*
 	* Feature in GTK.  The default behavior when the return key
@@ -577,30 +654,30 @@ int /*long*/ gtk_key_press_event (int /*long*/ widget, int /*long*/ event) {
 	return result;
 }
 
-int /*long*/ gtk_focus (int /*long*/ widget, int /*long*/ directionType) {
+long /*int*/ gtk_focus (long /*int*/ widget, long /*int*/ directionType) {
 	if (widget == socketHandle) return 0;
 	return super.gtk_focus (widget, directionType);
 }
 
-int /*long*/ gtk_focus_in_event (int /*long*/ widget, int /*long*/ event) {
-	int /*long*/ result = super.gtk_focus_in_event (widget, event);
+long /*int*/ gtk_focus_in_event (long /*int*/ widget, long /*int*/ event) {
+	long /*int*/ result = super.gtk_focus_in_event (widget, event);
 	return (state & CANVAS) != 0 ? 1 : result;
 }
 
-int /*long*/ gtk_focus_out_event (int /*long*/ widget, int /*long*/ event) {
-	int /*long*/ result = super.gtk_focus_out_event (widget, event);
+long /*int*/ gtk_focus_out_event (long /*int*/ widget, long /*int*/ event) {
+	long /*int*/ result = super.gtk_focus_out_event (widget, event);
 	return (state & CANVAS) != 0 ? 1 : result;
 }
 
-int /*long*/ gtk_map (int /*long*/ widget) {
+long /*int*/ gtk_map (long /*int*/ widget) {
 	fixZOrder ();
 	return 0;	
 }
 
-int /*long*/ gtk_realize (int /*long*/ widget) {
-	int /*long*/ result = super.gtk_realize (widget);
+long /*int*/ gtk_realize (long /*int*/ widget) {
+	long /*int*/ result = super.gtk_realize (widget);
 	if ((style & SWT.NO_BACKGROUND) != 0) {
-		int /*long*/ window = OS.GTK_WIDGET_WINDOW (paintHandle ());
+		long /*int*/ window = OS.GTK_WIDGET_WINDOW (paintHandle ());
 		if (window != 0) OS.gdk_window_set_back_pixmap (window, 0, false);
 	}
 	if (socketHandle != 0) {
@@ -609,18 +686,19 @@ int /*long*/ gtk_realize (int /*long*/ widget) {
 	return result;
 }
 
-int /*long*/ gtk_scroll_child (int /*long*/ widget, int /*long*/ scrollType, int /*long*/ horizontal) {
+long /*int*/ gtk_scroll_child (long /*int*/ widget, long /*int*/ scrollType, long /*int*/ horizontal) {
 	/* Stop GTK scroll child signal for canvas */
 	OS.g_signal_stop_emission_by_name (widget, OS.scroll_child);
 	return 1;
 }
 
-int /*long*/ gtk_style_set (int /*long*/ widget, int /*long*/ previousStyle) {
+long /*int*/ gtk_style_set (long /*int*/ widget, long /*int*/ previousStyle) {
+	long /*int*/ result = super.gtk_style_set (widget, previousStyle);
 	if ((style & SWT.NO_BACKGROUND) != 0) {
-		int /*long*/ window = OS.GTK_WIDGET_WINDOW (paintHandle ());
+		long /*int*/ window = OS.GTK_WIDGET_WINDOW (paintHandle ());
 		if (window != 0) OS.gdk_window_set_back_pixmap (window, 0, false);
 	}
-	return 0;
+	return result;
 }
 
 boolean hasBorder () {
@@ -632,10 +710,10 @@ void hookEvents () {
 	if ((state & CANVAS) != 0) {
 		OS.gtk_widget_add_events (handle, OS.GDK_POINTER_MOTION_HINT_MASK);
 		if (scrolledHandle != 0) {
-			OS.g_signal_connect (scrolledHandle, OS.scroll_child, display.windowProc4, SCROLL_CHILD);
+			OS.g_signal_connect_closure (scrolledHandle, OS.scroll_child, display.closures [SCROLL_CHILD], false);
 		}
-		if ((style & SWT.NO_BACKGROUND) != 0) {
-			OS.g_signal_connect (handle, OS.style_set, display.windowProc3, STYLE_SET);
+		if ((state & PARENT_BACKGROUND) == 0 && (style & SWT.NO_BACKGROUND) != 0) {
+			OS.g_signal_connect_closure_by_id (handle, display.signalIds [STYLE_SET], 0, display.closures [STYLE_SET], false);
 		}
 	}
 }
@@ -644,7 +722,7 @@ boolean hooksKeys () {
 	return hooks (SWT.KeyDown) || hooks (SWT.KeyUp);
 }
 
-int /*long*/ imHandle () {
+long /*int*/ imHandle () {
 	return imHandle;
 }
 
@@ -668,7 +746,7 @@ int /*long*/ imHandle () {
  */
 public boolean isLayoutDeferred () {
 	checkWidget ();
-	return layoutCount > 0 || parent.isLayoutDeferred ();
+	return findDeferredControl () != null;
 }
 
 boolean isTabGroup() {
@@ -836,16 +914,16 @@ void markLayout (boolean changed, boolean all) {
 	}
 }
 
-void moveAbove (int /*long*/ child, int /*long*/ sibling) {
+void moveAbove (long /*int*/ child, long /*int*/ sibling) {
 	if (child == sibling) return;
-	int /*long*/ parentHandle = parentingHandle ();
+	long /*int*/ parentHandle = parentingHandle ();
 	GtkFixed fixed = new GtkFixed ();
 	OS.memmove (fixed, parentHandle);
-	int /*long*/ children = fixed.children;
+	long /*int*/ children = fixed.children;
 	if (children == 0) return;
-	int /*long*/ [] data = new int /*long*/ [1];
-	int /*long*/ [] widget = new int /*long*/ [1];
-	int /*long*/ childData = 0, childLink = 0, siblingLink = 0, temp = children;
+	long /*int*/ [] data = new long /*int*/ [1];
+	long /*int*/ [] widget = new long /*int*/ [1];
+	long /*int*/ childData = 0, childLink = 0, siblingLink = 0, temp = children;
 	while (temp != 0) {
 		OS.memmove (data, temp, OS.PTR_SIZEOF);
 		OS.memmove (widget, data [0], OS.PTR_SIZEOF);
@@ -873,20 +951,20 @@ void moveAbove (int /*long*/ child, int /*long*/ sibling) {
 	OS.memmove (parentHandle, fixed);
 }
 
-void moveBelow (int /*long*/ child, int /*long*/ sibling) {
+void moveBelow (long /*int*/ child, long /*int*/ sibling) {
 	if (child == sibling) return;
-	int /*long*/ parentHandle = parentingHandle ();
+	long /*int*/ parentHandle = parentingHandle ();
 	if (sibling == 0 && parentHandle == fixedHandle) {
 		moveAbove (child, scrolledHandle != 0  ? scrolledHandle : handle);
 		return;
 	}
 	GtkFixed fixed = new GtkFixed ();
 	OS.memmove (fixed, parentHandle);
-	int /*long*/ children = fixed.children;
+	long /*int*/ children = fixed.children;
 	if (children == 0) return;
-	int /*long*/ [] data = new int /*long*/ [1];
-	int /*long*/ [] widget = new int /*long*/ [1];
-	int /*long*/ childData = 0, childLink = 0, siblingLink = 0, temp = children;
+	long /*int*/ [] data = new long /*int*/ [1];
+	long /*int*/ [] widget = new long /*int*/ [1];
+	long /*int*/ childData = 0, childLink = 0, siblingLink = 0, temp = children;
 	while (temp != 0) {
 		OS.memmove (data, temp, OS.PTR_SIZEOF);
 		OS.memmove (widget, data [0], OS.PTR_SIZEOF);
@@ -925,30 +1003,50 @@ Point minimumSize (int wHint, int hHint, boolean changed) {
 	return new Point (width, height);
 }
 
-int /*long*/ parentingHandle () {
+long /*int*/ parentingHandle () {
 	if ((state & CANVAS) != 0) return handle;
 	return fixedHandle != 0 ? fixedHandle : handle;
 }
 
+void redrawChildren () {
+	super.redrawChildren ();
+	Control [] children = _getChildren ();
+	for (int i = 0; i < children.length; i++) {
+		Control child = children [i];
+		if ((child.state & PARENT_BACKGROUND) != 0) {
+			child.redrawWidget (0, 0, 0, 0, true, false, true);
+			child.redrawChildren ();
+		}
+	}
+}
+
 void register () {
 	super.register ();
 	if (socketHandle != 0) display.addWidget (socketHandle, this);
 }
 
-void releaseChildren () {
+void releaseChildren (boolean destroy) {
 	Control [] children = _getChildren ();
 	for (int i=0; i<children.length; i++) {
 		Control child = children [i];
-		if (child != null && !child.isDisposed ()) child.releaseResources ();
+		if (child != null && !child.isDisposed ()) {
+			child.release (false);
+		}
 	}
+	super.releaseChildren (destroy);
+}
+
+void releaseHandle () {
+	super.releaseHandle ();
+	socketHandle = embeddedHandle = 0;
 }
 
 void releaseWidget () {
-	releaseChildren ();
 	super.releaseWidget ();
 	if (imHandle != 0) OS.g_object_unref (imHandle);
 	imHandle = 0;
 	layout = null;
+	tabList = null;
 }
 
 void removeControl (Control control) {
@@ -960,6 +1058,32 @@ void resizeHandle (int width, int height) {
 	if (socketHandle != 0) OS.gtk_widget_set_size_request (socketHandle, width, height);
 }
 
+/**
+ * Sets the background drawing mode to the argument which should
+ * be one of the following constants defined in class <code>SWT</code>:
+ * <code>INHERIT_NONE</code>, <code>INHERIT_DEFAULT</code>,
+ * <code>INHERIT_FORCE</code>.
+ *
+ * @param mode the new background mode
+ *
+ * @exception SWTException <ul>
+ *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
+ *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
+ * </ul>
+ *
+ * @see SWT
+ * 
+ * @since 3.2
+ */
+public void setBackgroundMode (int mode) {
+	checkWidget ();
+	backgroundMode = mode;
+	Control[] children = _getChildren ();
+	for (int i = 0; i < children.length; i++) {
+		children [i].updateBackgroundMode ();
+	}
+}
+
 int setBounds (int x, int y, int width, int height, boolean move, boolean resize) {
 	int result = super.setBounds (x, y, width, height, move, resize);
 	if ((result & RESIZED) != 0 && layout != null) {
@@ -1020,7 +1144,9 @@ public void setLayout (Layout layout) {
 public void setLayoutDeferred (boolean defer) {
 	if (!defer) {
 		if (--layoutCount == 0) {
-			if (!isLayoutDeferred ()) updateLayout (true);
+			if ((state & LAYOUT_CHILD) != 0 || (state & LAYOUT_NEEDED) != 0) {
+				updateLayout (true);
+			}
 		}
 	} else {
 		layoutCount++;
@@ -1098,6 +1224,7 @@ void showWidget () {
 		OS.gtk_widget_show (socketHandle);
 		embeddedHandle = OS.gtk_socket_get_id (socketHandle);
 	}
+	if (scrolledHandle == 0) fixStyle (handle);
 }
 
 boolean translateMnemonic (Event event, Control control) {
@@ -1125,14 +1252,27 @@ boolean translateTraversal (GdkEventKey keyEvent) {
 	return super.translateTraversal (keyEvent);
 }
 
+void updateBackgroundMode () {
+	super.updateBackgroundMode ();
+	Control [] children = _getChildren ();
+	for (int i = 0; i < children.length; i++) {
+		children [i].updateBackgroundMode ();
+	}
+}
+
 void updateLayout (boolean all) {
-	if (isLayoutDeferred ()) return;
+	Composite parent = findDeferredControl ();
+	if (parent != null) {
+		parent.state |= LAYOUT_CHILD;
+		return;
+	}
 	if ((state & LAYOUT_NEEDED) != 0) {
 		boolean changed = (state & LAYOUT_CHANGED) != 0;
 		state &= ~(LAYOUT_NEEDED | LAYOUT_CHANGED);
 		layout.layout (this, changed);
 	}
 	if (all) {
+		state &= ~LAYOUT_CHILD;
 		Control [] children = _getChildren ();
 		for (int i=0; i<children.length; i++) {
 			children [i].updateLayout (all);
