@@ -38,7 +38,7 @@ import org.eclipse.swt.internal.gtk.*;
 public class MessageBox extends Dialog {
 
 	String message = "";
-	long /*int*/ handle;
+	int /*long*/ handle;
 /**
  * Constructs a new instance of this class given only its parent.
  *
@@ -124,7 +124,7 @@ public void setMessage (String string) {
  * </ul>
  */
 public int open () {
-	long /*int*/ parentHandle = (parent != null) ? parent.topHandle() : 0;
+	int /*long*/ parentHandle = (parent != null) ? parent.topHandle() : 0;
 	int dialogFlags = OS.GTK_DIALOG_DESTROY_WITH_PARENT;
 	if ((style & (SWT.PRIMARY_MODAL | SWT.APPLICATION_MODAL | SWT.SYSTEM_MODAL)) != 0) {
 		dialogFlags |= OS.GTK_DIALOG_MODAL;
@@ -138,7 +138,7 @@ public int open () {
 	handle = OS.gtk_message_dialog_new(parentHandle, dialogFlags, messageType, 0, buffer);
 	if (handle == 0) SWT.error(SWT.ERROR_NO_HANDLES);
 	if (parentHandle != 0) {
-		long /*int*/ pixbufs = OS.gtk_window_get_icon_list (parentHandle);
+		int /*long*/ pixbufs = OS.gtk_window_get_icon_list (parentHandle);
 		if (pixbufs != 0) {
 			OS.gtk_window_set_icon_list (handle, pixbufs);
 			OS.g_list_free (pixbufs);
