@@ -83,6 +83,7 @@ public class ScrollBar extends Widget {
 	Scrollable parent;
 	long /*int*/ adjustmentHandle;
 	int detail;
+	boolean dragSent;
 	
 ScrollBar () {
 }
@@ -340,6 +341,14 @@ public boolean getVisible () {
 	}
 }
 
+long /*int*/ gtk_button_press_event (long /*int*/ widget, long /*int*/ eventPtr) {
+	long /*int*/ result = super.gtk_button_press_event (widget, eventPtr);
+	if (result != 0) return result;
+	detail = OS.GTK_SCROLL_NONE;
+	dragSent = false;	
+	return result;
+}
+
 long /*int*/ gtk_change_value (long /*int*/ widget, long /*int*/ scroll, long /*int*/ value1, long /*int*/ value2) {
 	detail = (int)/*64*/scroll;
 	return 0;
@@ -347,6 +356,7 @@ long /*int*/ gtk_change_value (long /*int*/ widget, long /*int*/ scroll, long /*
 
 long /*int*/ gtk_value_changed (long /*int*/ adjustment) {
 	Event event = new Event ();
+	dragSent = detail == OS.GTK_SCROLL_JUMP;
 	switch (detail) {
 		case OS.GTK_SCROLL_NONE:			event.detail = SWT.NONE; break;
 		case OS.GTK_SCROLL_JUMP:			event.detail = SWT.DRAG; break;
@@ -366,17 +376,43 @@ long /*int*/ gtk_value_changed (long /*int*/ adjustment) {
 		case OS.GTK_SCROLL_STEP_BACKWARD:	event.detail = SWT.ARROW_UP; break;
 	}
 	detail = OS.GTK_SCROLL_NONE;
+	if (!dragSent) detail = OS.GTK_SCROLL_NONE;
 	postEvent (SWT.Selection, event);
 	parent.updateScrollBarValue (this);
 	return 0;
 }
 
+long /*int*/ gtk_event_after (long /*int*/ widget, long /*int*/ gdkEvent) {
+	GdkEvent gtkEvent = new GdkEvent ();
+	OS.memmove (gtkEvent, gdkEvent, GdkEvent.sizeof);
+	switch (gtkEvent.type) {
+		case OS.GDK_BUTTON_RELEASE: {
+			GdkEventButton gdkEventButton = new GdkEventButton ();
+			OS.memmove (gdkEventButton, gdkEvent, GdkEventButton.sizeof);
+			if (gdkEventButton.button == 1 && detail == SWT.DRAG) {
+				if (!dragSent) {
+					Event event = new Event ();
+					event.detail = SWT.DRAG;
+					postEvent (SWT.Selection, event);
+				}
+				postEvent (SWT.Selection);
+			}
+			detail = OS.GTK_SCROLL_NONE;
+			dragSent = false;
+			break;
+		}
+	}
+	return super.gtk_event_after (widget, gdkEvent);
+}
+
 void hookEvents () {
 	super.hookEvents ();
 	if (OS.GTK_VERSION >= OS.VERSION (2, 6, 0)) {
 		OS.g_signal_connect_closure (handle, OS.change_value, display.closures [CHANGE_VALUE], false);
 	}
 	OS.g_signal_connect_closure (adjustmentHandle, OS.value_changed, display.closures [VALUE_CHANGED], false);
+	OS.g_signal_connect_closure_by_id (handle, display.signalIds [EVENT_AFTER], 0, display.closures [EVENT_AFTER], false);
+	OS.g_signal_connect_closure_by_id (handle, display.signalIds [BUTTON_PRESS_EVENT], 0, display.closures [BUTTON_PRESS_EVENT], false);	
 }
 
 /**
