@@ -66,7 +66,7 @@ import org.eclipse.swt.events.*;
  */
 public class Slider extends Control {
 	int detail;
-
+	boolean dragSent;
 /**
  * Constructs a new instance of this class given its parent
  * and a style value describing its behavior and appearance.
@@ -149,7 +149,7 @@ void createHandle (int index) {
 	fixedHandle = OS.g_object_new (display.gtk_fixed_get_type (), 0);
 	if (fixedHandle == 0) error (SWT.ERROR_NO_HANDLES);
 	OS.gtk_fixed_set_has_window (fixedHandle, true);
-	long /*int*/ hAdjustment = OS.gtk_adjustment_new (0, 0, 100, 1, 10, 10);
+	int /*long*/ hAdjustment = OS.gtk_adjustment_new (0, 0, 100, 1, 10, 10);
 	if (hAdjustment == 0) error (SWT.ERROR_NO_HANDLES);
 	if ((style & SWT.HORIZONTAL) != 0) {
 		handle = OS.gtk_hscrollbar_new (hAdjustment);
@@ -161,13 +161,22 @@ void createHandle (int index) {
 	OS.gtk_container_add (fixedHandle, handle);
 }
 
-long /*int*/ gtk_change_value (long /*int*/ widget, long /*int*/ scroll, long /*int*/ value1, long /*int*/ value2) {
+int /*long*/ gtk_button_press_event (int /*long*/ widget, int /*long*/ eventPtr) {
+	int /*long*/ result = super.gtk_button_press_event (widget, eventPtr);
+	if (result != 0) return result;
+	detail = OS.GTK_SCROLL_NONE;
+	dragSent = false;
+	return result;
+}
+
+int /*long*/ gtk_change_value (int /*long*/ widget, int /*long*/ scroll, int /*long*/ value1, int /*long*/ value2) {
 	detail = (int)/*64*/scroll;
 	return 0;
 }
 
-long /*int*/ gtk_value_changed (long /*int*/ adjustment) {
+int /*long*/ gtk_value_changed (int /*long*/ adjustment) {
 	Event event = new Event ();
+	dragSent = detail == OS.GTK_SCROLL_JUMP;
 	switch (detail) {
 		case OS.GTK_SCROLL_NONE:			event.detail = SWT.NONE; break;
 		case OS.GTK_SCROLL_JUMP:			event.detail = SWT.DRAG; break;
@@ -186,11 +195,34 @@ long /*int*/ gtk_value_changed (long /*int*/ adjustment) {
 		case OS.GTK_SCROLL_STEP_LEFT:
 		case OS.GTK_SCROLL_STEP_BACKWARD:	event.detail = SWT.ARROW_UP; break;
 	}
-	detail = OS.GTK_SCROLL_NONE;
+	if (!dragSent) detail = OS.GTK_SCROLL_NONE;
 	postEvent (SWT.Selection, event);
 	return 0;
 }
 
+int /*long*/ gtk_event_after (int /*long*/ widget, int /*long*/ gdkEvent) {
+	GdkEvent gtkEvent = new GdkEvent ();
+	OS.memmove (gtkEvent, gdkEvent, GdkEvent.sizeof);
+	switch (gtkEvent.type) {
+		case OS.GDK_BUTTON_RELEASE: {
+			GdkEventButton gdkEventButton = new GdkEventButton ();
+			OS.memmove (gdkEventButton, gdkEvent, GdkEventButton.sizeof);
+			if (gdkEventButton.button == 1 && detail == SWT.DRAG) {
+				if (!dragSent) {
+					Event event = new Event ();
+					event.detail = SWT.DRAG;
+					postEvent (SWT.Selection, event);
+				}
+				postEvent (SWT.Selection);
+			}
+			detail = OS.GTK_SCROLL_NONE;
+			dragSent = false;
+			break;
+		}
+	}
+	return super.gtk_event_after (widget, gdkEvent);
+}
+
 void hookEvents () {
 	super.hookEvents ();
 	if (OS.GTK_VERSION >= OS.VERSION (2, 6, 0)) {
@@ -201,13 +233,13 @@ void hookEvents () {
 
 void register () {
 	super.register ();
-	long /*int*/ hAdjustment = OS.gtk_range_get_adjustment (handle);
+	int /*long*/ hAdjustment = OS.gtk_range_get_adjustment (handle);
 	display.addWidget (hAdjustment, this);
 }
 
 void deregister () {
 	super.deregister ();
-	long /*int*/ hAdjustment = OS.gtk_range_get_adjustment (handle);
+	int /*long*/ hAdjustment = OS.gtk_range_get_adjustment (handle);
 	display.removeWidget (hAdjustment);
 }
 
@@ -238,7 +270,7 @@ public Point computeSize (int wHint, int hHint, boolean changed) {
  */
 public int getIncrement () {
 	checkWidget ();
-	long /*int*/ hAdjustment = OS.gtk_range_get_adjustment (handle);
+	int /*long*/ hAdjustment = OS.gtk_range_get_adjustment (handle);
 	GtkAdjustment adjustment = new GtkAdjustment ();
 	OS.memmove (adjustment, hAdjustment);
 	return (int) adjustment.step_increment;
@@ -256,7 +288,7 @@ public int getIncrement () {
  */
 public int getMaximum () {
 	checkWidget ();
-	long /*int*/ hAdjustment = OS.gtk_range_get_adjustment (handle);
+	int /*long*/ hAdjustment = OS.gtk_range_get_adjustment (handle);
 	GtkAdjustment adjustment = new GtkAdjustment ();
 	OS.memmove (adjustment, hAdjustment);
 	return (int) adjustment.upper;
@@ -274,7 +306,7 @@ public int getMaximum () {
  */
 public int getMinimum () {
 	checkWidget ();
-	long /*int*/ hAdjustment = OS.gtk_range_get_adjustment (handle);
+	int /*long*/ hAdjustment = OS.gtk_range_get_adjustment (handle);
 	GtkAdjustment adjustment = new GtkAdjustment ();
 	OS.memmove (adjustment, hAdjustment);
 	return (int) adjustment.lower;
@@ -294,7 +326,7 @@ public int getMinimum () {
  */
 public int getPageIncrement () {
 	checkWidget ();
-	long /*int*/ hAdjustment = OS.gtk_range_get_adjustment (handle);
+	int /*long*/ hAdjustment = OS.gtk_range_get_adjustment (handle);
 	GtkAdjustment adjustment = new GtkAdjustment ();
 	OS.memmove (adjustment, hAdjustment);
 	return (int) adjustment.page_increment;
@@ -312,7 +344,7 @@ public int getPageIncrement () {
  */
 public int getSelection () {
 	checkWidget ();
-	long /*int*/ hAdjustment = OS.gtk_range_get_adjustment (handle);
+	int /*long*/ hAdjustment = OS.gtk_range_get_adjustment (handle);
 	GtkAdjustment adjustment = new GtkAdjustment ();
 	OS.memmove (adjustment, hAdjustment);
 	return (int) adjustment.value;
@@ -331,7 +363,7 @@ public int getSelection () {
  */
 public int getThumb () {
 	checkWidget ();
-	long /*int*/ hAdjustment = OS.gtk_range_get_adjustment (handle);
+	int /*long*/ hAdjustment = OS.gtk_range_get_adjustment (handle);
 	GtkAdjustment adjustment = new GtkAdjustment ();
 	OS.memmove (adjustment, hAdjustment);
 	return (int) adjustment.page_size;
@@ -398,7 +430,7 @@ public void setIncrement (int value) {
  */
 public void setMaximum (int value) {
 	checkWidget ();
-	long /*int*/ hAdjustment = OS.gtk_range_get_adjustment (handle);
+	int /*long*/ hAdjustment = OS.gtk_range_get_adjustment (handle);
 	GtkAdjustment adjustment = new GtkAdjustment ();
 	OS.memmove (adjustment, hAdjustment);
 	int minimum = (int) adjustment.lower;
@@ -428,7 +460,7 @@ public void setMaximum (int value) {
 public void setMinimum (int value) {
 	checkWidget ();
 	if (value < 0) return;
-	long /*int*/ hAdjustment = OS.gtk_range_get_adjustment (handle);
+	int /*long*/ hAdjustment = OS.gtk_range_get_adjustment (handle);
 	GtkAdjustment adjustment = new GtkAdjustment ();
 	OS.memmove (adjustment, hAdjustment);
 	int maximum = (int) adjustment.upper;
@@ -508,7 +540,7 @@ public void setSelection (int value) {
 public void setThumb (int value) {
 	checkWidget ();
 	if (value < 1) return;
-	long /*int*/ hAdjustment = OS.gtk_range_get_adjustment (handle);
+	int /*long*/ hAdjustment = OS.gtk_range_get_adjustment (handle);
 	GtkAdjustment adjustment = new GtkAdjustment ();
 	OS.memmove (adjustment, hAdjustment);
 	value = (int) Math.min (value, (int)(adjustment.upper - adjustment.lower));
@@ -549,7 +581,7 @@ public void setValues (int selection, int minimum, int maximum, int thumb, int i
 	if (increment < 1) return;
 	if (pageIncrement < 1) return;
 	thumb = Math.min (thumb, maximum - minimum);
-	long /*int*/ hAdjustment = OS.gtk_range_get_adjustment (handle);
+	int /*long*/ hAdjustment = OS.gtk_range_get_adjustment (handle);
 	GtkAdjustment adjustment = new GtkAdjustment ();
 	OS.memmove (adjustment, hAdjustment);
 	adjustment.value = Math.min (Math.max (selection, minimum), maximum - thumb);
