@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
  * http://www.eclipse.org/legal/epl-v10.html
- * 
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -18,8 +18,8 @@ import java.util.List;
 import org.eclipse.compare.CompareConfiguration;
 import org.eclipse.compare.structuremergeviewer.IDiffContainer;
 import org.eclipse.compare.structuremergeviewer.IDiffElement;
-import org.eclipse.core.internal.resources.mapping.*;
 import org.eclipse.core.resources.IResource;
+import org.eclipse.core.resources.mapping.*;
 import org.eclipse.core.runtime.*;
 import org.eclipse.core.runtime.jobs.Job;
 import org.eclipse.jface.action.IAction;
@@ -32,16 +32,25 @@ import org.eclipse.jface.viewers.StructuredViewer;
 import org.eclipse.osgi.util.NLS;
 import org.eclipse.swt.custom.BusyIndicator;
 import org.eclipse.swt.widgets.*;
+import org.eclipse.team.core.RepositoryProvider;
 import org.eclipse.team.core.TeamException;
+import org.eclipse.team.core.diff.IDiff;
+import org.eclipse.team.core.diff.IThreeWayDiff;
+import org.eclipse.team.core.history.IFileHistoryProvider;
+import org.eclipse.team.core.history.IFileRevision;
+import org.eclipse.team.core.mapping.IResourceDiff;
+import org.eclipse.team.core.mapping.ISynchronizationScope;
 import org.eclipse.team.core.synchronize.FastSyncInfoFilter;
 import org.eclipse.team.core.synchronize.SyncInfo;
 import org.eclipse.team.core.variants.IResourceVariant;
+import org.eclipse.team.internal.core.mapping.CompoundResourceTraversal;
 import org.eclipse.team.internal.ui.synchronize.SyncInfoModelElement;
 import org.eclipse.team.ui.TeamImages;
 import org.eclipse.team.ui.TeamUI;
+import org.eclipse.team.ui.mapping.ISynchronizationCompareAdapter;
 import org.eclipse.team.ui.synchronize.*;
 import org.eclipse.ui.*;
-import org.eclipse.ui.internal.LegacyResourceSupport;
+import org.eclipse.ui.ide.IContributorResourceAdapter2;
 import org.eclipse.ui.progress.IWorkbenchSiteProgressService;
 
 public class Utils {
@@ -117,14 +126,14 @@ public class Utils {
 	
 	/**
 	 * Shows the given errors to the user.
-	 * @param Exception
+	 * @param shell
+	 *            the shell to open the error dialog in
+	 * @param exception
 	 *            the exception containing the error
 	 * @param title
 	 *            the title of the error dialog
 	 * @param message
 	 *            the message for the error dialog
-	 * @param shell
-	 *            the shell to open the error dialog in
 	 */
 	public static void handleError(Shell shell, Exception exception, String title, String message) {
 		IStatus status = null;
@@ -148,7 +157,7 @@ public class Utils {
 			} else if (t instanceof InterruptedException) {
 				return;
 			} else {
-				status = new Status(IStatus.ERROR, TeamUIPlugin.ID, 1, TeamUIMessages.TeamAction_internal, t); //$NON-NLS-1$
+				status = new Status(IStatus.ERROR, TeamUIPlugin.ID, 1, TeamUIMessages.TeamAction_internal, t); 
 				log = true;
 				dialog = true;
 			}
@@ -259,13 +268,13 @@ public class Utils {
 				} else if (t instanceof TeamException) {
 					error = ((TeamException) t).getStatus();
 				} else {
-					error = new Status(IStatus.ERROR, TeamUIPlugin.ID, 1, TeamUIMessages.simpleInternal, t); //$NON-NLS-1$
+					error = new Status(IStatus.ERROR, TeamUIPlugin.ID, 1, TeamUIMessages.simpleInternal, t); 
 				}
 				Shell shell = new Shell(Display.getDefault());
 				if (error.getSeverity() == IStatus.INFO) {
-					MessageDialog.openInformation(shell, TeamUIMessages.information, error.getMessage()); //$NON-NLS-1$
+					MessageDialog.openInformation(shell, TeamUIMessages.information, error.getMessage()); 
 				} else {
-					ErrorDialog.openError(shell, TeamUIMessages.exception, null, error); //$NON-NLS-1$
+					ErrorDialog.openError(shell, TeamUIMessages.exception, null, error); 
 				}
 				shell.dispose();
 				// Let's log non-team exceptions
@@ -298,21 +307,147 @@ public class Utils {
 		final IResourceVariant base = sync.getBase();
 		String localContentId = sync.getLocalContentIdentifier();
 		if (localContentId != null) {
-			config.setLeftLabel(NLS.bind(TeamUIMessages.SyncInfoCompareInput_localLabelExists, new String[] { localContentId })); //$NON-NLS-1$
+			config.setLeftLabel(NLS.bind(TeamUIMessages.SyncInfoCompareInput_localLabelExists, new String[] { localContentId })); 
 		} else {
-			config.setLeftLabel(TeamUIMessages.SyncInfoCompareInput_localLabel); //$NON-NLS-1$
+			config.setLeftLabel(TeamUIMessages.SyncInfoCompareInput_localLabel); 
 		}
 		if (remote != null) {
-			config.setRightLabel(NLS.bind(TeamUIMessages.SyncInfoCompareInput_remoteLabelExists, new String[] { remote.getContentIdentifier() })); //$NON-NLS-1$
+			config.setRightLabel(NLS.bind(TeamUIMessages.SyncInfoCompareInput_remoteLabelExists, new String[] { remote.getContentIdentifier() })); 
 		} else {
-			config.setRightLabel(TeamUIMessages.SyncInfoCompareInput_remoteLabel); //$NON-NLS-1$
+			config.setRightLabel(TeamUIMessages.SyncInfoCompareInput_remoteLabel); 
 		}
 		if (base != null) {
-			config.setAncestorLabel(NLS.bind(TeamUIMessages.SyncInfoCompareInput_baseLabelExists, new String[] { base.getContentIdentifier() })); //$NON-NLS-1$
+			config.setAncestorLabel(NLS.bind(TeamUIMessages.SyncInfoCompareInput_baseLabelExists, new String[] { base.getContentIdentifier() })); 
 		} else {
-			config.setAncestorLabel(TeamUIMessages.SyncInfoCompareInput_baseLabel); //$NON-NLS-1$
+			config.setAncestorLabel(TeamUIMessages.SyncInfoCompareInput_baseLabel); 
 		}
 	}
+	
+	public static void updateLabels(IDiff diff, CompareConfiguration config, IProgressMonitor monitor) {
+		final IFileRevision remote = getRemote(diff);
+		final IFileRevision base = getBase(diff);
+		String localContentId = getLocalContentId(diff);
+		if (localContentId != null) {
+			config.setLeftLabel(NLS.bind(TeamUIMessages.SyncInfoCompareInput_localLabelExists, new String[] { localContentId })); 
+		} else {
+			config.setLeftLabel(TeamUIMessages.SyncInfoCompareInput_localLabel); 
+		}
+		if (remote != null) {
+			config.setRightLabel(NLS.bind(TeamUIMessages.SyncInfoCompareInput_remoteLabelExists, new String[] { remote.getContentIdentifier() })); 
+		} else {
+			config.setRightLabel(TeamUIMessages.SyncInfoCompareInput_remoteLabel); 
+		}
+		if (base != null) {
+			config.setAncestorLabel(NLS.bind(TeamUIMessages.SyncInfoCompareInput_baseLabelExists, new String[] { base.getContentIdentifier() })); 
+		} else {
+			config.setAncestorLabel(TeamUIMessages.SyncInfoCompareInput_baseLabel); 
+		}
+		updateLabelsWithAuthor(diff, config, monitor);
+	}
+	
+    public static void updateLabelsWithAuthor(IDiff diff, CompareConfiguration config, IProgressMonitor monitor) {
+        // Add the author to the remote or base
+        if (TeamUIPlugin.getPlugin().getPreferenceStore().getBoolean(IPreferenceIds.SHOW_AUTHOR_IN_COMPARE_EDITOR)) {
+	        if (diff != null) {
+	    		IFileRevision remote = getRemote(diff);
+	    		IFileRevision base = getBase(diff);
+	    		String remoteAuthor = null;
+	    		if (remote != null) {
+                    remote = getRevisionWithAuthor(remote, monitor);
+                    remoteAuthor = remote.getAuthor();
+                    if (remoteAuthor != null)
+                    	config.setRightLabel(NLS.bind(TeamUIMessages.Utils_21, new String[] { remote.getContentIdentifier(), remoteAuthor })); 
+	    		}
+	    		if (base != null) {
+                    String baseAuthor;
+                    if (remoteAuthor != null && remote.getContentIdentifier().equals(base.getContentIdentifier())) {
+                        baseAuthor = remoteAuthor;
+                    } else {
+                    	base = getRevisionWithAuthor(base, monitor);
+                        baseAuthor = base.getAuthor();
+                    }
+                    if (baseAuthor != null)
+                    	config.setAncestorLabel(NLS.bind(TeamUIMessages.Utils_20, new String[] { base.getContentIdentifier(), baseAuthor })); 
+	    		}
+	        }
+        }
+    }
+
+	private static IFileRevision getRevisionWithAuthor(IFileRevision remote, IProgressMonitor monitor) {
+		if (remote.getAuthor() == null && remote.isPropertyMissing()) {
+			try {
+				return remote.withAllProperties(monitor);
+			} catch (CoreException e) {
+				TeamUIPlugin.log(e);
+			}
+		}
+		return remote;
+	}
+
+	public static String getLocalContentId(IDiff diff) {
+		if (diff instanceof IThreeWayDiff) {
+			IThreeWayDiff twd = (IThreeWayDiff) diff;
+			diff = twd.getLocalChange();
+			if (diff == null)
+				diff = twd.getRemoteChange();
+		}
+		if (diff instanceof IResourceDiff) {
+			IResourceDiff rd = (IResourceDiff) diff;
+			IResource resource = rd.getResource();
+			IFileHistoryProvider provider = getHistoryProvider(resource);
+			if (provider != null) {
+				IFileRevision revision = provider.getWorkspaceFileRevision(resource);
+				if (revision != null)
+					return revision.getContentIdentifier();
+			}
+		}
+		return null;
+	}
+
+	public static IFileHistoryProvider getHistoryProvider(IResource resource) {
+		RepositoryProvider rp = RepositoryProvider.getProvider(resource.getProject());
+		if (rp != null)
+			return rp.getFileHistoryProvider();
+		return null;
+	}
+
+	public static IFileRevision getBase(IDiff diff) {
+		if (diff instanceof IThreeWayDiff) {
+			IThreeWayDiff twd = (IThreeWayDiff) diff;
+			IDiff remoteChange = twd.getRemoteChange();
+			if (remoteChange instanceof IResourceDiff) {
+				IResourceDiff rd = (IResourceDiff) remoteChange;
+				return rd.getBeforeState();
+			}
+			IDiff localChange = twd.getLocalChange();
+			if (localChange instanceof IResourceDiff) {
+				IResourceDiff ld = (IResourceDiff) localChange;
+				return ld.getBeforeState();
+			}
+		}
+		return null;
+	}
+
+	public static IFileRevision getRemote(IDiff diff) {
+		if (diff instanceof IResourceDiff) {
+			IResourceDiff rd = (IResourceDiff) diff;
+			return rd.getAfterState();
+		}
+		if (diff instanceof IThreeWayDiff) {
+			IThreeWayDiff twd = (IThreeWayDiff) diff;
+			IDiff remoteChange = twd.getRemoteChange();
+			if (remoteChange instanceof IResourceDiff) {
+				IResourceDiff rd = (IResourceDiff) remoteChange;
+				return rd.getAfterState();
+			}
+			IDiff localChange = twd.getLocalChange();
+			if (localChange instanceof IResourceDiff) {
+				IResourceDiff ld = (IResourceDiff) localChange;
+				return ld.getBeforeState();
+			}
+		}
+		return null;
+	}
 
 	/**
 	 * Initialize the given Action from a ResourceBundle.
@@ -376,15 +511,15 @@ public class Utils {
 	public static String modeToString(int mode) {
 		switch (mode) {
 			case ISynchronizePageConfiguration.INCOMING_MODE :
-				return TeamUIMessages.Utils_22; //$NON-NLS-1$
+				return TeamUIMessages.Utils_22; 
 			case ISynchronizePageConfiguration.OUTGOING_MODE :
-				return TeamUIMessages.Utils_23; //$NON-NLS-1$
+				return TeamUIMessages.Utils_23; 
 			case ISynchronizePageConfiguration.BOTH_MODE :
-				return TeamUIMessages.Utils_24; //$NON-NLS-1$
+				return TeamUIMessages.Utils_24; 
 			case ISynchronizePageConfiguration.CONFLICTING_MODE :
-				return TeamUIMessages.Utils_25; //$NON-NLS-1$
+				return TeamUIMessages.Utils_25; 
 		}
-		return TeamUIMessages.Utils_26; //$NON-NLS-1$
+		return TeamUIMessages.Utils_26; 
 	}
 
 	/**
@@ -392,7 +527,7 @@ public class Utils {
 	 * @param elements
 	 * @return the list of resources contained in the given elements.
 	 */
-	private static IResource[] getResources(Object[] elements, List nonResources, boolean isContributed) {
+	private static IResource[] getResources(Object[] elements, List nonResources, boolean isContributed, boolean includeMappingResources) {
 		List resources = new ArrayList();
 		for (int i = 0; i < elements.length; i++) {
 			Object element = elements[i];
@@ -407,12 +542,14 @@ public class Utils {
                     isResource = true;
                 }
             } else if (element instanceof ResourceMapping) {
-                isResource = true;
-                getResources((ResourceMapping)element, resources);
-			} else {
+            	if (includeMappingResources) {
+	                isResource = true;
+	                getResources((ResourceMapping)element, resources);
+            	}
+			} else if (element != null) {
                 Object adapted;
                 if (isContributed) {
-                    adapted = LegacyResourceSupport.getAdaptedResource(element);
+                    adapted = getResource(element);
                 } else {
                     adapted = getAdapter(element, IResource.class);
                 }
@@ -424,11 +561,11 @@ public class Utils {
                     }
                 } else {
                     if (isContributed) {
-                        adapted = LegacyResourceSupport.getAdaptedContributorResourceMapping(element);
+                        adapted = getResourceMapping(element);
                     } else {
                         adapted = getAdapter(element, ResourceMapping.class);
                     }
-                    if (adapted instanceof ResourceMapping) {
+                    if (adapted instanceof ResourceMapping && includeMappingResources) {
                         isResource = true;
                         getResources((ResourceMapping) adapted, resources);
                     }
@@ -460,25 +597,42 @@ public class Utils {
 	
 	public static Object[] getNonResources(Object[] elements) {
 		List nonResources = new ArrayList();
-		getResources(elements, nonResources, false);
+		getResources(elements, nonResources, false, false);
 		return nonResources.toArray();
 	}
 	
 	public static IResource[] getResources(Object[] element) {
-		return getResources(element, null, false);
+		return getResources(element, null, false /* isContributed */, false /* includeMappingResources */);
 	}
     
     public static IResource[] getContributedResources(Object[] elements) {
-        return getResources(elements, null, true);
+        return getResources(elements, null, true /* isContributed */, true /* isIncudeMappings */);
     }
 	
-	public static Object getAdapter(Object element, Class adapter) {
+	public static Object getAdapter(Object element, Class adapterType, boolean load) {
+		if (adapterType.isInstance(element))
+			return element;
 		if (element instanceof IAdaptable) {
-			return ((IAdaptable) element).getAdapter(adapter);
-		} 
+			Object adapted = ((IAdaptable) element).getAdapter(adapterType);
+			if (adapterType.isInstance(adapted))
+				return adapted;
+		}
+		if (load) {
+			Object adapted = Platform.getAdapterManager().loadAdapter(element, adapterType.getName());
+			if (adapterType.isInstance(adapted))
+				return adapted;
+		} else {
+			Object adapted = Platform.getAdapterManager().getAdapter(element, adapterType);
+			if (adapterType.isInstance(adapted))
+				return adapted;
+		}
 		return null;
 	}
 	
+	public static Object getAdapter(Object element, Class adapterType) {
+		return getAdapter(element, adapterType, false);
+	}
+	
 	/**
 	 * Return whether any sync nodes in the given selection or their
 	 * descendants match the given filter.
@@ -613,6 +767,9 @@ public class Utils {
 	 * doesn't exceed the given width. This implementation
 	 * replaces characters in the center of the original string with an
 	 * ellipsis ("...").
+	 * @param maxWidth the maximum length for the text
+	 * @param textValue the text to be shortened
+	 * @return the shortened text
 	 */
 	public static String shortenText(int maxWidth, String textValue) {
 		int length = textValue.length();
@@ -671,6 +828,10 @@ public class Utils {
     public static void syncExec(final Runnable r, StructuredViewer v) {
 		if(v == null) return;
 		final Control ctrl = v.getControl();
+		syncExec(r, ctrl);
+    }
+
+	public static void syncExec(final Runnable r, final Control ctrl) {
 		if (ctrl != null && !ctrl.isDisposed()) {
 			ctrl.getDisplay().syncExec(new Runnable() {
 				public void run() {
@@ -680,7 +841,7 @@ public class Utils {
 				}
 			});
 		}
-    }
+	}
 
     public static SyncInfo getSyncInfo(ISynchronizeModelElement node) {
         if (node instanceof IAdaptable) {
@@ -688,4 +849,122 @@ public class Utils {
         }
         return null;
     }
+
+	public static ISynchronizationCompareAdapter getCompareAdapter(Object element) {
+		ModelProvider provider = getModelProvider(element);
+		if (provider != null) {
+			Object o = provider.getAdapter(ISynchronizationCompareAdapter.class);
+			if (o instanceof ISynchronizationCompareAdapter) {
+				return (ISynchronizationCompareAdapter) o;
+			}
+		}
+		return null;
+	}
+
+	public static ModelProvider getModelProvider(Object o) {
+		if (o instanceof ModelProvider) {
+			return (ModelProvider) o;	
+		}
+		ResourceMapping mapping = getResourceMapping(o);
+		if (mapping != null)
+			return mapping.getModelProvider();
+		return null;
+	}
+	
+	public static IResource getResource(Object o) {
+		IResource resource = null;
+		if (o instanceof IResource) {
+			resource = (IResource) o;
+		} else if (o instanceof IAdaptable) {
+			IAdaptable adaptable = (IAdaptable) o;
+			resource = (IResource)adaptable.getAdapter(IResource.class);
+			if (resource == null) {
+				IContributorResourceAdapter adapter = (IContributorResourceAdapter)adaptable.getAdapter(IContributorResourceAdapter.class);
+				if (adapter != null)
+					resource = adapter.getAdaptedResource(adaptable);
+			}
+		}
+		return resource;
+	}
+	
+	
+	public static ResourceMapping getResourceMapping(Object o) {
+		if (o instanceof ResourceMapping) {
+			return (ResourceMapping) o;
+		}
+		if (o instanceof IAdaptable) {
+			IAdaptable adaptable = (IAdaptable) o;
+			Object adapted = adaptable.getAdapter(ResourceMapping.class);
+			if (adapted instanceof ResourceMapping) {
+				return(ResourceMapping) adapted;
+			}
+			adapted = adaptable.getAdapter(IContributorResourceAdapter.class);
+			if (adapted instanceof IContributorResourceAdapter2) {
+				IContributorResourceAdapter2 cra = (IContributorResourceAdapter2) adapted;
+				return cra.getAdaptedResourceMapping(adaptable);
+			}
+		} else {
+			Object adapted = Platform.getAdapterManager().getAdapter(o, ResourceMapping.class);
+			if (adapted instanceof ResourceMapping) {
+				return(ResourceMapping) adapted;
+			}
+		}
+		return null;
+	}
+
+	public static ResourceMapping[] getResourceMappings(Object[] objects) {
+		List result = new ArrayList();
+		for (int i = 0; i < objects.length; i++) {
+			Object object = objects[i];
+			ResourceMapping mapping = getResourceMapping(object);
+			if (mapping != null)
+				result.add(mapping);
+		}
+		return (ResourceMapping[]) result.toArray(new ResourceMapping[result.size()]);
+	}
+
+	public static String getLabel(ResourceMapping mapping) {
+		ModelProvider provider = mapping.getModelProvider();
+		ISynchronizationCompareAdapter adapter = getCompareAdapter(provider);
+		if (adapter == null)
+			return ""; //$NON-NLS-1$
+		String pathString = adapter.getPathString(mapping);
+		if (pathString == null || pathString.length() == 0)
+			return adapter.getName(mapping);
+		return pathString;
+	}
+	
+	public static String getLabel(ModelProvider provider) {
+		ResourceMapping mapping = Utils.getResourceMapping(provider);
+		if (mapping != null) {
+			String base = Utils.getLabel(mapping);
+			if (base != null && base.length() > 0)
+				return base;
+		}
+		return provider.getDescriptor().getLabel();
+	}
+
+	public static String getScopeDescription(ISynchronizationScope scope) {
+		ResourceMapping[] mappings = scope.getInputMappings();
+		if (mappings.length == 1) {
+			String label = getLabel(mappings[0]);
+			if (label == null)
+				return TeamUIMessages.Utils_19;
+			else
+				return label;
+		}
+		return NLS.bind(TeamUIMessages.Utils_18, new Integer(mappings.length));
+	}
+
+	public static ResourceTraversal[] getTraversals(Object[] elements) throws CoreException {
+		CompoundResourceTraversal traversal = new CompoundResourceTraversal();
+		for (int i = 0; i < elements.length; i++) {
+			Object object = elements[i];
+			ResourceMapping mapping = getResourceMapping(object);
+			if (mapping != null) {
+				traversal.addTraversals(mapping.getTraversals(ResourceMappingContext.LOCAL_CONTEXT, null));
+			}
+		}
+		return traversal.asTraversals();
+	}
 }
