@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -11,7 +11,8 @@
 
 package org.eclipse.ui.internal;
 
-import java.text.MessageFormat;
+import java.io.OutputStream;
+import com.ibm.icu.text.MessageFormat;
 import java.util.Locale;
 
 import org.eclipse.core.runtime.CoreException;
@@ -41,11 +42,10 @@ import org.eclipse.ui.internal.intro.IIntroRegistry;
 import org.eclipse.ui.internal.intro.IntroRegistry;
 import org.eclipse.ui.internal.misc.StatusUtil;
 import org.eclipse.ui.internal.operations.WorkbenchOperationSupport;
-import org.eclipse.ui.internal.part.components.services.IStatusFactory;
-import org.eclipse.ui.internal.part.services.StatusFactory;
 import org.eclipse.ui.internal.progress.ProgressManager;
 import org.eclipse.ui.internal.registry.ActionSetRegistry;
 import org.eclipse.ui.internal.registry.EditorRegistry;
+import org.eclipse.ui.internal.registry.IWorkbenchRegistryConstants;
 import org.eclipse.ui.internal.registry.PerspectiveRegistry;
 import org.eclipse.ui.internal.registry.PreferencePageRegistryReader;
 import org.eclipse.ui.internal.registry.ViewRegistry;
@@ -66,6 +66,8 @@ import org.eclipse.ui.wizards.IWizardRegistry;
 import org.osgi.framework.Bundle;
 import org.osgi.framework.BundleContext;
 import org.osgi.framework.BundleListener;
+import org.osgi.framework.InvalidSyntaxException;
+import org.osgi.framework.ServiceReference;
 
 /**
  * This class represents the TOP of the workbench UI world
@@ -93,10 +95,10 @@ public class WorkbenchPlugin extends AbstractUIPlugin {
 	private static final String ORIENTATION_COMMAND_LINE = "-dir";//$NON-NLS-1$
 	private static final String ORIENTATION_PROPERTY = "eclipse.orientation";//$NON-NLS-1$
 	private static final String NL_USER_PROPERTY = "osgi.nl.user"; //$NON-NLS-1$
+	private static final String UI_BUNDLE_ACTIVATOR = "org.eclipse.ui.internal.UIPlugin"; //$NON-NLS-1$
     
     // Default instance of the receiver
     private static WorkbenchPlugin inst;
-    private static StatusFactory statusFactory;
 
     // Manager that maps resources to descriptors of editors to use
     private EditorRegistry editorRegistry;
@@ -243,8 +245,9 @@ public class WorkbenchPlugin extends AbstractUIPlugin {
                     }
                 }
             });
-            if (exc[0] != null)
-                throw exc[0];
+            if (exc[0] != null) {
+				throw exc[0];
+			}
             return ret[0];
 
         } catch (CoreException core) {
@@ -316,11 +319,11 @@ public class WorkbenchPlugin extends AbstractUIPlugin {
         // Get the extension point registry.
         IExtensionPoint extensionPoint;
         extensionPoint = Platform.getExtensionRegistry().getExtensionPoint(
-                PI_WORKBENCH, IWorkbenchConstants.PL_ELEMENT_FACTORY);
+                PI_WORKBENCH, IWorkbenchRegistryConstants.PL_ELEMENT_FACTORY);
 
         if (extensionPoint == null) {
             WorkbenchPlugin
-                    .log("Unable to find element factory. Extension point: " + IWorkbenchConstants.PL_ELEMENT_FACTORY + " not found"); //$NON-NLS-2$ //$NON-NLS-1$
+                    .log("Unable to find element factory. Extension point: " + IWorkbenchRegistryConstants.PL_ELEMENT_FACTORY + " not found"); //$NON-NLS-2$ //$NON-NLS-1$
             return null;
         }
 
@@ -362,7 +365,7 @@ public class WorkbenchPlugin extends AbstractUIPlugin {
      */
     public AbstractPresentationFactory getPresentationFactory(String targetID) {
         Object o = createExtension(
-                IWorkbenchConstants.PL_PRESENTATION_FACTORIES,
+                IWorkbenchRegistryConstants.PL_PRESENTATION_FACTORIES,
                 "factory", targetID); //$NON-NLS-1$
         if (o instanceof AbstractPresentationFactory) {
             return (AbstractPresentationFactory) o;
@@ -519,8 +522,9 @@ public class WorkbenchPlugin extends AbstractUIPlugin {
      * @return the shared image manager
      */
     public ISharedImages getSharedImages() {
-        if (sharedImages == null)
-            sharedImages = new SharedImages();
+        if (sharedImages == null) {
+			sharedImages = new SharedImages();
+		}
         return sharedImages;
     }
 
@@ -584,40 +588,64 @@ public class WorkbenchPlugin extends AbstractUIPlugin {
      */
     public static void log(String message) {
         getDefault().getLog().log(
-                StatusUtil.newStatus(IStatus.ERROR, message, null));
-        System.err.println(message);
-        //1FTTJKV: ITPCORE:ALL - log(status) does not allow plugin information to be recorded
-    }
-
-    public static IStatusFactory getStatusFactory() {
-        return getDefault().statusFactory();
-    }
-    
-    public static IStatus getStatus(Throwable t) {
-        return getStatusFactory().newError(t);
-    }
-    
-    public static void log(Throwable t) {
-        getDefault().getLog().log(getStatus(t));
-    }
-    
-    public static IStatus getStatus(String message, Throwable t) {
-        return getStatusFactory().newError(message, t);
+                StatusUtil.newStatus(IStatus.ERROR, message, null));    
     }
     
     /**
-     * Logs the given message and throwable to the platform log.
-     * 
-     * If you have a status object in hand call log(String, IStatus) instead.
-     * 
-     * This convenience method is for internal use by the Workbench only and
-     * must not be called outside the Workbench.
-     * 
-     * @param message
-     *            A high level UI message describing when the problem happened.
+     * Log the throwable.
      * @param t
-     *            The throwable from where the problem actually occurred.
      */
+    public static void log(Throwable t) {
+		getDefault().getLog().log(getStatus(t));
+	}
+
+	/**
+	 * Return the status from throwable
+	 * @param t throwable
+	 * @return IStatus
+	 */
+	public static IStatus getStatus(Throwable t) {
+		String message = StatusUtil.getLocalizedMessage(t);
+
+		return newError(message, t);
+	}
+
+	/**
+	 * Create a new error from the message and the
+	 * throwable.
+	 * @param message
+	 * @param t
+	 * @return IStatus
+	 */
+	public static IStatus newError(String message, Throwable t) {
+		String pluginId = "org.eclipse.ui.workbench"; //$NON-NLS-1$
+		int errorCode = IStatus.OK;
+
+		// If this was a CoreException, keep the original plugin ID and error
+		// code
+		if (t instanceof CoreException) {
+			CoreException ce = (CoreException) t;
+			pluginId = ce.getStatus().getPlugin();
+			errorCode = ce.getStatus().getCode();
+		}
+
+		return new Status(IStatus.ERROR, pluginId, errorCode, message,
+				StatusUtil.getCause(t));
+	}
+    
+    /**
+	 * Logs the given message and throwable to the platform log.
+	 * 
+	 * If you have a status object in hand call log(String, IStatus) instead.
+	 * 
+	 * This convenience method is for internal use by the Workbench only and
+	 * must not be called outside the Workbench.
+	 * 
+	 * @param message
+	 *            A high level UI message describing when the problem happened.
+	 * @param t
+	 *            The throwable from where the problem actually occurred.
+	 */
     public static void log(String message, Throwable t) {
         IStatus status = StatusUtil.newStatus(IStatus.ERROR, message, t);
         log(message, status);
@@ -663,15 +691,15 @@ public class WorkbenchPlugin extends AbstractUIPlugin {
         if (message != null) {
             getDefault().getLog().log(
                     StatusUtil.newStatus(IStatus.ERROR, message, null));
-            System.err.println(message + "\nReason:"); //$NON-NLS-1$
         }
 
         getDefault().getLog().log(status);
-        System.err.println(status.getMessage());
-
-        //1FTTJKV: ITPCORE:ALL - log(status) does not allow plugin information to be recorded
     }
 
+    /**
+     * Log the status to the default log.
+     * @param status
+     */
     public static void log(IStatus status) {
         getDefault().getLog().log(status);
     }
@@ -700,11 +728,20 @@ public class WorkbenchPlugin extends AbstractUIPlugin {
 		
 		 Window.setDefaultOrientation(getDefaultOrientation());
 
-        // Start the UI plugin so that it can install the callback in PrefUtil,
+        // The UI plugin needs to be initialized so that it can install the callback in PrefUtil,
         // which needs to be done as early as possible, before the workbench
         // accesses any API preferences.
-        Bundle uiBundle = Platform.getBundle(PlatformUI.PLUGIN_ID); 
-        uiBundle.start();
+        Bundle uiBundle = Platform.getBundle(PlatformUI.PLUGIN_ID);
+        try {
+            // Attempt to load the activator of the ui bundle.  This will force lazy start
+            // of the ui bundle.  Using the bundle activator class here because it is a
+            // class that needs to be loaded anyway so it should not cause extra classes
+            // to be loaded.
+        	if(uiBundle != null)
+        		uiBundle.loadClass(UI_BUNDLE_ACTIVATOR);
+        } catch (ClassNotFoundException e) {
+            WorkbenchPlugin.log("Unable to load UI activator", e); //$NON-NLS-1$
+        }
 		/*
 		 * DO NOT RUN ANY OTHER CODE AFTER THIS LINE.  If you do, then you are
 		 * likely to cause a deadlock in class loader code.  Please see Bug 86450
@@ -727,13 +764,15 @@ public class WorkbenchPlugin extends AbstractUIPlugin {
 		
 		int orientation = getCommandLineOrientation(commandLineArgs);
 		
-		if(orientation != SWT.NONE)
+		if(orientation != SWT.NONE) {
 			return orientation;
+		}
 		
 		orientation = getSystemPropertyOrientation();
 		
-		if(orientation != SWT.NONE)
+		if(orientation != SWT.NONE) {
 			return orientation;
+		}
 
 		return checkCommandLineLocale(); //Use the default value if there is nothing specified
 	}
@@ -758,15 +797,17 @@ public class WorkbenchPlugin extends AbstractUIPlugin {
 		
 		//Check if the user property is set. If not do not
 		//rely on the vm.
-		if(System.getProperty(NL_USER_PROPERTY) == null)
+		if(System.getProperty(NL_USER_PROPERTY) == null) {
 			return SWT.NONE;
+		}
 		
 		Locale locale = Locale.getDefault();
 		String lang = locale.getLanguage();
 
-		if ("iw".equals(lang) || "ar".equals(lang) || "fa".equals(lang) //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
-			|| "ur".equals(lang)) //$NON-NLS-1$
+		if ("iw".equals(lang) || "he".equals(lang) || "ar".equals(lang) ||  //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
+				"fa".equals(lang) || "ur".equals(lang)) { //$NON-NLS-1$ //$NON-NLS-2$ 
 			return SWT.RIGHT_TO_LEFT;
+		}
 			
 		return SWT.NONE;
 	}
@@ -782,10 +823,12 @@ public class WorkbenchPlugin extends AbstractUIPlugin {
 	 */
 	private int getSystemPropertyOrientation() {
 		String orientation = System.getProperty(ORIENTATION_PROPERTY);
-		if(RIGHT_TO_LEFT.equals(orientation))
+		if(RIGHT_TO_LEFT.equals(orientation)) {
 			return SWT.RIGHT_TO_LEFT;
-		if(LEFT_TO_RIGHT.equals(orientation))
+		}
+		if(LEFT_TO_RIGHT.equals(orientation)) {
 			return SWT.LEFT_TO_RIGHT;
+		}
 		return SWT.NONE;
 	}
 
@@ -882,8 +925,9 @@ public class WorkbenchPlugin extends AbstractUIPlugin {
      * @return ProductInfo the product info for the receiver
      */
     private ProductInfo getProductInfo() {
-        if (productInfo == null)
-            productInfo = new ProductInfo(Platform.getProduct());
+        if (productInfo == null) {
+			productInfo = new ProductInfo(Platform.getProduct());
+		}
         return productInfo;
     }
 
@@ -928,14 +972,6 @@ public class WorkbenchPlugin extends AbstractUIPlugin {
     public IWizardRegistry getExportWizardRegistry() {
     	return ExportWizardRegistry.getInstance();
     }
-
-    IStatusFactory statusFactory() {
-        if (statusFactory == null) {
-            statusFactory = new StatusFactory(getBundle());
-        }
-        
-        return statusFactory;
-    }
     
     /**
      * FOR INTERNAL WORKBENCH USE ONLY. 
@@ -971,4 +1007,28 @@ public class WorkbenchPlugin extends AbstractUIPlugin {
 	/* package */ int getBundleCount() {
 		return bundleContext.getBundles().length;
 	}
+	
+	/* package */ OutputStream getSplashStream() {
+		// assumes the output stream is available as a service
+		// see EclipseStarter.publishSplashScreen
+		ServiceReference[] ref;
+		try {
+			ref = bundleContext.getServiceReferences(OutputStream.class.getName(), null);
+		} catch (InvalidSyntaxException e) {
+			return null;
+		}
+		if(ref==null) {
+			return null;
+		}
+		for (int i = 0; i < ref.length; i++) {
+			String name = (String) ref[i].getProperty("name"); //$NON-NLS-1$
+			if (name != null && name.equals("splashstream")) {  //$NON-NLS-1$
+				Object result = bundleContext.getService(ref[i]);
+				bundleContext.ungetService(ref[i]);
+				return (OutputStream) result;
+			}
+		}
+		return null;
+	}
+	
 }
