@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2004 IBM Corporation and others.
+ * Copyright (c) 2000, 2005 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -13,23 +13,32 @@ package org.eclipse.ui.views.markers.internal;
 
 import org.eclipse.jface.action.Action;
 import org.eclipse.ui.IWorkbenchPart;
+import org.eclipse.ui.internal.ide.IDEWorkbenchPlugin;
 
+/**
+ * The ActionAddGlobalTask is the action for adding a global task.
+ *
+ */
 public class ActionAddGlobalTask extends Action {
 
     private static final String ENABLED_IMAGE_PATH = "elcl16/addtsk_tsk.gif"; //$NON-NLS-1$
 
     private IWorkbenchPart part;
 
+    /**
+     * Create a new instance of the global task.
+     * @param part
+     */
     public ActionAddGlobalTask(IWorkbenchPart part) {
-        setText(Messages.getString("addGlobalTaskAction.title")); //$NON-NLS-1$
-        setImageDescriptor(ImageFactory.getImageDescriptor(ENABLED_IMAGE_PATH));
-        setToolTipText(Messages.getString("addGlobalTaskAction.tooltip")); //$NON-NLS-1$
+        setText(MarkerMessages.addGlobalTaskAction_title); 
+        setImageDescriptor(IDEWorkbenchPlugin.getIDEImageDescriptor(ENABLED_IMAGE_PATH));
+        setToolTipText(MarkerMessages.addGlobalTaskAction_tooltip);
         this.part = part;
     }
 
     public void run() {
         DialogTaskProperties dialog = new DialogTaskProperties(part.getSite()
-                .getShell(), Messages.getString("addGlobalTaskDialog.title")); //$NON-NLS-1$
+                .getShell(), MarkerMessages.addGlobalTaskDialog_title);
         dialog.open();
     }
 }
