@@ -18,8 +18,8 @@
 
 #ifdef NATIVE_STATS
 
-int OS_nativeFunctionCount = 1105;
-int OS_nativeFunctionCallCount[1105];
+int OS_nativeFunctionCount = 1118;
+int OS_nativeFunctionCallCount[1118];
 char * OS_nativeFunctionNames[] = {
 	"Call",
 	"GDK_1EVENT_1TYPE",
@@ -120,6 +120,7 @@ char * OS_nativeFunctionNames[] = {
 	"_1GTK_1IS_1CONTAINER",
 	"_1GTK_1IS_1IMAGE_1MENU_1ITEM",
 	"_1GTK_1IS_1PLUG",
+	"_1GTK_1IS_1WINDOW",
 	"_1GTK_1STOCK_1CANCEL",
 	"_1GTK_1STOCK_1OK",
 	"_1GTK_1TYPE_1CELL_1RENDERER_1PIXBUF",
@@ -271,6 +272,8 @@ char * OS_nativeFunctionNames[] = {
 	"_1gdk_1atom_1name",
 	"_1gdk_1beep",
 	"_1gdk_1bitmap_1create_1from_1data",
+	"_1gdk_1cairo_1region",
+	"_1gdk_1cairo_1set_1source_1color",
 	"_1gdk_1color_1white",
 	"_1gdk_1colormap_1alloc_1color",
 	"_1gdk_1colormap_1free_1colors",
@@ -371,6 +374,7 @@ char * OS_nativeFunctionNames[] = {
 	"_1gdk_1region_1union_1with_1rect",
 	"_1gdk_1rgb_1init",
 	"_1gdk_1screen_1get_1default",
+	"_1gdk_1screen_1get_1monitor_1at_1point",
 	"_1gdk_1screen_1get_1monitor_1at_1window",
 	"_1gdk_1screen_1get_1monitor_1geometry",
 	"_1gdk_1screen_1get_1n_1monitors",
@@ -590,6 +594,7 @@ char * OS_nativeFunctionNames[] = {
 	"_1gtk_1image_1set_1from_1pixbuf",
 	"_1gtk_1image_1set_1from_1pixmap",
 	"_1gtk_1init_1check",
+	"_1gtk_1label_1get_1layout",
 	"_1gtk_1label_1get_1mnemonic_1keyval",
 	"_1gtk_1label_1new",
 	"_1gtk_1label_1new_1with_1mnemonic",
@@ -955,6 +960,7 @@ char * OS_nativeFunctionNames[] = {
 	"_1gtk_1window_1get_1focus",
 	"_1gtk_1window_1get_1icon_1list",
 	"_1gtk_1window_1get_1mnemonic_1modifier",
+	"_1gtk_1window_1get_1modal",
 	"_1gtk_1window_1get_1position",
 	"_1gtk_1window_1get_1size",
 	"_1gtk_1window_1iconify",
@@ -986,6 +992,12 @@ char * OS_nativeFunctionNames[] = {
 	"_1pango_1attr_1strikethrough_1new",
 	"_1pango_1attr_1underline_1new",
 	"_1pango_1attr_1weight_1new",
+	"_1pango_1cairo_1create_1layout",
+	"_1pango_1cairo_1font_1map_1create_1context",
+	"_1pango_1cairo_1font_1map_1get_1default",
+	"_1pango_1cairo_1font_1map_1new",
+	"_1pango_1cairo_1layout_1path",
+	"_1pango_1cairo_1show_1layout",
 	"_1pango_1context_1get_1base_1dir",
 	"_1pango_1context_1get_1language",
 	"_1pango_1context_1get_1metrics",
@@ -1018,6 +1030,7 @@ char * OS_nativeFunctionNames[] = {
 	"_1pango_1layout_1context_1changed",
 	"_1pango_1layout_1get_1alignment",
 	"_1pango_1layout_1get_1attributes",
+	"_1pango_1layout_1get_1context",
 	"_1pango_1layout_1get_1indent",
 	"_1pango_1layout_1get_1iter",
 	"_1pango_1layout_1get_1justify",
