@@ -124,6 +124,7 @@ typedef enum {
 	_1GTK_1IS_1CONTAINER_FUNC,
 	_1GTK_1IS_1IMAGE_1MENU_1ITEM_FUNC,
 	_1GTK_1IS_1PLUG_FUNC,
+	_1GTK_1IS_1WINDOW_FUNC,
 	_1GTK_1STOCK_1CANCEL_FUNC,
 	_1GTK_1STOCK_1OK_FUNC,
 	_1GTK_1TYPE_1CELL_1RENDERER_1PIXBUF_FUNC,
@@ -275,6 +276,8 @@ typedef enum {
 	_1gdk_1atom_1name_FUNC,
 	_1gdk_1beep_FUNC,
 	_1gdk_1bitmap_1create_1from_1data_FUNC,
+	_1gdk_1cairo_1region_FUNC,
+	_1gdk_1cairo_1set_1source_1color_FUNC,
 	_1gdk_1color_1white_FUNC,
 	_1gdk_1colormap_1alloc_1color_FUNC,
 	_1gdk_1colormap_1free_1colors_FUNC,
@@ -375,6 +378,7 @@ typedef enum {
 	_1gdk_1region_1union_1with_1rect_FUNC,
 	_1gdk_1rgb_1init_FUNC,
 	_1gdk_1screen_1get_1default_FUNC,
+	_1gdk_1screen_1get_1monitor_1at_1point_FUNC,
 	_1gdk_1screen_1get_1monitor_1at_1window_FUNC,
 	_1gdk_1screen_1get_1monitor_1geometry_FUNC,
 	_1gdk_1screen_1get_1n_1monitors_FUNC,
@@ -594,6 +598,7 @@ typedef enum {
 	_1gtk_1image_1set_1from_1pixbuf_FUNC,
 	_1gtk_1image_1set_1from_1pixmap_FUNC,
 	_1gtk_1init_1check_FUNC,
+	_1gtk_1label_1get_1layout_FUNC,
 	_1gtk_1label_1get_1mnemonic_1keyval_FUNC,
 	_1gtk_1label_1new_FUNC,
 	_1gtk_1label_1new_1with_1mnemonic_FUNC,
@@ -959,6 +964,7 @@ typedef enum {
 	_1gtk_1window_1get_1focus_FUNC,
 	_1gtk_1window_1get_1icon_1list_FUNC,
 	_1gtk_1window_1get_1mnemonic_1modifier_FUNC,
+	_1gtk_1window_1get_1modal_FUNC,
 	_1gtk_1window_1get_1position_FUNC,
 	_1gtk_1window_1get_1size_FUNC,
 	_1gtk_1window_1iconify_FUNC,
@@ -990,6 +996,12 @@ typedef enum {
 	_1pango_1attr_1strikethrough_1new_FUNC,
 	_1pango_1attr_1underline_1new_FUNC,
 	_1pango_1attr_1weight_1new_FUNC,
+	_1pango_1cairo_1create_1layout_FUNC,
+	_1pango_1cairo_1font_1map_1create_1context_FUNC,
+	_1pango_1cairo_1font_1map_1get_1default_FUNC,
+	_1pango_1cairo_1font_1map_1new_FUNC,
+	_1pango_1cairo_1layout_1path_FUNC,
+	_1pango_1cairo_1show_1layout_FUNC,
 	_1pango_1context_1get_1base_1dir_FUNC,
 	_1pango_1context_1get_1language_FUNC,
 	_1pango_1context_1get_1metrics_FUNC,
@@ -1022,6 +1034,7 @@ typedef enum {
 	_1pango_1layout_1context_1changed_FUNC,
 	_1pango_1layout_1get_1alignment_FUNC,
 	_1pango_1layout_1get_1attributes_FUNC,
+	_1pango_1layout_1get_1context_FUNC,
 	_1pango_1layout_1get_1indent_FUNC,
 	_1pango_1layout_1get_1iter_FUNC,
 	_1pango_1layout_1get_1justify_FUNC,
